@@ -3,25 +3,21 @@
 
 defmodule HelloWorld do
 
-  import CloudILogger
+  require CloudILogger
 
   def cloudi_service_init(_args, _prefix, _timeout, dispatcher) do
     :cloudi_service.subscribe(dispatcher, 'hello_world_elixir/get')
     {:ok, :undefined}
   end
 
-  def cloudi_service_handle_request(_type, _name, _pattern,
-                                    _requestinfo, _request,
+  def cloudi_service_handle_request(_request_type, _name, _pattern,
+                                    _request_info, _request,
                                     _timeout, _priority,
                                     _transid, _pid, state, _dispatcher) do
+    CloudILogger.log_info('Hello Elixir World!', [])
     {:reply, "Hello World!", state}
   end
 
-  def cloudi_service_handle_info(request, state, _dispatcher) do
-    log_warn('Unknown info "~p"', [request])
-    {:noreply, state}
-  end
-
   def cloudi_service_terminate(_reason, _timeout, _state) do
     :ok
   end
