@@ -3,7 +3,7 @@
 //
 // MIT License
 //
-// Copyright (c) 2011-2022 Michael Truog <mjtruog at protonmail dot com>
+// Copyright (c) 2011-2023 Michael Truog <mjtruog at protonmail dot com>
 //
 // Permission is hereby granted, free of charge, to any person obtaining a
 // copy of this software and associated documentation files (the "Software"),
@@ -32,6 +32,7 @@
 #include "realloc_ptr.hpp"
 #include "copy_ptr.hpp"
 #include "timer.hpp"
+#include "bind.hpp"
 #include <unistd.h>
 #include <errno.h>
 #include <poll.h>
@@ -569,6 +570,19 @@ static int poll_request(cloudi_instance_t * api,
                         int timeout,
                         int external);
 
+static int getenv_to_uint32(char const * const name,
+                            uint32_t * const value)
+{
+    char const * const value_str = ::getenv(name);
+    if (value_str == 0)
+        return cloudi_invalid_input;
+    int const value_int = ::atoi(value_str);
+    if (value_int < 0)
+        return cloudi_invalid_input;
+    *value = static_cast<uint32_t>(value_int);
+    return cloudi_success;
+}
+
 int cloudi_initialize(cloudi_instance_t * api,
                       unsigned int const thread_index,
                       void * state)
@@ -582,12 +596,12 @@ int cloudi_initialize(cloudi_instance_t * api,
             "CloudI service execution must occur in CloudI" << std::endl;
         return cloudi_invalid_input;
     }
-    char const * const buffer_size_p = ::getenv("CLOUDI_API_INIT_BUFFER_SIZE");
-    if (buffer_size_p == 0)
-        return cloudi_invalid_input;
+    uint32_t buffer_size;
+    int result = getenv_to_uint32("CLOUDI_API_INIT_BUFFER_SIZE", &buffer_size);
+    if (result)
+        return result;
     ::memset(api, 0, sizeof(cloudi_instance_t));
     api->state = state;
-    uint32_t const buffer_size = ::atoi(buffer_size_p);
     if (::strcmp(protocol, "tcp") == 0)
     {
         api->fd = thread_index + 3;
@@ -646,8 +660,8 @@ int cloudi_initialize(cloudi_instance_t * api,
         return cloudi_error_ei_encode;
     if (ei_encode_atom(buffer.get<char>(), &index, "init"))
         return cloudi_error_ei_encode;
-    int result = write_exact(api->fd, api->use_header,
-                             buffer.get<char>(), index);
+    result = write_exact(api->fd, api->use_header,
+                         buffer.get<char>(), index);
     if (result)
         return result;
 
@@ -677,13 +691,11 @@ void * cloudi_destroy(cloudi_instance_t * api)
 
 int cloudi_initialize_thread_count(unsigned int * const thread_count)
 {
-    char const * const value_str = ::getenv("CLOUDI_API_INIT_THREAD_COUNT");
-    if (value_str == 0)
-        return cloudi_invalid_input;
-    int const value = ::atoi(value_str);
-    if (value < 0)
-        return cloudi_invalid_input;
-    *thread_count = static_cast<unsigned int>(value);
+    uint32_t value = 0;
+    int result = getenv_to_uint32("CLOUDI_API_INIT_THREAD_COUNT", &value);
+    if (result)
+        return result;
+    *thread_count = value;
     return cloudi_success;
 }
 
@@ -1568,6 +1580,14 @@ static void store_incoming_uint32(buffer_t const & buffer,
     index += sizeof(uint32_t);
 }
 
+static void store_incoming_int32(buffer_t const & buffer,
+                                 uint32_t & index,
+                                 int32_t & i)
+{
+    i = *reinterpret_cast<int32_t *>(&buffer[index]);
+    index += sizeof(int32_t);
+}
+
 static void store_incoming_int8(buffer_t const & buffer,
                                 uint32_t & index,
                                 int8_t & i)
@@ -1579,6 +1599,7 @@ static void store_incoming_int8(buffer_t const & buffer,
 static uint8_t get_incoming_uint8(buffer_t const & buffer,
                                   uint32_t & index)
 {
+    // return value required for safe bit field use
     uint8_t const i = *reinterpret_cast<uint8_t *>(&buffer[index]);
     index += sizeof(uint8_t);
     return i;
@@ -1666,6 +1687,8 @@ static int poll_request(cloudi_instance_t * api,
         if (result)
             return result;
         api->initialization_complete = 1;
+        if (bind_set(api->bind))
+            return cloudi_invalid_input;
     }
 
     buffer_t & buffer_recv = *reinterpret_cast<buffer_t *>(api->buffer_recv);
@@ -1702,16 +1725,20 @@ static int poll_request(cloudi_instance_t * api,
             {
                 store_incoming_uint32(buffer_recv, index, api->process_index);
                 store_incoming_uint32(buffer_recv, index, api->process_count);
-                store_incoming_uint32(buffer_recv, index, api->process_count_max);
-                store_incoming_uint32(buffer_recv, index, api->process_count_min);
+                store_incoming_uint32(buffer_recv, index,
+                                      api->process_count_max);
+                store_incoming_uint32(buffer_recv, index,
+                                      api->process_count_min);
                 store_incoming_binary(buffer_recv, index, api->prefix);
                 store_incoming_uint32(buffer_recv, index,
                                       api->timeout_initialize);
                 store_incoming_uint32(buffer_recv, index, api->timeout_async);
                 store_incoming_uint32(buffer_recv, index, api->timeout_sync);
-                store_incoming_uint32(buffer_recv, index, api->timeout_terminate);
+                store_incoming_uint32(buffer_recv, index,
+                                      api->timeout_terminate);
                 store_incoming_int8(buffer_recv, index, api->priority_default);
                 api->fatal_exceptions = get_incoming_uint8(buffer_recv, index);
+                store_incoming_int32(buffer_recv, index, api->bind);
                 if (index != api->buffer_recv_index)
                 {
                     assert(! external);
@@ -2070,6 +2097,32 @@ void cloudi_free_response(cloudi_instance_t * api)
     api->free_response = 1;
 }
 
+int cloudi_initialize_process_index(uint32_t * const process_index)
+{
+    return getenv_to_uint32("CLOUDI_API_INIT_PROCESS_INDEX",
+                            process_index);
+}
+int cloudi_initialize_process_count_max(uint32_t * const process_count_max)
+{
+    return getenv_to_uint32("CLOUDI_API_INIT_PROCESS_COUNT_MAX",
+                            process_count_max);
+}
+int cloudi_initialize_process_count_min(uint32_t * const process_count_min)
+{
+    return getenv_to_uint32("CLOUDI_API_INIT_PROCESS_COUNT_MIN",
+                            process_count_min);
+}
+int cloudi_initialize_timeout_initialize(uint32_t * const timeout_initialize)
+{
+    return getenv_to_uint32("CLOUDI_API_INIT_TIMEOUT_INITIALIZE",
+                            timeout_initialize);
+}
+int cloudi_initialize_timeout_terminate(uint32_t * const timeout_terminate)
+{
+    return getenv_to_uint32("CLOUDI_API_INIT_TIMEOUT_TERMINATE",
+                            timeout_terminate);
+}
+
 } // extern C
 
 // C++ API
@@ -2523,6 +2576,16 @@ uint32_t API::process_index() const
     return m_impl.api()->process_index;
 }
 
+uint32_t API::process_index_()
+{
+    uint32_t process_index;
+    int const result =
+        cloudi_initialize_process_index(&process_index);
+    if (result != return_value::success)
+        throw invalid_input_exception(result);
+    return process_index;
+}
+
 uint32_t API::process_count() const
 {
     return m_impl.api()->process_count;
@@ -2533,11 +2596,31 @@ uint32_t API::process_count_max() const
     return m_impl.api()->process_count_max;
 }
 
+uint32_t API::process_count_max_()
+{
+    uint32_t process_count_max;
+    int const result =
+        cloudi_initialize_process_count_max(&process_count_max);
+    if (result != return_value::success)
+        throw invalid_input_exception(result);
+    return process_count_max;
+}
+
 uint32_t API::process_count_min() const
 {
     return m_impl.api()->process_count_min;
 }
 
+uint32_t API::process_count_min_()
+{
+    uint32_t process_count_min;
+    int const result =
+        cloudi_initialize_process_count_min(&process_count_min);
+    if (result != return_value::success)
+        throw invalid_input_exception(result);
+    return process_count_min;
+}
+
 char const * API::prefix() const
 {
     return m_impl.api()->prefix;
@@ -2548,6 +2631,16 @@ uint32_t API::timeout_initialize() const
     return m_impl.api()->timeout_initialize;
 }
 
+uint32_t API::timeout_initialize_()
+{
+    uint32_t timeout_initialize;
+    int const result =
+        cloudi_initialize_timeout_initialize(&timeout_initialize);
+    if (result != return_value::success)
+        throw invalid_input_exception(result);
+    return timeout_initialize;
+}
+
 uint32_t API::timeout_async() const
 {
     return m_impl.api()->timeout_async;
@@ -2563,6 +2656,16 @@ uint32_t API::timeout_terminate() const
     return m_impl.api()->timeout_terminate;
 }
 
+uint32_t API::timeout_terminate_()
+{
+    uint32_t timeout_terminate;
+    int const result =
+        cloudi_initialize_timeout_terminate(&timeout_terminate);
+    if (result != return_value::success)
+        throw invalid_input_exception(result);
+    return timeout_terminate;
+}
+
 int8_t API::priority_default() const
 {
     return m_impl.api()->priority_default;
