@@ -5,7 +5,7 @@ package cloudi
 //
 // MIT License
 //
-// Copyright (c) 2017-2022 Michael Truog <mjtruog at protonmail dot com>
+// Copyright (c) 2017-2023 Michael Truog <mjtruog at protonmail dot com>
 //
 // Permission is hereby granted, free of charge, to any person obtaining a
 // copy of this software and associated documentation files (the "Software"),
@@ -111,7 +111,7 @@ func API(threadIndex uint32, state interface{}) (*Instance, error) {
 		os.Stderr.WriteString("CloudI service execution must occur in CloudI\n")
 		return nil, invalidInputErrorNew()
 	}
-	bufferSize, err := uintGetenv("CLOUDI_API_INIT_BUFFER_SIZE")
+	bufferSize, err := getenvToUint32("CLOUDI_API_INIT_BUFFER_SIZE")
 	if err != nil {
 		return nil, err
 	}
@@ -163,7 +163,7 @@ func API(threadIndex uint32, state interface{}) (*Instance, error) {
 
 // ThreadCount returns the thread count from the service configuration
 func ThreadCount() (uint32, error) {
-	return uintGetenv("CLOUDI_API_INIT_THREAD_COUNT")
+	return getenvToUint32("CLOUDI_API_INIT_THREAD_COUNT")
 }
 
 // Subscribe subscribes to a service name pattern with a callback
@@ -664,6 +664,11 @@ func (api *Instance) ProcessIndex() uint32 {
 	return api.processIndex
 }
 
+// ProcessIndex_ returns the 0-based index of this process in the service instance
+func ProcessIndex_() (uint32, error) {
+	return getenvToUint32("CLOUDI_API_INIT_PROCESS_INDEX")
+}
+
 // ProcessCount returns the current process count based on the service configuration
 func (api *Instance) ProcessCount() uint32 {
 	return api.processCount
@@ -674,11 +679,21 @@ func (api *Instance) ProcessCountMax() uint32 {
 	return api.processCountMax
 }
 
+// ProcessCountMax_ returns the count_process_dynamic maximum count based on the service configuration
+func ProcessCountMax_() (uint32, error) {
+	return getenvToUint32("CLOUDI_API_INIT_PROCESS_COUNT_MAX")
+}
+
 // ProcessCountMin returns the count_process_dynamic minimum count based on the service configuration
 func (api *Instance) ProcessCountMin() uint32 {
 	return api.processCountMin
 }
 
+// ProcessCountMin_ returns the count_process_dynamic minimum count based on the service configuration
+func ProcessCountMin_() (uint32, error) {
+	return getenvToUint32("CLOUDI_API_INIT_PROCESS_COUNT_MIN")
+}
+
 // Prefix returns the service name pattern prefix from the service configuration
 func (api *Instance) Prefix() string {
 	return api.prefix
@@ -689,6 +704,11 @@ func (api *Instance) TimeoutInitialize() uint32 {
 	return api.timeoutInitialize
 }
 
+// TimeoutInitialize_ returns the service initialization timeout from the service configuration
+func TimeoutInitialize_() (uint32, error) {
+	return getenvToUint32("CLOUDI_API_INIT_TIMEOUT_INITIALIZE")
+}
+
 // TimeoutAsync returns the default asynchronous service request send timeout from the service configuration
 func (api *Instance) TimeoutAsync() uint32 {
 	return api.timeoutAsync
@@ -704,6 +724,11 @@ func (api *Instance) TimeoutTerminate() uint32 {
 	return api.timeoutTerminate
 }
 
+// TimeoutTerminate_ returns the service termination timeout based on the service configuration
+func TimeoutTerminate_() (uint32, error) {
+	return getenvToUint32("CLOUDI_API_INIT_TIMEOUT_TERMINATE")
+}
+
 // PriorityDefault returns the default service request send priority from the service configuration
 func (api *Instance) PriorityDefault() int8 {
 	return api.priorityDefault
@@ -991,6 +1016,14 @@ func (api *Instance) pollRequest(timeout int32, external bool) (bool, error) {
 			if err != nil {
 				return false, err
 			}
+			var bind int32
+			err = binary.Read(reader, nativeEndian, &bind)
+			if err != nil {
+				return false, err
+			}
+			if bind >= 0 {
+				return false, invalidInputErrorNew()
+			}
 			if reader.Len() > 0 {
 				_, err = api.handleEvents(external, reader, 0)
 				if err != nil {
@@ -1461,7 +1494,7 @@ func (api *Instance) recvFragment(size int) (int, error) {
 	return i, nil
 }
 
-func uintGetenv(key string) (uint32, error) {
+func getenvToUint32(key string) (uint32, error) {
 	s := os.Getenv(key)
 	if s == "" {
 		return 0, invalidInputErrorNew()
