@@ -5,7 +5,7 @@
 
   MIT License
 
-  Copyright (c) 2017-2022 Michael Truog <mjtruog at protonmail dot com>
+  Copyright (c) 2017-2023 Michael Truog <mjtruog at protonmail dot com>
 
   Permission is hereby granted, free of charge, to any person obtaining a
   copy of this software and associated documentation files (the "Software"),
@@ -60,14 +60,19 @@ module Foreign.CloudI
     , returnSync
     , recvAsync
     , processIndex
+    , processIndex_
     , processCount
     , processCountMax
+    , processCountMax_
     , processCountMin
+    , processCountMin_
     , prefix
     , timeoutInitialize
+    , timeoutInitialize_
     , timeoutAsync
     , timeoutSync
     , timeoutTerminate
+    , timeoutTerminate_
     , priorityDefault
     , poll
     , shutdown
@@ -241,13 +246,7 @@ api threadIndex state terminateReturnValueOpt = do
 
 -- | returns the thread count from the service configuration
 threadCount :: IO (Result Int)
-threadCount = do
-    threadCountValue <- POSIX.getEnv "CLOUDI_API_INIT_THREAD_COUNT"
-    case threadCountValue of
-        Nothing ->
-            return $ Left invalidInputError
-        Just threadCountStr ->
-            return $ Right (read threadCountStr :: Int)
+threadCount = getEnvToUnsignedInt "CLOUDI_API_INIT_THREAD_COUNT"
 
 -- | subscribes to a service name pattern with a callback
 subscribe :: Instance.T s -> ByteString -> Instance.Callback s ->
@@ -609,6 +608,10 @@ processIndex :: Instance.T s -> Int
 processIndex Instance.T{Instance.processIndex = processIndex'} =
     processIndex'
 
+-- | returns the 0-based index of this process in the service instance
+processIndex_ :: IO (Result Int)
+processIndex_ = getEnvToUnsignedInt "CLOUDI_API_INIT_PROCESS_INDEX"
+
 -- | returns the current process count based on the service configuration
 processCount :: Instance.T s -> Int
 processCount Instance.T{Instance.processCount = processCount'} =
@@ -620,12 +623,22 @@ processCountMax :: Instance.T s -> Int
 processCountMax Instance.T{Instance.processCountMax = processCountMax'} =
     processCountMax'
 
+-- | returns the count_process_dynamic maximum count
+-- based on the service configuration
+processCountMax_ :: IO (Result Int)
+processCountMax_ = getEnvToUnsignedInt "CLOUDI_API_INIT_PROCESS_COUNT_MAX"
+
 -- | returns the count_process_dynamic minimum count
 -- based on the service configuration
 processCountMin :: Instance.T s -> Int
 processCountMin Instance.T{Instance.processCountMin = processCountMin'} =
     processCountMin'
 
+-- | returns the count_process_dynamic minimum count
+-- based on the service configuration
+processCountMin_ :: IO (Result Int)
+processCountMin_ = getEnvToUnsignedInt "CLOUDI_API_INIT_PROCESS_COUNT_MIN"
+
 -- | returns the service name pattern prefix from the service configuration
 prefix :: Instance.T s -> ByteString
 prefix Instance.T{Instance.prefix = prefix'} =
@@ -637,6 +650,11 @@ timeoutInitialize :: Instance.T s -> Int
 timeoutInitialize Instance.T{Instance.timeoutInitialize = timeoutInitialize'} =
     timeoutInitialize'
 
+-- | returns the service initialization timeout
+-- from the service configuration
+timeoutInitialize_ :: IO (Result Int)
+timeoutInitialize_ = getEnvToUnsignedInt "CLOUDI_API_INIT_TIMEOUT_INITIALIZE"
+
 -- | returns the default asynchronous service request send timeout
 -- from the service configuration
 timeoutAsync :: Instance.T s -> Int
@@ -655,6 +673,11 @@ timeoutTerminate :: Instance.T s -> Int
 timeoutTerminate Instance.T{Instance.timeoutTerminate = timeoutTerminate'} =
     timeoutTerminate'
 
+-- | returns the service termination timeout
+-- based on the service configuration
+timeoutTerminate_ :: IO (Result Int)
+timeoutTerminate_ = getEnvToUnsignedInt "CLOUDI_API_INIT_TIMEOUT_TERMINATE"
+
 -- | returns the default service request send priority
 -- from the service configuration
 priorityDefault :: Instance.T s -> Int
@@ -863,6 +886,7 @@ pollRequestDataGet messages api0 external = do
             timeoutTerminate' <- Get.getWord32host
             priorityDefault' <- Get.getInt8
             fatalExceptions' <- Get.getInt8
+            bind <- Get.getInt32host
             let api1 = Instance.init api0
                     processIndex'
                     processCount'
@@ -876,7 +900,9 @@ pollRequestDataGet messages api0 external = do
                     priorityDefault'
                     fatalExceptions'
             empty <- Get.isEmpty
-            if not empty then
+            if bind >= 0 then
+                fail invalidInputError
+            else if not empty then
                 handleEvents messages api1 external 0
             else
                 return (messages, api1)
@@ -1231,6 +1257,19 @@ setTerminate api0 =
     api0{ Instance.terminate = True
         , Instance.timeout = Just False}
 
+getEnvToUnsignedInt :: String -> IO (Result Int)
+getEnvToUnsignedInt name = do
+    valueResult <- POSIX.getEnv name
+    case valueResult of
+        Nothing ->
+            return $ Left invalidInputError
+        Just valueStr ->
+            let value = read valueStr :: Int in
+            if value < 0 then
+                return $ Left invalidInputError
+            else
+                return $ Right value
+
 threadList :: Concurrent.MVar [Concurrent.MVar ()]
 threadList = Unsafe.unsafePerformIO (Concurrent.newMVar [])
 
