@@ -5,7 +5,7 @@
 
   MIT License
 
-  Copyright (c) 2017 Michael Truog <mjtruog at protonmail dot com>
+  Copyright (c) 2017-2019 Michael Truog <mjtruog at protonmail dot com>
 
   Permission is hereby granted, free of charge, to any person obtaining a
   copy of this software and associated documentation files (the "Software"),
@@ -41,6 +41,6 @@ data Port = Port
     { nodeTag :: !Word8
     , node :: !ByteString
     , id :: !ByteString
-    , creation :: !Word8
+    , creation :: !ByteString
     } deriving (Ord, Eq, Show)
 
