@@ -3,7 +3,7 @@
 //
 // MIT License
 //
-// Copyright (c) 2014-2022 Michael Truog <mjtruog at protonmail dot com>
+// Copyright (c) 2014-2023 Michael Truog <mjtruog at protonmail dot com>
 //
 // Permission is hereby granted, free of charge, to any person obtaining a
 // copy of this software and associated documentation files (the "Software"),
@@ -80,6 +80,13 @@ var packUint32big = function packUint32big (value) {
                            (value >>> 8) & 0xff,
                            value & 0xff]);
 };
+var unpackInt32 = function unpackInt32 (i, data) {
+    var value = unpackUint32(i, data);
+    if ((0x80000000 & value) != 0) {
+        value = -2147483648 + (value & 0x7fffffff);
+    }
+    return value;
+};
 if (Erlang.nodejs_version_after('10.0.0',true)) {
     var originalEmitWarning = process.emitWarning;
     process.emitWarning = function(warning, type, code, ctor) {
@@ -102,6 +109,17 @@ if (Erlang.nodejs_version_after('10.0.0',true)) {
         originalEmitWarning(warning, type, code, ctor);
     };
 }
+var getEnvToUnsignedInt = function getEnvToUnsignedInt (name) {
+    var value_str = process.env[name];
+    if (value_str === undefined) {
+        throw new InvalidInputException();
+    }
+    var value = parseInt(value_str);
+    if (value < 0) {
+        throw new InvalidInputException();
+    }
+    return value;
+};
 
 var InvalidInputException = function InvalidInputException () {
     var error = new Error('Invalid Input');
@@ -229,10 +247,7 @@ CloudI.API = function API (thread_index, callback) {
         CloudI.stderr_write('CloudI service execution must occur in CloudI\n');
         throw new InvalidInputException();
     }
-    var buffer_size_str = process.env.CLOUDI_API_INIT_BUFFER_SIZE;
-    if (buffer_size_str === undefined) {
-        throw new InvalidInputException();
-    }
+    var buffer_size = getEnvToUnsignedInt('CLOUDI_API_INIT_BUFFER_SIZE');
     if (protocol_str == 'tcp') {
         API._use_header = true;
     }
@@ -252,7 +267,7 @@ CloudI.API = function API (thread_index, callback) {
     API._fatal_exceptions = false;
     API._terminate = false;
     API._terminate_callback = undefined;
-    API._size = parseInt(buffer_size_str);
+    API._size = buffer_size;
     API._callbacks = {};
     API._timeout_terminate = 10; // TIMEOUT_TERMINATE_MIN
     API._poll_callback = callback;
@@ -312,11 +327,7 @@ CloudI.API = function API (thread_index, callback) {
 
 // class method
 CloudI.API.thread_count = function thread_count () {
-    var count_str = process.env.CLOUDI_API_INIT_THREAD_COUNT;
-    if (count_str === undefined) {
-        throw new InvalidInputException();
-    }
-    return parseInt(count_str);
+    return getEnvToUnsignedInt('CLOUDI_API_INIT_THREAD_COUNT');
 };
 
 CloudI.API.prototype.subscribe = function (pattern, obj, obj_f, callback) {
@@ -531,6 +542,11 @@ CloudI.API.prototype.process_index = function () {
     return this._process_index;
 };
 
+// class method
+CloudI.API.process_index_ = function process_index_ () {
+    return getEnvToUnsignedInt('CLOUDI_API_INIT_PROCESS_INDEX');
+};
+
 CloudI.API.prototype.process_count = function () {
     return this._process_count;
 };
@@ -539,10 +555,20 @@ CloudI.API.prototype.process_count_max = function () {
     return this._process_count_max;
 };
 
+// class method
+CloudI.API.process_count_max_ = function process_count_max_ () {
+    return getEnvToUnsignedInt('CLOUDI_API_INIT_PROCESS_COUNT_MAX');
+};
+
 CloudI.API.prototype.process_count_min = function () {
     return this._process_count_min;
 };
 
+// class method
+CloudI.API.process_count_min_ = function process_count_min_ () {
+    return getEnvToUnsignedInt('CLOUDI_API_INIT_PROCESS_COUNT_MIN');
+};
+
 CloudI.API.prototype.prefix = function () {
     return this._prefix;
 };
@@ -551,6 +577,11 @@ CloudI.API.prototype.timeout_initialize = function () {
     return this._timeout_initialize;
 };
 
+// class method
+CloudI.API.timeout_initialize_ = function timeout_initialize_ () {
+    return getEnvToUnsignedInt('CLOUDI_API_INIT_TIMEOUT_INITIALIZE');
+};
+
 CloudI.API.prototype.timeout_async = function () {
     return this._timeout_async;
 };
@@ -563,6 +594,11 @@ CloudI.API.prototype.timeout_terminate = function () {
     return this._timeout_terminate;
 };
 
+// class method
+CloudI.API.timeout_terminate_ = function timeout_terminate_ () {
+    return getEnvToUnsignedInt('CLOUDI_API_INIT_TIMEOUT_TERMINATE');
+};
+
 CloudI.API.prototype.priority_default = function () {
     return this._priority_default;
 };
@@ -856,6 +892,11 @@ CloudI.API.prototype._poll_request = function (data) {
                 i += 1;
                 API._fatal_exceptions = unpackUint8(i, data) != 0;
                 i += 1;
+                var bind = unpackInt32(i, data);
+                i += 4;
+                if (bind >= 0) {
+                    throw new InvalidInputException();
+                }
                 if (i != data_size) {
                     API._handle_events(data, data_size, i);
                 }
