@@ -5,7 +5,7 @@
 
   MIT License
 
-  Copyright (c) 2017-2022 Michael Truog <mjtruog at protonmail dot com>
+  Copyright (c) 2017-2023 Michael Truog <mjtruog at protonmail dot com>
 
   Permission is hereby granted, free of charge, to any person obtaining a
   copy of this software and associated documentation files (the "Software"),
@@ -228,13 +228,18 @@ let backtrace (e : exn) : string =
 let null_response _ _ _ _ _ _ _ _ _ _ _ =
   Null
 
-let getenv (key : string) : string =
-  try Sys.getenv key
+let getenv (name : string) : string =
+  try Sys.getenv name
   with Not_found -> ""
 
-let getenv_to_uint (key : string) : (int, string) result =
-  try Ok (int_of_string (Sys.getenv key))
-  with _ -> Error (invalid_input_error)
+let getenv_to_uint (name : string) : (int, string) result =
+  let value = try int_of_string (Sys.getenv name)
+    with _ -> -1
+  in
+  if value < 0 then
+    Error (invalid_input_error)
+  else
+    Ok (value)
 
 let fd_of_int (fd: int) : Unix.file_descr = Obj.magic fd
 
@@ -255,6 +260,28 @@ let unpack_uint32_native i binary : (int, string) result =
       )
     )
 
+let unpack_int32_native i binary : (int, string) result =
+  let byte0 = int_of_char binary.[i + (if Sys.big_endian then 0 else 3)]
+  and byte1 = int_of_char binary.[i + (if Sys.big_endian then 1 else 2)]
+  and byte2 = int_of_char binary.[i + (if Sys.big_endian then 2 else 1)]
+  and byte3 = int_of_char binary.[i + (if Sys.big_endian then 3 else 0)] in
+  let byte0u = 0x7f land byte0 in
+  let byte0s = if (byte0 lsr 7) = 1 then
+    -128 + byte0u
+  else
+    byte0u in
+  if byte0u > max_int lsr 24 then
+    (* 32 bit system *)
+    Error ("ocaml int overflow")
+  else
+    Ok (
+      (byte0s lsl 24) lor (
+        (byte1 lsl 16) lor (
+          (byte2 lsl 8) lor byte3
+        )
+      )
+    )
+
 let unpack_uint32_big i binary : (int, string) result =
   let byte0 = int_of_char binary.[i]
   and byte1 = int_of_char binary.[i + 1]
@@ -805,28 +832,36 @@ and poll_request_data api ext data data_size i : (bool option, string) result =
                         let priority_default =
                           unpack_int8 (i1 + 16) data
                         and fatal_exceptions =
-                          (unpack_uint8 (i1 + 17) data) != 0
-                        and i2 = i1 + 18 in
-                        Instance.init api
-                          process_index
-                          process_count
-                          process_count_max
-                          process_count_min
-                          prefix
-                          timeout_initialize
-                          timeout_async
-                          timeout_sync
-                          timeout_terminate
-                          priority_default
-                          fatal_exceptions ;
-                        if i2 <> data_size then
-                          match handle_events api ext data data_size i2 0 with
-                          | Error (error) ->
-                            Error (error)
-                          | Ok _ ->
-                            Ok (Some false)
-                        else
-                          Ok (Some false)
+                          (unpack_uint8 (i1 + 17) data) != 0 in
+                        match unpack_int32_native (i1 + 18) data with
+                        | Error (error) ->
+                          Error (error)
+                        | Ok (bind) ->
+                          if bind >= 0 then
+                            Error (invalid_input_error)
+                          else
+                            let i2 = i1 + 22 in
+                            Instance.init api
+                              process_index
+                              process_count
+                              process_count_max
+                              process_count_min
+                              prefix
+                              timeout_initialize
+                              timeout_async
+                              timeout_sync
+                              timeout_terminate
+                              priority_default
+                              fatal_exceptions ;
+                            if i2 <> data_size then
+                              match handle_events
+                                api ext data data_size i2 0 with
+                              | Error (error) ->
+                                Error (error)
+                              | Ok _ ->
+                                Ok (Some false)
+                            else
+                              Ok (Some false)
     else if cmd = message_send_async || cmd = message_send_sync then
       match unpack_uint32_native (i + 4) data with
       | Error (error) ->
@@ -1342,21 +1377,33 @@ let recv_async
 let process_index (api : 's Instance.t) : int =
   api.Instance.process_index
 
+let process_index_ () : (int, string) result =
+  getenv_to_uint "CLOUDI_API_INIT_PROCESS_INDEX"
+
 let process_count (api : 's Instance.t) : int =
   api.Instance.process_count
 
 let process_count_max (api : 's Instance.t) : int =
   api.Instance.process_count_max
 
+let process_count_max_ () : (int, string) result =
+  getenv_to_uint "CLOUDI_API_INIT_PROCESS_COUNT_MAX"
+
 let process_count_min (api : 's Instance.t) : int =
   api.Instance.process_count_min
 
+let process_count_min_ () : (int, string) result =
+  getenv_to_uint "CLOUDI_API_INIT_PROCESS_COUNT_MIN"
+
 let prefix (api : 's Instance.t) : string =
   api.Instance.prefix
 
 let timeout_initialize (api : 's Instance.t) : int =
   api.Instance.timeout_initialize
 
+let timeout_initialize_ () : (int, string) result =
+  getenv_to_uint "CLOUDI_API_INIT_TIMEOUT_INITIALIZE"
+
 let timeout_async (api : 's Instance.t) : int =
   api.Instance.timeout_async
 
@@ -1366,6 +1413,9 @@ let timeout_sync (api : 's Instance.t) : int =
 let timeout_terminate (api : 's Instance.t) : int =
   api.Instance.timeout_terminate
 
+let timeout_terminate_ () : (int, string) result =
+  getenv_to_uint "CLOUDI_API_INIT_TIMEOUT_TERMINATE"
+
 let priority_default (api : 's Instance.t) : int =
   api.Instance.priority_default
 
