@@ -5,7 +5,7 @@
 
   MIT License
 
-  Copyright (c) 2017-2022 Michael Truog <mjtruog at protonmail dot com>
+  Copyright (c) 2017-2023 Michael Truog <mjtruog at protonmail dot com>
 
   Permission is hereby granted, free of charge, to any person obtaining a
   copy of this software and associated documentation files (the "Software"),
@@ -59,6 +59,8 @@ let tag_map_ext = 116
 let tag_fun_ext = 117
 let tag_atom_utf8_ext = 118
 let tag_small_atom_utf8_ext = 119
+let tag_v4_port_ext = 120
+let tag_local_ext = 121
 
 let buffer_size = 65536
 
@@ -103,7 +105,7 @@ module Function = struct
 end
 type t =
     OtpErlangInteger of int
-  | OtpErlangIntegerBig of Big_int.big_int
+  | OtpErlangIntegerBig of Big_int_Z.big_int
   | OtpErlangFloat of float
   | OtpErlangAtom of string
   | OtpErlangAtomUTF8 of string
@@ -184,7 +186,7 @@ let unpack_integer i binary : t =
     )
   in
   if byte0 > min_int lsr 24 then
-    OtpErlangIntegerBig (Big_int.big_int_of_int32 value)
+    OtpErlangIntegerBig (Big_int_Z.big_int_of_int32 value)
   else
     OtpErlangInteger (Int32.to_int value)
 
@@ -295,22 +297,26 @@ let rec binary_to_term_ i binary : (int, t, string) result2 =
     let float_data = try (String.sub data 0 (String.index data '\000'))
     with Not_found -> data in
     Ok2 (i0 + 31, OtpErlangFloat (float_of_string float_data))
-  else if tag = tag_new_port_ext ||
+  else if tag = tag_v4_port_ext || tag = tag_new_port_ext ||
           tag = tag_reference_ext || tag = tag_port_ext then
     match binary_to_atom i0 binary with
     | Error3 (error) ->
       Error2 (error)
     | Ok3(i1, node_tag, node) ->
-      let id = String.sub binary i1 4
-      and i2 = i1 + 4
-      and creation_size = if tag = tag_new_port_ext then 4 else 1 in
+      let id_size = if tag = tag_v4_port_ext then 8 else 4 in
+      let id = String.sub binary i1 id_size
+      and i2 = i1 + id_size
+      and creation_size = if tag = tag_v4_port_ext ||
+                             tag = tag_new_port_ext then 4 else 1 in
       let creation = String.sub binary i2 creation_size
       and i3 = i2 + creation_size in
       if tag = tag_reference_ext then
         Ok2 (i3, OtpErlangReference (
           Reference.make
             ~node_tag:node_tag ~node:node ~id:id ~creation:creation))
-      else (* tag = tag_new_port_ext || tag = tag_port_ext *)
+      else
+        (* tag = tag_v4_port_ext || tag = tag_new_port_ext ||
+           tag = tag_port_ext *)
         Ok2 (i3, OtpErlangPort (
           Port.make
             ~node_tag:node_tag ~node:node ~id:id ~creation:creation))
@@ -400,13 +406,13 @@ let rec binary_to_term_ i binary : (int, t, string) result2 =
         else
           loop
             (bignum_index + 1)
-            (Big_int.add_int_big_int
+            (Big_int_Z.add_int_big_int
               (int_of_char binary.[i1 + j - bignum_index])
-              (Big_int.mult_int_big_int 256 bignum_value))
+              (Big_int_Z.mult_int_big_int 256 bignum_value))
       in
-      let bignum = loop 0 Big_int.zero_big_int in
+      let bignum = loop 0 Big_int_Z.zero_big_int in
       let bignum_result = if sign = 1 then
-        Big_int.minus_big_int bignum
+        Big_int_Z.minus_big_int bignum
       else
         bignum
       and i2 = i1 + 1 in
@@ -529,6 +535,8 @@ let rec binary_to_term_ i binary : (int, t, string) result2 =
       Ok2 (i2, OtpErlangAtom (atom_name))
   else if tag = tag_compressed_zlib then
     Error2 (parse_error "ocaml doesn't provide zlib")
+  else if tag = tag_local_ext then
+    Error2 (parse_error "LOCAL_EXT is opaque")
   else
     Error2 (parse_error "invalid tag")
 
@@ -704,6 +712,7 @@ and tuple_to_binary value buffer =
     Error (output_error "uint32 overflow")
 
 and hashtbl_to_binary value buffer =
+  Buffer.add_char buffer (char_of_int tag_map_ext) ;
   let length = Hashtbl.length value in
   if valid_uint32_positive length then (
     pack_uint32 length buffer ;
@@ -765,24 +774,24 @@ and integer_to_binary value buffer =
     pack_uint32 value buffer ;
     Ok (buffer))
   else
-    bignum_to_binary (Big_int.big_int_of_int value) buffer
+    bignum_to_binary (Big_int_Z.big_int_of_int value) buffer
 
 and bignum_to_binary value buffer =
-  let bits8 = Big_int.big_int_of_int 0xff
-  and sign = if (Big_int.sign_big_int value) = -1 then
+  let bits8 = Big_int_Z.big_int_of_int 0xff
+  and sign = if (Big_int_Z.sign_big_int value) = -1 then
     char_of_int 1
   else
     char_of_int 0
   in
   let rec loop bignum l =
-    if Big_int.gt_big_int bignum Big_int.zero_big_int then (
+    if Big_int_Z.gt_big_int bignum Big_int_Z.zero_big_int then (
       Buffer.add_char l (char_of_int
-        (Big_int.int_of_big_int (Big_int.and_big_int bignum bits8))) ;
-      loop (Big_int.shift_right_towards_zero_big_int bignum 8) l)
+        (Big_int_Z.int_of_big_int (Big_int_Z.and_big_int bignum bits8))) ;
+      loop (Big_int_Z.shift_right_towards_zero_big_int bignum 8) l)
     else
       l
   in
-  let l_result = loop (Big_int.abs_big_int value) (Buffer.create 255) in
+  let l_result = loop (Big_int_Z.abs_big_int value) (Buffer.create 255) in
   let l_length = Buffer.length l_result in
   if l_length <= 255 then (
     Buffer.add_char buffer (char_of_int tag_small_big_ext) ;
@@ -854,11 +863,6 @@ and binary_bits_to_binary value bits buffer =
   else
     Error (output_error "uint32 overflow")
 
-and function_to_binary tag value buffer =
-  Buffer.add_char buffer (char_of_int tag) ;
-  Buffer.add_string buffer value ;
-  Ok (buffer)
-
 and pid_to_binary node_tag node id serial creation buffer =
   let creation_size = String.length creation in
   let tag =
@@ -876,9 +880,12 @@ and pid_to_binary node_tag node id serial creation buffer =
   Ok (buffer)
 
 and port_to_binary node_tag node id creation buffer =
-  let creation_size = String.length creation in
+  let id_size = String.length id
+  and creation_size = String.length creation in
   let tag =
-    if creation_size = 4 then
+    if id_size = 8 then
+      tag_v4_port_ext
+    else if creation_size = 4 then
       tag_new_port_ext
     else
       tag_port_ext
@@ -917,6 +924,11 @@ and reference_to_binary node_tag node id creation buffer =
   else
     Error (output_error "uint16 overflow")
 
+and function_to_binary tag value buffer =
+  Buffer.add_char buffer (char_of_int tag) ;
+  Buffer.add_string buffer value ;
+  Ok (buffer)
+
 let binary_to_term (binary : string) : (t, string) result =
   let size = String.length binary in
   if size <= 1 then
@@ -950,7 +962,7 @@ let rec t_to_string (term : t) : string =
   | OtpErlangInteger (value) ->
     "OtpErlangInteger(" ^ (string_of_int value) ^ ")"
   | OtpErlangIntegerBig (value) ->
-    "OtpErlangIntegerBig(" ^ (Big_int.string_of_big_int value) ^ ")"
+    "OtpErlangIntegerBig(" ^ (Big_int_Z.string_of_big_int value) ^ ")"
   | OtpErlangFloat (value) ->
     "OtpErlangFloat(" ^ (string_of_float value) ^ ")"
   | OtpErlangAtom (value) ->
@@ -1051,6 +1063,12 @@ let binary_ok (value : (string, string) result) : string =
   | Error (error) ->
     raise (BinaryOk error)
 
+let print_binary value =
+  String.iter (fun c ->
+    Printf.printf "\\x%02x" (int_of_char c)
+  ) value ;
+  Printf.printf "\n"
+
 let register_printers () =
   Printexc.register_printer (function
     | TermOk e ->
@@ -1067,7 +1085,7 @@ let register_printers () =
 
   MIT LICENSE (of tests below)
 
-  Copyright (c) 2017-2022 Michael Truog <mjtruog at protonmail dot com>
+  Copyright (c) 2017-2023 Michael Truog <mjtruog at protonmail dot com>
   Copyright (c) 2009-2013 Dmitry Vasiliev <dima@hlabs.org>
 
   Permission is hereby granted, free of charge, to any person obtaining a
@@ -1139,19 +1157,11 @@ let test_port () =
     "\x6F\x73\x74\x00\x00\x00\x06\x00\x00\x00\x00") in
   let port_new = term_ok (binary_to_term port_new_binary) in
   assert ((binary_ok (term_to_binary port_new)) = port_new_binary) ;
-  true
-
-let test_function () =
-  let function1 = OtpErlangFunction (Function.make
-    ~tag:113
-    ~value:(
-      "\x64\x00\x05\x6c\x69\x73\x74\x73\x64" ^
-      "\x00\x06\x6d\x65\x6d\x62\x65\x72\x61\x02")
-    )
-  and binary = "\x83\x71\x64\x00\x05\x6C\x69\x73\x74\x73\x64\x00\x06\x6D" ^
-    "\x65\x6D\x62\x65\x72\x61\x02" in
-  assert ((binary_ok (term_to_binary function1)) = binary) ;
-  assert ((term_ok (binary_to_term binary)) = function1) ;
+  let port_v4_binary = (
+    "\x83\x78\x77\x0D\x6E\x6F\x6E\x6F\x64\x65\x40\x6E\x6F\x68\x6F" ^
+    "\x73\x74\x00\x00\x00\x00\x00\x00\x00\x04\x00\x00\x00\x00") in
+  let port_v4 = term_ok (binary_to_term port_v4_binary) in
+  assert ((binary_ok (term_to_binary port_v4)) = port_v4_binary) ;
   true
 
 let test_reference () =
@@ -1178,6 +1188,19 @@ let test_reference () =
   assert ((binary_ok (term_to_binary ref_newer)) = ref_newer_binary) ;
   true
 
+let test_function () =
+  let function1 = OtpErlangFunction (Function.make
+    ~tag:113
+    ~value:(
+      "\x64\x00\x05\x6c\x69\x73\x74\x73\x64" ^
+      "\x00\x06\x6d\x65\x6d\x62\x65\x72\x61\x02")
+    )
+  and binary = "\x83\x71\x64\x00\x05\x6C\x69\x73\x74\x73\x64\x00\x06\x6D" ^
+    "\x65\x6D\x62\x65\x72\x61\x02" in
+  assert ((binary_ok (term_to_binary function1)) = binary) ;
+  assert ((term_ok (binary_to_term binary)) = function1) ;
+  true
+
 let test_decode_basic () =
   assert (
     (term_error (binary_to_term "")) =
@@ -1370,7 +1393,7 @@ let test_decode_integer () =
     (term_ok (binary_to_term "\x83b\xff\xff\xff\xff")) =
     OtpErlangInteger (-1)) ;
   let max_int_plus_1 = OtpErlangIntegerBig (
-    Big_int.big_int_of_string "4611686018427387904")
+    Big_int_Z.big_int_of_string "4611686018427387904")
   and max_int_plus_1_check = term_ok
     (binary_to_term "\x83n\x08\x00\x00\x00\x00\x00\x00\x00\x00@") in
   assert (
@@ -1445,21 +1468,21 @@ let test_decode_small_big_integer () =
   assert (
     (term_error (binary_to_term "\x83n\x01\x00")) =
     "parse_error: missing data") ;
-  let zero = OtpErlangIntegerBig (Big_int.zero_big_int)
+  let zero = OtpErlangIntegerBig (Big_int_Z.zero_big_int)
   and zero_check = term_ok
     (binary_to_term "\x83n\x00\x00") in
   assert (
     (t_to_string zero) =
     (t_to_string zero_check)) ;
   let bigint1 = OtpErlangIntegerBig (
-    Big_int.big_int_of_string "6618611909121")
+    Big_int_Z.big_int_of_string "6618611909121")
   and bigint1_check = term_ok
     (binary_to_term "\x83n\x06\x00\x01\x02\x03\x04\x05\x06") in
   assert (
     (t_to_string bigint1) =
     (t_to_string bigint1_check)) ;
   let bigint2 = OtpErlangIntegerBig (
-    Big_int.big_int_of_string "-6618611909121")
+    Big_int_Z.big_int_of_string "-6618611909121")
   and bigint2_check = term_ok
     (binary_to_term "\x83n\x06\x01\x01\x02\x03\x04\x05\x06") in
   assert (
@@ -1486,21 +1509,21 @@ let test_decode_large_big_integer () =
   assert (
     (term_error (binary_to_term "\x83o\x00\x00\x00\x01\x00")) =
     "parse_error: missing data") ;
-  let zero = OtpErlangIntegerBig (Big_int.zero_big_int)
+  let zero = OtpErlangIntegerBig (Big_int_Z.zero_big_int)
   and zero_check = term_ok
     (binary_to_term "\x83o\x00\x00\x00\x00\x00") in
   assert (
     (t_to_string zero) =
     (t_to_string zero_check)) ;
   let bigint1 = OtpErlangIntegerBig (
-    Big_int.big_int_of_string "6618611909121")
+    Big_int_Z.big_int_of_string "6618611909121")
   and bigint1_check = term_ok
     (binary_to_term "\x83o\x00\x00\x00\x06\x00\x01\x02\x03\x04\x05\x06") in
   assert (
     (t_to_string bigint1) =
     (t_to_string bigint1_check)) ;
   let bigint2 = OtpErlangIntegerBig (
-    Big_int.big_int_of_string "-6618611909121")
+    Big_int_Z.big_int_of_string "-6618611909121")
   and bigint2_check = term_ok
     (binary_to_term "\x83o\x00\x00\x00\x06\x01\x01\x02\x03\x04\x05\x06") in
   assert (
@@ -1508,6 +1531,41 @@ let test_decode_large_big_integer () =
     (t_to_string bigint2_check)) ;
   true
 
+let test_decode_map () =
+  assert (
+    (term_error (binary_to_term "\x83t")) =
+    "parse_error: missing data") ;
+  assert (
+    (term_error (binary_to_term "\x83t\x00")) =
+    "parse_error: missing data") ;
+  assert (
+    (term_error (binary_to_term "\x83t\x00\x00")) =
+    "parse_error: missing data") ;
+  assert (
+    (term_error (binary_to_term "\x83t\x00\x00\x00")) =
+    "parse_error: missing data") ;
+  assert (
+    (term_error (binary_to_term "\x83t\x00\x00\x00\x01")) =
+    "parse_error: missing data") ;
+  assert (
+    (term_ok (binary_to_term "\x83t\x00\x00\x00\x00")) =
+    OtpErlangMap (Hashtbl.create 0)) ;
+  let map1 = Hashtbl.create 1 in
+  Hashtbl.add map1 (OtpErlangAtom ("a")) (OtpErlangInteger (1)) ;
+  assert (
+    (term_ok (binary_to_term "\x83t\x00\x00\x00\x01s\x01aa\x01")) =
+    OtpErlangMap (map1)) ;
+  (* only able to compare OtpErlangMap of size 1 due to being unordered *)
+  let map2 = Hashtbl.create 1 in
+  Hashtbl.add map2
+    (OtpErlangBinaryBits (("\xA8", 6))) (OtpErlangBinary ("everything")) ;
+  assert (
+    (term_ok (binary_to_term (
+      "\x83\x74\x00\x00\x00\x01\x4D\x00\x00\x00\x01\x06\xA8\x6D\x00\x00" ^
+      "\x00\x0A\x65\x76\x65\x72\x79\x74\x68\x69\x6E\x67"))) =
+    OtpErlangMap (map2)) ;
+  true
+
 let test_encode_tuple () =
   assert (
     (binary_ok (term_to_binary (OtpErlangTuple ([])))) =
@@ -1582,7 +1640,7 @@ let test_encode_list_basic () =
     (binary_ok (term_to_binary (OtpErlangList ([integer1])))) =
     "\x83\x6C\x00\x00\x00\x01\x62\x7F\xFF\xFF\xFF\x6A") ;
   let bigint1 = OtpErlangIntegerBig (
-    Big_int.add_int_big_int 1 (Big_int.big_int_of_int32 Int32.max_int)) in
+    Big_int_Z.add_int_big_int 1 (Big_int_Z.big_int_of_int32 Int32.max_int)) in
   assert (
     (binary_ok (term_to_binary (OtpErlangList ([bigint1])))) =
     "\x83\x6C\x00\x00\x00\x01\x6E\x04\x00\x00\x00\x00\x80\x6A") ;
@@ -1603,7 +1661,7 @@ let test_encode_list_basic () =
     (binary_ok (term_to_binary (OtpErlangList ([integer2])))) =
     "\x83\x6C\x00\x00\x00\x01\x62\x80\x00\x00\x00\x6A") ;
   let bigint2 = OtpErlangIntegerBig (
-    Big_int.add_int_big_int (-1) (Big_int.big_int_of_int32 Int32.min_int)) in
+    Big_int_Z.add_int_big_int (-1) (Big_int_Z.big_int_of_int32 Int32.min_int)) in
   assert (
     (binary_ok (term_to_binary (OtpErlangList ([bigint2])))) =
     "\x83\x6C\x00\x00\x00\x01\x6E\x04\x01\x01\x00\x00\x80\x6A") ;
@@ -1783,12 +1841,12 @@ let test_encode_integer () =
 
 let test_encode_small_big_integer () =
   let bigint1 = OtpErlangIntegerBig (
-    Big_int.add_int_big_int 1 (Big_int.big_int_of_int32 Int32.max_int)) in
+    Big_int_Z.add_int_big_int 1 (Big_int_Z.big_int_of_int32 Int32.max_int)) in
   assert (
     (binary_ok (term_to_binary bigint1)) =
     "\x83n\x04\x00\x00\x00\x00\x80") ;
   let bigint2 = OtpErlangIntegerBig (
-    Big_int.add_int_big_int (-1) (Big_int.big_int_of_int32 Int32.min_int)) in
+    Big_int_Z.add_int_big_int (-1) (Big_int_Z.big_int_of_int32 Int32.min_int)) in
   assert (
     (binary_ok (term_to_binary bigint2)) =
     "\x83n\x04\x01\x01\x00\x00\x80") ;
@@ -1796,12 +1854,12 @@ let test_encode_small_big_integer () =
 
 let test_encode_large_big_integer () =
   let bigint1 = OtpErlangIntegerBig (
-    Big_int.power_int_positive_int 2 2040) in
+    Big_int_Z.power_int_positive_int 2 2040) in
   assert (
     (binary_ok (term_to_binary bigint1)) =
     "\x83o\x00\x00\x01\x00\x00" ^ (String.make 255 '\x00') ^ "\x01") ;
   let bigint2 = OtpErlangIntegerBig (
-    Big_int.minus_big_int (Big_int.power_int_positive_int 2 2040)) in
+    Big_int_Z.minus_big_int (Big_int_Z.power_int_positive_int 2 2040)) in
   assert (
     (binary_ok (term_to_binary bigint2)) =
     "\x83o\x00\x00\x01\x00\x01" ^ (String.make 255 '\x00') ^ "\x01") ;
@@ -1825,13 +1883,32 @@ let test_encode_float () =
     "\x83F\xc0\t!\xfbM\x12\xd8J") ;
   true
 
+let test_encode_map () =
+  assert (
+    (binary_ok (term_to_binary (OtpErlangMap (Hashtbl.create 0)))) =
+    "\x83t\x00\x00\x00\x00") ;
+  let map1 = Hashtbl.create 1 in
+  Hashtbl.add map1 (OtpErlangAtom ("a")) (OtpErlangInteger (1)) ;
+  assert (
+    (binary_ok (term_to_binary (OtpErlangMap (map1)))) =
+    "\x83t\x00\x00\x00\x01s\x01aa\x01") ;
+  (* only able to compare OtpErlangMap of size 1 due to being unordered *)
+  let map2 = Hashtbl.create 1 in
+  Hashtbl.add map2
+    (OtpErlangBinaryBits (("\xA8", 6))) (OtpErlangBinary ("everything")) ;
+  assert (
+    (binary_ok (term_to_binary (OtpErlangMap (map2)))) =
+    "\x83\x74\x00\x00\x00\x01\x4D\x00\x00\x00\x01\x06\xA8\x6D\x00\x00" ^
+    "\x00\x0A\x65\x76\x65\x72\x79\x74\x68\x69\x6E\x67") ;
+  true
+
 let tests =
   register_printers () ;
 [
   "binary_to_term/term_to_binary (pid)", test_pid;
   "binary_to_term/term_to_binary (port)", test_port;
-  "binary_to_term/term_to_binary (function)", test_function;
   "binary_to_term/term_to_binary (reference)", test_reference;
+  "binary_to_term/term_to_binary (function)", test_function;
   "binary_to_term (basic)", test_decode_basic;
   "binary_to_term (atom)", test_decode_atom;
   "binary_to_term (predefined atom)", test_decode_predefined_atom;
@@ -1847,6 +1924,7 @@ let tests =
   "binary_to_term (float)", test_decode_float;
   "binary_to_term (small bigint)", test_decode_small_big_integer;
   "binary_to_term (large bigint)", test_decode_large_big_integer;
+  "binary_to_term (map)", test_decode_map;
   "term_to_binary (tuple)", test_encode_tuple;
   "term_to_binary (empty list)", test_encode_empty_list;
   "term_to_binary (string list)", test_encode_string_list;
@@ -1863,5 +1941,6 @@ let tests =
   "term_to_binary (small bigint)", test_encode_small_big_integer;
   "term_to_binary (large bigint)", test_encode_large_big_integer;
   "term_to_binary (float)", test_encode_float;
+  "term_to_binary (map)", test_encode_map;
 ]
 
