@@ -5,7 +5,7 @@
 
   MIT License
 
-  Copyright (c) 2017 Michael Truog <mjtruog at protonmail dot com>
+  Copyright (c) 2017-2023 Michael Truog <mjtruog at protonmail dot com>
 
   Permission is hereby granted, free of charge, to any person obtaining a
   copy of this software and associated documentation files (the "Software"),
@@ -27,7 +27,7 @@
 
  *)
 
-(** Erlang Binary Term Format Encoding/Decoding *)
+(** Erlang External Term Format Encoding/Decoding *)
 
 module Pid :
   sig
@@ -36,7 +36,7 @@ module Pid :
       node : string;
       id : string;
       serial : string;
-      creation : int;
+      creation : string;
     }
   end
 module Port :
@@ -45,7 +45,7 @@ module Port :
       node_tag : int;
       node : string;
       id : string;
-      creation : int;
+      creation : string;
     }
   end
 module Reference :
@@ -54,7 +54,7 @@ module Reference :
       node_tag : int;
       node : string;
       id : string;
-      creation : int;
+      creation : string;
     }
   end
 module Function :
@@ -67,7 +67,7 @@ module Function :
 
 type t =
     OtpErlangInteger of int
-  | OtpErlangIntegerBig of Big_int.big_int
+  | OtpErlangIntegerBig of Big_int_Z.big_int
   | OtpErlangFloat of float
   | OtpErlangAtom of string
   | OtpErlangAtomUTF8 of string
@@ -94,6 +94,9 @@ val term_to_binary : t -> (string, string) result
 (** Convert the OCaml Erlang type to a string *)
 val t_to_string : t -> string
 
+(** Print a binary string *)
+val print_binary : string -> unit
+
 (**/**)
 (* begin section ignored by ocamldoc *)
 
