@@ -3,7 +3,7 @@
 #
 # MIT License
 #
-# Copyright (c) 2014-2022 Michael Truog <mjtruog at protonmail dot com>
+# Copyright (c) 2014-2023 Michael Truog <mjtruog at protonmail dot com>
 #
 # Permission is hereby granted, free of charge, to any person obtaining a
 # copy of this software and associated documentation files (the "Software"),
@@ -29,7 +29,7 @@ use strict;
 use warnings;
 use 5.010;
 
-$CloudI::API::VERSION = '2.05';
+$CloudI::API::VERSION = '2.06';
 
 use POSIX qw(getenv);
 use IO::Handle;
@@ -82,11 +82,7 @@ sub new
         print STDERR "CloudI service execution must occur in CloudI";
         die CloudI::InvalidInputException->new();
     }
-    my $buffer_size = getenv('CLOUDI_API_INIT_BUFFER_SIZE');
-    if (! defined($buffer_size) or $buffer_size !~ /^\d+$/)
-    {
-        die CloudI::InvalidInputException->new();
-    }
+    my $buffer_size = _getenv_to_uint('CLOUDI_API_INIT_BUFFER_SIZE');
     my $s = IO::Handle->new_from_fd($thread_index + 3, 'r+');
     $s->autoflush(1);
     my $use_header;
@@ -153,12 +149,7 @@ sub DESTROY
 # callable without an object
 sub thread_count
 {
-    my $count = getenv('CLOUDI_API_INIT_THREAD_COUNT');
-    if (! defined($count) or $count !~ /^\d+$/)
-    {
-        die CloudI::InvalidInputException->new();
-    }
-    return $count;
+    return _getenv_to_uint('CLOUDI_API_INIT_THREAD_COUNT');
 }
 
 sub subscribe
@@ -418,6 +409,12 @@ sub process_index
     return $self->{_process_index};
 }
 
+# callable without an object
+sub process_index_
+{
+    return _getenv_to_uint('CLOUDI_API_INIT_PROCESS_INDEX');
+}
+
 sub process_count
 {
     my $self = shift;
@@ -430,12 +427,24 @@ sub process_count_max
     return $self->{_process_count_max};
 }
 
+# callable without an object
+sub process_count_max_
+{
+    return _getenv_to_uint('CLOUDI_API_INIT_PROCESS_COUNT_MAX');
+}
+
 sub process_count_min
 {
     my $self = shift;
     return $self->{_process_count_min};
 }
 
+# callable without an object
+sub process_count_min_
+{
+    return _getenv_to_uint('CLOUDI_API_INIT_PROCESS_COUNT_MIN');
+}
+
 sub prefix
 {
     my $self = shift;
@@ -448,6 +457,12 @@ sub timeout_initialize
     return $self->{_timeout_initialize};
 }
 
+# callable without an object
+sub timeout_initialize_
+{
+    return _getenv_to_uint('CLOUDI_API_INIT_TIMEOUT_INITIALIZE');
+}
+
 sub timeout_async
 {
     my $self = shift;
@@ -466,6 +481,12 @@ sub timeout_terminate
     return $self->{_timeout_terminate};
 }
 
+# callable without an object
+sub timeout_terminate_
+{
+    return _getenv_to_uint('CLOUDI_API_INIT_TIMEOUT_TERMINATE');
+}
+
 sub priority_default
 {
     my $self = shift;
@@ -804,16 +825,21 @@ sub _poll_request
                 $process_count_max,
                 $process_count_min,
                 $prefix_size) = unpack('L5', substr($data, $i, $j));
-            $i += $j; $j = $prefix_size + 4 + 4 + 4 + 4 + 1 + 1;
+            $i += $j; $j = $prefix_size + 4 + 4 + 4 + 4 + 1 + 1 + 4;
             my ($prefix,
                 $timeout_initialize,
                 $timeout_async,
                 $timeout_sync,
                 $timeout_terminate,
                 $priority_default,
-                $fatal_exceptions) = unpack("Z$prefix_size L4 c C",
-                                            substr($data, $i, $j));
+                $fatal_exceptions,
+                $bind) = unpack("Z$prefix_size L4 c C l",
+                                substr($data, $i, $j));
             $i += $j;
+            if ($bind >= 0)
+            {
+                die CloudI::InvalidInputException->new();
+            }
             if ($i != $data_size)
             {
                 assert($external == 0);
@@ -1209,6 +1235,17 @@ sub _recv
     return $data;
 }
 
+sub _getenv_to_uint
+{
+    my ($name) = @_;
+    my $value = getenv($name);
+    if (! defined($value) or $value !~ /^\d+$/)
+    {
+        die CloudI::InvalidInputException->new();
+    }
+    return $value;
+}
+
 sub assert
 {
     my ($test) = @_;
