@@ -3,7 +3,7 @@
 //
 // MIT License
 //
-// Copyright (c) 2014-2022 Michael Truog <mjtruog at protonmail dot com>
+// Copyright (c) 2014-2023 Michael Truog <mjtruog at protonmail dot com>
 //
 // Permission is hereby granted, free of charge, to any person obtaining a
 // copy of this software and associated documentation files (the "Software"),
@@ -86,9 +86,7 @@ class API
             fwrite(STDERR, "CloudI service execution must occur in CloudI\n");
             throw new InvalidInputException();
         }
-        $buffer_size_str = getenv('CLOUDI_API_INIT_BUFFER_SIZE');
-        if ($buffer_size_str == false)
-            throw new InvalidInputException();
+        $buffer_size = self::getenv_to_uint('CLOUDI_API_INIT_BUFFER_SIZE');
         $this->s = fopen('php://fd/' . strval($thread_index + 3), 'rwb');
         if ($protocol_str == 'tcp')
             $this->use_header = true;
@@ -101,7 +99,7 @@ class API
         $this->initialization_complete = false;
         $this->fatal_exceptions = false;
         $this->terminate = false;
-        $this->size = intval($buffer_size_str);
+        $this->size = $buffer_size;
         $this->callbacks = array();
         $this->timeout_terminate = 10; // TIMEOUT_TERMINATE_MIN
         $this->send(\Erlang\term_to_binary(new \Erlang\OtpErlangAtom('init')));
@@ -121,10 +119,7 @@ class API
 
     public static function thread_count()
     {
-        $count_str = getenv('CLOUDI_API_INIT_THREAD_COUNT');
-        if ($count_str == false)
-            throw new InvalidInputException();
-        return intval($count_str);
+        return self::getenv_to_uint('CLOUDI_API_INIT_THREAD_COUNT');
     }
 
     public function subscribe($pattern, $object, $method)
@@ -305,6 +300,11 @@ class API
         return $this->process_index;
     }
 
+    public static function process_index_()
+    {
+        return self::getenv_to_uint('CLOUDI_API_INIT_PROCESS_INDEX');
+    }
+
     public function process_count()
     {
         return $this->process_count;
@@ -315,11 +315,21 @@ class API
         return $this->process_count_max;
     }
 
+    public static function process_count_max_()
+    {
+        return self::getenv_to_uint('CLOUDI_API_INIT_PROCESS_COUNT_MAX');
+    }
+
     public function process_count_min()
     {
         return $this->process_count_min;
     }
 
+    public static function process_count_min_()
+    {
+        return self::getenv_to_uint('CLOUDI_API_INIT_PROCESS_COUNT_MIN');
+    }
+
     public function prefix()
     {
         return $this->prefix;
@@ -330,6 +340,11 @@ class API
         return $this->timeout_initialize;
     }
 
+    public static function timeout_initialize_()
+    {
+        return self::getenv_to_uint('CLOUDI_API_INIT_TIMEOUT_INITIALIZE');
+    }
+
     public function timeout_async()
     {
         return $this->timeout_async;
@@ -345,6 +360,11 @@ class API
         return $this->timeout_terminate;
     }
 
+    public static function timeout_terminate_()
+    {
+        return self::getenv_to_uint('CLOUDI_API_INIT_TIMEOUT_TERMINATE');
+    }
+
     public function priority_default()
     {
         return $this->priority_default;
@@ -657,15 +677,20 @@ class API
                          $prefix_size) = unpack('L5', substr($data, $i, $j));
                     $i += $j; $j = $prefix_size;
                     $prefix = substr($data, $i, $j - 1);
-                    $i += $j; $j = 4 + 4 + 4 + 4 + 1 + 1;
-                    $tmp = unpack('L4a/cb/Cc', substr($data, $i, $j));
+                    $i += $j; $j = 4 + 4 + 4 + 4 + 1 + 1 + 4;
+                    $tmp = unpack('L4a/cb/Cc/ld', substr($data, $i, $j));
                     $timeout_initialize = $tmp['a1'];
                     $timeout_async = $tmp['a2'];
                     $timeout_sync = $tmp['a3'];
                     $timeout_terminate = $tmp['a4'];
                     $priority_default = $tmp['b'];
                     $fatal_exceptions = $tmp['c'];
+                    $bind = $tmp['d'];
                     $i += $j;
+                    if ($bind >= 0)
+                    {
+                        throw new InvalidInputException();
+                    }
                     if ($i != $data_size)
                     {
                         assert($external == false);
@@ -995,6 +1020,17 @@ class API
         }
         return $data;
     }
+
+    private static function getenv_to_uint($name)
+    {
+        $value_str = getenv($name);
+        if ($value_str == false)
+            throw new InvalidInputException();
+        $value = intval($value_str);
+        if ($value < 0)
+            throw new InvalidInputException();
+        return $value;
+    }
 }
 
 class InvalidInputException extends \Exception
