@@ -4,7 +4,7 @@
 #
 # MIT License
 #
-# Copyright (c) 2011-2022 Michael Truog <mjtruog at protonmail dot com>
+# Copyright (c) 2011-2023 Michael Truog <mjtruog at protonmail dot com>
 #
 # Permission is hereby granted, free of charge, to any person obtaining a
 # copy of this software and associated documentation files (the "Software"),
@@ -43,7 +43,7 @@ module CloudI
 
         def initialize(thread_index)
             protocol = API.getenv('CLOUDI_API_INIT_PROTOCOL')
-            buffer_size_str = API.getenv('CLOUDI_API_INIT_BUFFER_SIZE')
+            buffer_size = API.getenv_to_uint('CLOUDI_API_INIT_BUFFER_SIZE')
             if protocol == 'tcp'
                 @s = IO.for_fd(thread_index + 3, File::RDWR, autoclose: false)
                 @s.sync = true
@@ -62,7 +62,7 @@ module CloudI
             end
             @initialization_complete = false
             @terminate = false
-            @size = buffer_size_str.to_i
+            @size = buffer_size
             @callbacks = Hash.new
             @timeout_terminate = 10 # TIMEOUT_TERMINATE_MIN
             send(Erlang.term_to_binary(:init))
@@ -80,8 +80,7 @@ module CloudI
         attr_reader :priority_default
 
         def self.thread_count
-            s = getenv('CLOUDI_API_INIT_THREAD_COUNT')
-            s.to_i
+            return API.getenv_to_uint('CLOUDI_API_INIT_THREAD_COUNT')
         end
 
         def subscribe(pattern, function)
@@ -246,6 +245,26 @@ module CloudI
             return poll_request(nil, false)
         end
 
+        def self.process_index_
+            return API.getenv_to_uint('CLOUDI_API_INIT_PROCESS_INDEX')
+        end
+
+        def self.process_count_max_
+            return API.getenv_to_uint('CLOUDI_API_INIT_PROCESS_COUNT_MAX')
+        end
+
+        def self.process_count_min_
+            return API.getenv_to_uint('CLOUDI_API_INIT_PROCESS_COUNT_MIN')
+        end
+
+        def self.timeout_initialize_
+            return API.getenv_to_uint('CLOUDI_API_INIT_TIMEOUT_INITIALIZE')
+        end
+
+        def self.timeout_terminate_
+            return API.getenv_to_uint('CLOUDI_API_INIT_TIMEOUT_TERMINATE')
+        end
+
         def null_response(request_type, name, pattern, request_info, request,
                           timeout, priority, trans_id, source)
             return ''
@@ -484,8 +503,8 @@ module CloudI
                     @process_count_max = tmp[2]
                     @process_count_min = tmp[3]
                     prefix_size = tmp[4]
-                    i += j; j = prefix_size + 4 + 4 + 4 + 4 + 1 + 1
-                    tmp = data[i, j].unpack("Z#{prefix_size}LLLLcC")
+                    i += j; j = prefix_size + 4 + 4 + 4 + 4 + 1 + 1 + 4
+                    tmp = data[i, j].unpack("Z#{prefix_size}LLLLcCl")
                     @prefix = tmp[0]
                     @timeout_initialize = tmp[1]
                     @timeout_async = tmp[2]
@@ -493,6 +512,10 @@ module CloudI
                     @timeout_terminate = tmp[4]
                     @priority_default = tmp[5]
                     @fatal_exceptions = tmp[6]
+                    bind = tmp[7]
+                    if bind >= 0 then
+                        raise InvalidInputException
+                    end
                     i += j
                     if i != data_size
                         API.assert{external == false}
@@ -780,6 +803,14 @@ module CloudI
         def self.getenv(key)
             ENV[key] or raise InvalidInputException
         end
+
+        def self.getenv_to_uint(name)
+            value = API.getenv(name).to_i
+            if value < 0 then
+                raise InvalidInputException
+            end
+            return value
+        end
     end
 
     class InvalidInputException < Exception
