@@ -4,21 +4,19 @@
 %% then sends a 200 reply back.
 
 -module(loop_handler_body_h).
--behaviour(cowboy_loop_handler).
 
--export([init/3]).
+-export([init/2]).
 -export([info/3]).
 -export([terminate/3]).
 
-init(_, Req, _) ->
+init(Req, _) ->
 	self() ! timeout,
-	{loop, Req, undefined, 5000, hibernate}.
+	{cowboy_loop, Req, undefined, hibernate}.
 
-info(timeout, Req, State) ->
-	{ok, Body, Req2} = cowboy_req:body(Req),
+info(timeout, Req0, State) ->
+	{ok, Body, Req} = cowboy_req:read_body(Req0),
 	100000 = byte_size(Body),
-	{ok, Req3} = cowboy_req:reply(200, Req2),
-	{ok, Req3, State}.
+	{stop, cowboy_req:reply(200, Req), State}.
 
-terminate({normal, shutdown}, _, _) ->
+terminate(stop, _, _) ->
 	ok.
