@@ -1,3 +1,4 @@
+%% @hidden
 -module(cloudi_x_elli_example_middleware).
 -export([handle/2, handle_event/3]).
 -behaviour(cloudi_x_elli_handler).
@@ -8,13 +9,12 @@
 %%
 
 handle(Req, _Args) ->
-    case cloudi_x_elli_request:path(Req) of
-        [<<"middleware">>, <<"short-circuit">>] ->
-            {200, [], <<"short circuit!">>};
-        _ ->
-            ignore
-    end.
+    do_handle(cloudi_x_elli_request:path(Req)).
 
+do_handle([<<"middleware">>, <<"short-circuit">>]) ->
+    {200, [], <<"short circuit!">>};
+do_handle(_) ->
+    ignore.
 
 
 %%
