@@ -1,7 +1,14 @@
 %%% @doc
 %%% Codec for `geometry' PostGIS umbrella datatype.
-%%% http://postgis.net/docs/manual-2.4/geometry.html
-%%% $POSTGIS$/postgis/lwgeom_inout.c
+%%%
+%%% XXX: PostGIS is not a Postgres's built-in datatype! It should be instaled
+%%% separately and enabled via `CREATE EXTENSION postgis'.
+%%% <ul>
+%%%  <li>[http://postgis.net/docs/manual-2.4/geometry.html]</li>
+%%%  <li>$POSTGIS$/postgis/lwgeom_inout.c</li>
+%%% </ul>
+%%% @end
+%%% @see cloudi_x_ewkb
 %%% @end
 %%% Created : 14 Oct 2017 by Sergey Prokhorov <me@seriyps.ru>
 
@@ -12,7 +19,7 @@
 
 -export_type([data/0]).
 
--type data() :: cloudi_x_epgsql_ewkb:geometry().
+-type data() :: cloudi_x_ewkb:geometry().
 
 init(_, _) -> [].
 
@@ -20,9 +27,9 @@ names() ->
     [geometry].
 
 encode(Geo, geometry, _) ->
-    cloudi_x_epgsql_ewkb:encode_geometry(Geo).
+    cloudi_x_ewkb:encode_geometry(Geo).
 
 decode(Bin, geometry, _) ->
-    cloudi_x_epgsql_ewkb:decode_geometry(Bin).
+    cloudi_x_ewkb:decode_geometry(Bin).
 
 decode_text(V, _, _) -> V.
