@@ -376,8 +376,19 @@ connect_with_invalid_client_cert(Config) ->
     Dir = filename:join(code:lib_dir(epgsql), ?TEST_DATA_DIR),
     File = fun(Name) -> filename:join(Dir, Name) end,
     Trap = process_flag(trap_exit, true),
+    %% pre-otp23:
+    %% {error,
+    %%   {ssl_negotiation_failed,
+    %%     {tls_alert,
+    %%       {unknown_ca, "received SERVER ALERT: Fatal - Unknown CA"}}}}
+    %% otp23+:
+    %% {error,
+    %%   {sock_error,
+    %%     {tls_alert,
+    %%       {unknown_ca, "TLS client: <..> received SERVER ALERT: Fatal - Unknown CA\n"}}}}
     ?assertMatch(
-       {error, {ssl_negotiation_failed, _}},
+       {error, {Err, {tls_alert, _}}} when Err == ssl_negotiation_failed;
+                                           Err == sock_error,
        Module:connect(
          #{username => "epgsql_test_cert",
            database => "epgsql_test_db1",
@@ -386,9 +397,14 @@ connect_with_invalid_client_cert(Config) ->
            ssl => true,
            ssl_opts =>
                [{keyfile, File("bad-client.key")},
-                {certfile, File("bad-client.crt")}]}
+                {certfile, File("bad-client.crt")},
+                %% TLS-1.3 seems to connect fine, but then sends alert asynchronously
+                {versions, ['tlsv1.2']}
+               ]}
         )),
-    ?assertMatch({'EXIT', _, {ssl_negotiation_failed, _}}, receive Stop -> Stop end),
+    ?assertMatch({'EXIT', _, {Err, {tls_alert, _}}} when Err == ssl_negotiation_failed;
+                                                         Err == sock_error,
+                 receive Stop -> Stop end),
     process_flag(trap_exit, Trap).
 
 connect_map(Config) ->
