@@ -94,12 +94,12 @@
 %% generate an error log message by exometer.
 %%
 %%
-%% === exometer_report/4 ===
+%% === exometer_report/5 ===
 %%
 %% The `exometer_report()' function is invoked as follows:
 %%
 %% <pre lang="erlang">
-%%      exometer_report(Metric, DataPoint, State)</pre>
+%%      exometer_report(Metric, DataPoint, Extra, Value, State)</pre>
 %%
 %% The custom plugin will receive this call when a periodic subscription
 %% triggers and wants to report its current value through the plugin.
@@ -111,6 +111,9 @@
 %% + `DataPoint'<br/>Specifies the data point or data points within the metric
 %%  to be reported.
 %%
+%% + `Extra'<br/>Specifies the extra data, which can be anything the reporter
+%% can understand.
+%%
 %% + `State'<br/>Contains the state returned by the last called plugin function.
 %%
 %% The `exometer_report()' function should return `{ok, State}' where
@@ -140,6 +143,8 @@
 %% + `Extra'<br/>Specifies the extra data, which can be anything the reporter
 %% can understand.
 %%
+%% + `Value'<br/>Specifies the value for the datapoint, which is reported.
+%%
 %% + `State'<br/>Contains the state returned by the last called plugin function.
 %%
 %% The `exometer_unsubscribe()' function should return `{ok, State}' where
@@ -161,7 +166,7 @@
 %% That is, e.g. when a `select' pattern is used, all found values are passed
 %% to the reporter in one message. If bulk reporting is not enabled, each
 %% datapoint/value pair will be passed separately to the
-%% <a href="#exometer_report/4"><code>exometer_report/4</code></a> function. If `report_bulk' was enabled, the
+%% <a href="#exometer_report/5"><code>exometer_report/5</code></a> function. If `report_bulk' was enabled, the
 %% reporter callback will get all values at once. Note that this happens
 %% also for single values, which are then passed as a list of one metric,
 %% with a list of one datapoint/value pair.
@@ -276,7 +281,7 @@
           reporter              :: module()     | '_',
           metric                :: metric()     | '_',
           datapoint             :: datapoints() | '_',
-          retry_failed_metrics  :: boolean()    | '_',
+          retry_failed_metrics  :: boolean()    | undefined | '_',
           extra                 :: extra()      | '_'
          }).
 
@@ -302,7 +307,7 @@
 -record(reporter, {
           name      :: atom()                | '_',
           pid       :: pid()                 | atom(), % in select()
-          mref      :: reference()           | '_',
+          mref      :: reference()           | undefined | '_',
           module    :: module()              | '_',
           opts = [] :: [{atom(), any()}]     | '_',
           intervals = [] :: [#interval{}]    | '_',
@@ -375,12 +380,14 @@ subscribe(Reporter, Metric, DataPoint, Interval, Extra, Retry)
                           retry_failed_metrics = Retry,
                           extra = Extra}, Interval}).
 
+-dialyzer({no_return, unsubscribe/3}).
 -spec unsubscribe(module(), metric(), datapoint()) ->
                          ok | not_found.
 %% @equiv unsubscribe(Reporter, Metric, DataPoint, [])
 unsubscribe(Reporter, Metric, DataPoint) ->
     unsubscribe(Reporter, Metric, DataPoint, []).
 
+-dialyzer({no_return, unsubscribe/4}).
 -spec unsubscribe(module(), metric(), datapoint() | [datapoint()], extra()) ->
                          ok | not_found.
 %% @doc Removes a subscription.
@@ -720,6 +727,7 @@ do_start_reporters(S) ->
     end,
     S#st{}.
 
+-spec make_reporter_recs([{atom(), list()}]) -> [#reporter{}].
 make_reporter_recs([{R, Opts}|T]) when is_atom(R), is_list(Opts) ->
     [#reporter{name = R,
                module = get_module(R, Opts),
@@ -1299,7 +1307,7 @@ adjust_interval(Time, T0) ->
             %% Most likely due to clock adjustment
             {Time, T1};
         D ->
-            {D, T0}
+            {Time-D, T0}
     end.
 
 tdiff(T1, T0) ->
@@ -1522,6 +1530,7 @@ maybe_send_after(enabled, Key, Interval) when is_integer(Interval) ->
 maybe_send_after(_, _, _) ->
     undefined.
 
+-dialyzer({no_return, unsubscribe_/4}).
 unsubscribe_(Reporter, Metric, DataPoint, Extra) ->
     ?log(info, "unsubscribe_(~p, ~p, ~p, ~p)~n",
           [ Reporter, Metric, DataPoint, Extra]),
@@ -1539,19 +1548,17 @@ unsubscribe_(#subscriber{key = #key{reporter = Reporter,
                                     metric = Metric,
                                     datapoint = DataPoint,
                                     extra = Extra} = Key, t_ref = TRef}) ->
-    try_send(
-      Reporter, {exometer_unsubscribe, Metric, DataPoint, Extra}),
+    try_send(Reporter, {exometer_unsubscribe, Metric, DataPoint, Extra}),
     cancel_timer(TRef),
     ets:delete(?EXOMETER_SUBS, Key),
     ok.
 
-
 report_values(Found, #key{reporter = Reporter, extra = Extra} = Key) ->
     try Reporter ! {exometer_report, Found, Extra}
     catch
-        error:Reason ->
+        ?EXCEPTION(error, Reason, Stacktrace) ->
             ?log(error, "~p~nKey = ~p~nTrace: ~p",
-                        [Reason, Key, erlang:get_stacktrace()])
+                        [Reason, Key, ?GET_STACK(Stacktrace)])
     end.
 
 retrieve_metric({Metric, Type, Enabled}, Acc) ->
