@@ -8,7 +8,7 @@
 %%%
 %%% MIT License
 %%%
-%%% Copyright (c) 2016-2018 Michael Truog <mjtruog at gmail dot com>
+%%% Copyright (c) 2016-2023 Michael Truog <mjtruog at protonmail dot com>
 %%%
 %%% Permission is hereby granted, free of charge, to any person obtaining a
 %%% copy of this software and associated documentation files (the "Software"),
@@ -28,13 +28,13 @@
 %%% FROM, OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER
 %%% DEALINGS IN THE SOFTWARE.
 %%%
-%%% @author Michael Truog <mjtruog [at] gmail (dot) com>
-%%% @copyright 2016-2018 Michael Truog
-%%% @version 1.7.3 {@date} {@time}
+%%% @author Michael Truog <mjtruog at protonmail dot com>
+%%% @copyright 2016-2023 Michael Truog
+%%% @version 2.0.6 {@date} {@time}
 %%%------------------------------------------------------------------------
 
 -module(cloudi_x_cgroups).
--author('mjtruog [at] gmail (dot) com').
+-author('mjtruog at protonmail dot com').
 
 %% external interface
 -export([create/4,
@@ -45,13 +45,15 @@
          new/1,
          shell/2,
          update/4,
-         update_or_create/4]).
+         update_or_create/4,
+         version/1]).
 
 -record(cloudi_x_cgroups,
     {
         version :: pos_integer(),
         path :: string(),
-        mounted :: boolean()
+        mounted :: boolean(),
+        root :: boolean()
     }).
 -define(APPLICATION, cloudi_x_cgroups).
 
@@ -75,8 +77,9 @@
 %%-------------------------------------------------------------------------
 %% @doc
 %% ===Create a specific cgroup.===
-%% Files cpuset.cpus and cpuset.mems are set if they are not initialized
-%% due to cgroup.clone_children (using the root values).
+%% With cloudi_x_cgroups v1, files cpuset.cpus and cpuset.mems are set
+%% if they are not initialized due to cgroup.clone_children
+%% (using the root values).
 %% @end
 %%-------------------------------------------------------------------------
 
@@ -88,7 +91,8 @@
     {error, any()}.
 
 create([_ | _] = CGroupPath, OSPids, CGroupParameters,
-       #cloudi_x_cgroups{path = Path} = State) ->
+       #cloudi_x_cgroups{path = Path,
+                root = true} = State) ->
     CGroupPathValid = cgroup_path_valid(CGroupPath),
     if
         CGroupPathValid =:= false ->
@@ -101,7 +105,10 @@ create([_ | _] = CGroupPath, OSPids, CGroupParameters,
                 false ->
                     create_cgroup(CGroupPath, OSPids, CGroupParameters, State)
             end
-    end.
+    end;
+create(_, _, _,
+       #cloudi_x_cgroups{root = false}) ->
+    {error, root_required}.
 
 %%-------------------------------------------------------------------------
 %% @doc
@@ -117,7 +124,8 @@ create([_ | _] = CGroupPath, OSPids, CGroupParameters,
     {error, any()}.
 
 delete([_ | _] = CGroupPath,
-       #cloudi_x_cgroups{path = Path}) ->
+       #cloudi_x_cgroups{path = Path,
+                root = true}) ->
     CGroupPathValid = cgroup_path_valid(CGroupPath),
     if
         CGroupPathValid =:= false ->
@@ -130,7 +138,10 @@ delete([_ | _] = CGroupPath,
                 {Status, Output} ->
                     {error, {rmdir, Status, Output}}
             end
-    end.
+    end;
+delete(_,
+       #cloudi_x_cgroups{root = false}) ->
+    {error, root_required}.
 
 %%-------------------------------------------------------------------------
 %% @doc
@@ -146,7 +157,8 @@ delete([_ | _] = CGroupPath,
     {error, any()}.
 
 delete_recursive([_ | _] = CGroupPath,
-                 #cloudi_x_cgroups{path = Path}) ->
+                 #cloudi_x_cgroups{path = Path,
+                          root = true}) ->
     CGroupPathValid = cgroup_path_valid(CGroupPath),
     if
         CGroupPathValid =:= false ->
@@ -161,7 +173,10 @@ delete_recursive([_ | _] = CGroupPath,
                 {Status, Output} ->
                     {error, {rmdir, Status, Output}}
             end
-    end.
+    end;
+delete_recursive(_,
+                 #cloudi_x_cgroups{root = false}) ->
+    {error, root_required}.
 
 %%-------------------------------------------------------------------------
 %% @doc
@@ -185,9 +200,15 @@ destroy(#cloudi_x_cgroups{path = Path,
 %% @end
 %%-------------------------------------------------------------------------
 
+-type new_error_reasons() ::
+    {'version_default', Version :: pos_integer()} |
+    {'path_mounts', Status :: pos_integer(), Output :: list(binary())} |
+    {'path_v1', Status :: pos_integer(), Output :: list(binary())} |
+    {'path_v2', Status :: pos_integer(), Output :: list(binary())}.
+
 -spec new() ->
     {ok, #cloudi_x_cgroups{}} |
-    {error, any()}.
+    {error, new_error_reasons()}.
 
 new() ->
     new([]).
@@ -200,7 +221,7 @@ new() ->
 
 -spec new(Options0 :: options()) ->
     {ok, #cloudi_x_cgroups{}} |
-    {error, any()}.
+    {error, new_error_reasons()}.
 
 new(Options0) ->
     {VersionDefault,
@@ -222,8 +243,9 @@ new(Options0) ->
            ($/ == hd(lists:reverse(PathV2))) andalso (length(PathV2) > 1),
     true = (PathMounts =:= undefined) orelse
            (is_list(PathMounts) andalso is_integer(hd(PathMounts))),
+    Root = os:getenv("USER") == "root",
     new_state(VersionDefault, VersionDefaultRequired,
-              PathV1, PathV2, PathMounts).
+              PathV1, PathV2, PathMounts, Root).
 
 %%-------------------------------------------------------------------------
 %% @doc
@@ -240,7 +262,7 @@ shell(Command, Arguments) ->
                              [{args, ["-"]}, {cd, "/"},
                               stream, binary, stderr_to_stdout, exit_status]),
     Exec = io_lib:format(Command, Arguments),
-    true = erlang:port_command(Shell, ["exec ", Exec, "\n"]),
+    true = erlang:port_command(Shell, [Exec, "\nexit $?\n"]),
     shell_output(Shell, []).
 
 %%-------------------------------------------------------------------------
@@ -259,7 +281,8 @@ shell(Command, Arguments) ->
 
 update(CGroupPath, OSPids, CGroupParameters,
        #cloudi_x_cgroups{version = Version,
-                path = Path}) ->
+                path = Path,
+                root = Root}) ->
     CGroupPathValid = cgroup_path_valid(CGroupPath),
     OSPidsValid = lists:all(fun(OSPid) ->
         is_integer(OSPid) andalso (OSPid > 0)
@@ -281,10 +304,10 @@ update(CGroupPath, OSPids, CGroupParameters,
             {error, {invalid_cgroup_parameters, CGroupParameters}};
         true ->
             CGroupPathFull = Path ++ CGroupPath,
-            case update_parameters(CGroupParameters, Version,
-                                   CGroupPathFull, Path) of
+            case update_parameters(Version, CGroupParameters,
+                                   CGroupPathFull, Path, Root) of
                 ok ->
-                    update_pids(OSPids, Version, CGroupPathFull);
+                    update_pids(OSPids, CGroupPathFull, Root);
                 {error, _} = Error ->
                     Error
             end
@@ -304,7 +327,8 @@ update(CGroupPath, OSPids, CGroupParameters,
     {error, any()}.
 
 update_or_create([_ | _] = CGroupPath, OSPids, CGroupParameters,
-                 #cloudi_x_cgroups{path = Path} = State) ->
+                 #cloudi_x_cgroups{path = Path,
+                          root = true} = State) ->
     CGroupPathValid = cgroup_path_valid(CGroupPath),
     if
         CGroupPathValid =:= false ->
@@ -317,21 +341,36 @@ update_or_create([_ | _] = CGroupPath, OSPids, CGroupParameters,
                 false ->
                     create_cgroup(CGroupPath, OSPids, CGroupParameters, State)
             end
-    end.
+    end;
+update_or_create(_, _, _,
+                 #cloudi_x_cgroups{root = false}) ->
+    {error, root_required}.
+
+%%-------------------------------------------------------------------------
+%% @doc
+%% ===The cgroup version used.===
+%% @end
+%%-------------------------------------------------------------------------
+
+-spec version(#cloudi_x_cgroups{}) ->
+    pos_integer().
+
+version(#cloudi_x_cgroups{version = Version}) ->
+    Version.
 
 %%%------------------------------------------------------------------------
 %%% Private functions
 %%%------------------------------------------------------------------------
 
 new_state(1 = Version, VersionDefaultRequired,
-          PathV1, PathV2, PathMounts) ->
+          PathV1, PathV2, PathMounts, Root) ->
     case new_paths(PathV1, PathV2, PathMounts) of
         {ok, {MountedV1, NewPathV1}, {MountedV2, NewPathV2}} ->
-            case new_state_init(Version, MountedV1, NewPathV1) of
+            case new_state_init(Version, MountedV1, NewPathV1, Root) of
                 {ok, _} = Success ->
                     Success;
                 {error, _} = Error when VersionDefaultRequired =:= false ->
-                    case new_state_init(2, MountedV2, NewPathV2) of
+                    case new_state_init(2, MountedV2, NewPathV2, Root) of
                         {ok, _} = Success ->
                             Success;
                         {error, _} ->
@@ -344,14 +383,14 @@ new_state(1 = Version, VersionDefaultRequired,
             Error
     end;
 new_state(2 = Version, VersionDefaultRequired,
-          PathV1, PathV2, PathMounts) ->
+          PathV1, PathV2, PathMounts, Root) ->
     case new_paths(PathV1, PathV2, PathMounts) of
         {ok, {MountedV1, NewPathV1}, {MountedV2, NewPathV2}} ->
-            case new_state_init(Version, MountedV2, NewPathV2) of
+            case new_state_init(Version, MountedV2, NewPathV2, Root) of
                 {ok, _} = Success ->
                     Success;
                 {error, _} = Error when VersionDefaultRequired =:= false ->
-                    case new_state_init(1, MountedV1, NewPathV1) of
+                    case new_state_init(1, MountedV1, NewPathV1, Root) of
                         {ok, _} = Success ->
                             Success;
                         {error, _} ->
@@ -363,28 +402,31 @@ new_state(2 = Version, VersionDefaultRequired,
         {error, _} = Error ->
             Error
     end;
-new_state(Version, _, _, _, _) ->
+new_state(Version, _, _, _, _, _) ->
     {error, {version_default, Version}}.
 
-new_state_init(Version, true, Path) ->
+new_state_init(Version, true, Path, Root) ->
     {ok, #cloudi_x_cgroups{version = Version,
                   path = Path,
-                  mounted = false}};
-new_state_init(1 = Version, false, Path) ->
+                  mounted = false,
+                  root = Root}};
+new_state_init(1 = Version, false, Path, Root) ->
     case shell("mount -t cgroup none \"~s\"", [Path]) of
         {0, _} ->
             {ok, #cloudi_x_cgroups{version = Version,
                           path = Path,
-                          mounted = true}};
+                          mounted = true,
+                          root = Root}};
         {Status, Output} ->
             {error, {path_v1, Status, Output}}
     end;
-new_state_init(2 = Version, false, Path) ->
+new_state_init(2 = Version, false, Path, Root) ->
     case shell("mount -t cgroup2 none \"~s\"", [Path]) of
         {0, _} ->
             {ok, #cloudi_x_cgroups{version = Version,
                           path = Path,
-                          mounted = true}};
+                          mounted = true,
+                          root = Root}};
         {Status, Output} ->
             {error, {path_v2, Status, Output}}
     end.
@@ -430,11 +472,12 @@ new_mounts([Mount | MountsL], PathV1, PathV2) ->
     end.
 
 create_cgroup(CGroupPath, OSPids, CGroupParameters,
-              #cloudi_x_cgroups{path = Path} = State) ->
+              #cloudi_x_cgroups{version = Version,
+                       path = Path} = State) ->
     CGroupPathFull = Path ++ CGroupPath,
     case shell("mkdir -p \"~s\"", [CGroupPathFull]) of
         {0, _} ->
-            case create_update(CGroupPathFull, Path) of
+            case create_update(Version, CGroupPathFull, Path) of
                 ok ->
                     update(CGroupPath, OSPids, CGroupParameters, State);
                 {error, _} = Error ->
@@ -444,34 +487,37 @@ create_cgroup(CGroupPath, OSPids, CGroupParameters,
             {error, {mkdir, Status, Output}}
     end.
 
-create_update(CGroupPathFull, Path) ->
-    case create_update_get(CGroupPathFull, "cpuset.cpus", Path) of
+create_update(1, CGroupPathFull, Path) ->
+    case create_update_v1_get(CGroupPathFull, "cpuset.cpus", Path) of
         {ok, CPUS} ->
-            case create_update_get(CGroupPathFull, "cpuset.mems", Path) of
+            case create_update_v1_get(CGroupPathFull, "cpuset.mems", Path) of
                 {ok, MEMS} ->
                     if
                         CPUS =:= undefined,
                         MEMS =:= undefined ->
                             ok;
                         true ->
-                            create_update_set(CGroupPathFull, CPUS, MEMS, Path)
+                            create_update_v1_set(CGroupPathFull,
+                                                 CPUS, MEMS, Path)
                     end;
                 {error, _} = Error ->
                     Error
             end;
         {error, _} = Error ->
             Error
-    end.
+    end;
+create_update(2, _, _) ->
+    ok.
 
-create_update_set(CGroupPathFull, CPUS, MEMS, Path) ->
-    case create_update_set_subpath(CGroupPathFull, CPUS, MEMS, Path) of
+create_update_v1_set(CGroupPathFull, CPUS, MEMS, Path) ->
+    case create_update_v1_set_subpath(CGroupPathFull, CPUS, MEMS, Path) of
         ok ->
             CGroupSubPathFull = CGroupPathFull ++ "/",
-            case create_update_set_value(CGroupSubPathFull,
-                                         "cpuset.cpus", CPUS) of
+            case create_update_v1_set_value(CGroupSubPathFull,
+                                            "cpuset.cpus", CPUS) of
                 ok ->
-                    case create_update_set_value(CGroupSubPathFull,
-                                                 "cpuset.mems", MEMS) of
+                    case create_update_v1_set_value(CGroupSubPathFull,
+                                                    "cpuset.mems", MEMS) of
                         ok ->
                             ok;
                         {error, _} = Error ->
@@ -484,19 +530,20 @@ create_update_set(CGroupPathFull, CPUS, MEMS, Path) ->
             Error
     end.
 
-create_update_set_subpath(CGroupPathFull, CPUS, MEMS, Path) ->
+create_update_v1_set_subpath(CGroupPathFull, CPUS, MEMS, Path) ->
     case subdirectory(CGroupPathFull) of
         Path ->
             ok;
         CGroupSubPathFull ->
-            case create_update_set_subpath(CGroupSubPathFull,
-                                           CPUS, MEMS, Path) of
+            case create_update_v1_set_subpath(CGroupSubPathFull,
+                                              CPUS, MEMS, Path) of
                 ok ->
-                    case create_update_set_value(CGroupSubPathFull,
-                                                 "cpuset.cpus", CPUS) of
+                    case create_update_v1_set_value(CGroupSubPathFull,
+                                                    "cpuset.cpus", CPUS) of
                         ok ->
-                            case create_update_set_value(CGroupSubPathFull,
-                                                         "cpuset.mems", MEMS) of
+                            case create_update_v1_set_value(CGroupSubPathFull,
+                                                            "cpuset.mems",
+                                                            MEMS) of
                                 ok ->
                                     ok;
                                 {error, _} = Error ->
@@ -510,9 +557,9 @@ create_update_set_subpath(CGroupPathFull, CPUS, MEMS, Path) ->
             end
     end.
 
-create_update_set_value(_, _, undefined) ->
+create_update_v1_set_value(_, _, undefined) ->
     ok;
-create_update_set_value(CGroupSubPathFull, SubsystemParameter, Value) ->
+create_update_v1_set_value(CGroupSubPathFull, SubsystemParameter, Value) ->
     case shell("cat \"~s~s\"", [CGroupSubPathFull, SubsystemParameter]) of
         {0, OldValue} ->
             NeedsUpdate = strip(shell_output_string(OldValue)) == "",
@@ -537,7 +584,7 @@ create_update_set_value(CGroupSubPathFull, SubsystemParameter, Value) ->
             {error, {ErrorReason, Status, Output}}
     end.
 
-create_update_get(CGroupPathFull, SubsystemParameter, Path) ->
+create_update_v1_get(CGroupPathFull, SubsystemParameter, Path) ->
     case shell("cat \"~s/~s\"", [CGroupPathFull, SubsystemParameter]) of
         {0, Contents} ->
             NeedsUpdate = strip(shell_output_string(Contents)) == "",
@@ -564,32 +611,19 @@ delete_recursive_subpath(CGroupSubPathFull, Path) ->
             ok
     end.
 
-update_parameters(CGroupParameters, 1, CGroupPathFull, _) ->
+update_parameters(1, _, _, _, false) ->
+    {error, root_required};
+update_parameters(1, CGroupParameters, CGroupPathFull, _, true) ->
     update_parameters(CGroupParameters, CGroupPathFull);
-update_parameters(CGroupParameters, 2, CGroupPathFull, Path) ->
+update_parameters(2, CGroupParameters, CGroupPathFull, Path, Root) ->
     Controllers = lists:usort([subsystem(SubsystemParameter)
                                || {SubsystemParameter, _} <- CGroupParameters]),
     ControlAdded = ["+" ++ Controller || Controller <- Controllers],
-    ControlRemoved = ["-" ++ Controller || Controller <- Controllers],
-    CGroupSubPathFull = subdirectory(CGroupPathFull),
-    case subtree_control_add(CGroupSubPathFull,
+    case subtree_control_add(subdirectory(CGroupPathFull, Path),
                              lists:join($ , ControlAdded),
-                             Path) of
+                             Path, Root) of
         ok ->
-            Result = update_parameters(CGroupParameters, CGroupPathFull),
-            case subtree_control_remove(CGroupSubPathFull,
-                                        lists:join($ , ControlRemoved),
-                                        Path) of
-                ok ->
-                    Result;
-                {error, _} = Error ->
-                    if
-                        Result =:= ok ->
-                            Error;
-                        true ->
-                            Result
-                    end
-            end;
+            update_parameters(CGroupParameters, CGroupPathFull);
         {error, _} = Error ->
             Error
     end.
@@ -606,8 +640,25 @@ update_parameters([{SubsystemParameter, Value} | CGroupParameters],
             {error, {subsystem_parameter, Status, Output}}
     end.
 
-update_pids(OSPids, _, CGroupPathFull) ->
-    update_pids(OSPids, CGroupPathFull).
+update_pids(OSPids, CGroupPathFull, true) ->
+    update_pids(OSPids, CGroupPathFull);
+update_pids(OSPids, CGroupPathFull, false) ->
+    case shell("cat \"~s/cgroup.procs\"",
+               [CGroupPathFull]) of
+        {0, OSPidsUpdated} ->
+            OSPidsUpdatedL = split(shell_output_string(OSPidsUpdated), "\n"),
+            OSPidsSet = sets:from_list([erlang:integer_to_list(OSPid)
+                                        || OSPid <- OSPids]),
+            Updated = sets:is_subset(OSPidsSet, sets:from_list(OSPidsUpdatedL)),
+            if
+                Updated =:= true ->
+                    ok;
+                Updated =:= false ->
+                    {error, root_required}
+            end;
+        {Status, Output} ->
+            {error, {procs, Status, Output}}
+    end.
 
 update_pids([], _) ->
     ok;
@@ -657,28 +708,22 @@ subsystem_parameter_error_reason(SubsystemParameter, Suffix) ->
         end
     end, SubsystemParameter) ++ Suffix).
 
-subtree_control_add(Path, Value, Path) ->
-    subtree_control_set(Path, Value);
-subtree_control_add(CGroupSubPathFull, Value, Path) ->
+subtree_control_add(Path, Value, Path, Root) ->
+    if
+        Root =:= true ->
+            subtree_control_set(Path, Value);
+        Root =:= false ->
+            ok
+    end;
+subtree_control_add(CGroupSubPathFull, Value, Path, Root) ->
     case subtree_control_add(subdirectory(CGroupSubPathFull),
-                             Value, Path) of
+                             Value, Path, Root) of
         ok ->
             subtree_control_set(CGroupSubPathFull, Value);
         {error, _} = Error ->
             Error
     end.
 
-subtree_control_remove(Path, Value, Path) ->
-    subtree_control_set(Path, Value);
-subtree_control_remove(CGroupSubPathFull, Value, Path) ->
-    case subtree_control_set(CGroupSubPathFull, Value) of
-        ok ->
-            subtree_control_remove(subdirectory(CGroupSubPathFull),
-                                   Value, Path);
-        {error, _} = Error ->
-            Error
-    end.
-
 subtree_control_set(CGroupSubPathFull, Value) ->
     case shell("echo \"~s\" > \"~scgroup.subtree_control\"",
                [Value, CGroupSubPathFull]) of
@@ -688,6 +733,11 @@ subtree_control_set(CGroupSubPathFull, Value) ->
             {error, {subtree_control, Status, Output}}
     end.
 
+subdirectory(Path, Path) ->
+    Path;
+subdirectory(Path, _) ->
+    subdirectory(Path).
+
 subdirectory(Path) ->
     case lists:reverse(Path) of
         [$/ | PathRev] ->
