@@ -3,7 +3,7 @@
 //
 // MIT License
 //
-// Copyright (c) 2011-2021 Michael Truog <mjtruog at protonmail dot com>
+// Copyright (c) 2011-2023 Michael Truog <mjtruog at protonmail dot com>
 //
 // Permission is hereby granted, free of charge, to any person obtaining a
 // copy of this software and associated documentation files (the "Software"),
@@ -28,6 +28,7 @@
 #include <cstring>
 #include <errno.h>
 #include <unistd.h>
+#include <fcntl.h>
 #include <arpa/inet.h>
 #include <netinet/in.h>
 #include <netinet/tcp.h>
@@ -121,10 +122,10 @@ namespace
             exec_EPERM,
             exec_ETXTBSY,
             exec_unknown,
-            last_value                     // 78
+            last_value                     // 78 == GEPD::ExitStatus::min
         };
 
-        char const * string(int status)
+        char const * status_to_string(int const status)
         {
             switch (status)
             {
@@ -359,31 +360,40 @@ namespace
                     return "close_EIO";
                 case GEPD::ExitStatus::close_unknown:
                     return "close_unknown";
+                default:
+                    return 0;
+            }
+        }
 
-                // signals
-                case 129:
+        char const * signal_to_string(int const signal)
+        {
+            // only signals consistent among all platforms
+            // use a specific string
+            switch (signal)
+            {
+                case 1:
                     return "SIGHUP";
-                case 130:
+                case 2:
                     return "SIGINT";
-                case 131:
+                case 3:
                     return "SIGQUIT";
-                case 132:
+                case 4:
                     return "SIGILL";
-                case 133:
+                case 5:
                     return "SIGTRAP";
-                case 134:
+                case 6:
                     return "SIGABRT";
-                case 136:
+                case 8:
                     return "SIGFPE";
-                case 137:
+                case 9:
                     return "SIGKILL";
-                case 139:
+                case 11:
                     return "SIGSEGV";
-                case 141:
+                case 13:
                     return "SIGPIPE";
-                case 142:
+                case 14:
                     return "SIGALRM";
-                case 143:
+                case 15:
                     return "SIGTERM";
                 default:
                     return 0;
@@ -659,26 +669,36 @@ namespace
                     if (WIFEXITED(status))
                     {
                         status = WEXITSTATUS(status);
+                        if (status)
+                        {
+                            // the exit status is specific to the
+                            // external service's source code
+                            std::cerr << "OS pid " << m_pid <<
+                                " exited with " << status << std::endl;
+                        }
                     }
                     else if (WIFSIGNALED(status))
                     {
-                        status = WTERMSIG(status) + 128;
+                        int const signal = WTERMSIG(status);
+                        char const * const signal_name =
+                            spawn_status::signal_to_string(signal);
+                        if (signal_name)
+                        {
+                            std::cerr << "OS pid " << m_pid <<
+                                " exited with " <<
+                                signal_name << std::endl;
+                        }
+                        else
+                        {
+                            std::cerr << "OS pid " << m_pid <<
+                                " exited with " <<
+                                "SIG#" << signal << std::endl;
+                        }
                     }
                     else
                     {
                         assert(false);
                     }
-                    if (status != 0)
-                    {
-                        char const * const error = spawn_status::string(status);
-                        std::cerr << "OS pid " << m_pid << " exited with ";
-                        if (error)
-                            std::cerr << error << std::endl;
-                        else if (status > 128)
-                            std::cerr << "SIG#" << (status - 128) << std::endl;
-                        else
-                            std::cerr << status << std::endl;
-                    }
                 }
             }
 
@@ -807,6 +827,13 @@ namespace
         return status;
     }
 
+    void fork_exit(int const status, int const fd)
+    {
+        unsigned char const fork_status = status;
+        ::write(fd, &fork_status, 1);
+        ::_exit(status);
+    }
+
 } // anonymous namespace
 
 bool terminate_now()
@@ -853,34 +880,50 @@ int32_t spawn(char protocol,
     }
     else
     {
-        return spawn_status::invalid_input;
+        ::exit(spawn_status::invalid_input);
     }
+    int fds_fork_exit[2] = {-1, -1};
     int fds_stdout[2] = {-1, -1};
     int fds_stderr[2] = {-1, -1};
+    if (::pipe(fds_fork_exit) == -1)
+        ::exit(spawn_status::errno_pipe());
+    if (static_cast<uint32_t>(fds_fork_exit[1]) < ports_len + 3)
+    {
+        // ensure the fork_exit fd doesn't conflict with a thread's socket fd
+        if (::dup2(fds_fork_exit[1], ports_len + 3) == -1)
+            ::exit(spawn_status::errno_dup());
+        if (::close(fds_fork_exit[1]) == -1)
+            ::exit(spawn_status::errno_close());
+        fds_fork_exit[1] = ports_len + 3;
+    }
     if (::pipe(fds_stdout) == -1)
-        return spawn_status::errno_pipe();
+        ::exit(spawn_status::errno_pipe());
     if (::pipe(fds_stderr) == -1)
-        return spawn_status::errno_pipe();
-    pid_t const pid = fork();
+        ::exit(spawn_status::errno_pipe());
+    pid_t const pid = ::fork();
     if (pid == -1)
     {
-        return spawn_status::errno_fork();
+        ::exit(spawn_status::errno_fork());
     }
     else if (pid == 0)
     {
+        if (::close(fds_fork_exit[0]) == -1)
+            ::_exit(spawn_status::errno_close());
+        if (::fcntl(fds_fork_exit[1], F_SETFD, FD_CLOEXEC) == -1)
+            ::_exit(spawn_status::errno_pipe());
         for (size_t i = 0; i < GEPD::nfds; ++i)
         {
             if (::close(GEPD::fds[i].fd) == -1)
-                ::_exit(spawn_status::errno_close());
+                fork_exit(spawn_status::errno_close(), fds_fork_exit[1]);
         }
         if (::dup2(fds_stdout[1], 1) == -1)
-            ::_exit(spawn_status::errno_dup());
+            fork_exit(spawn_status::errno_dup(), fds_fork_exit[1]);
         if (::close(fds_stdout[0]) == -1 || close(fds_stdout[1]) == -1)
-            ::_exit(spawn_status::errno_close());
+            fork_exit(spawn_status::errno_close(), fds_fork_exit[1]);
         if (::dup2(fds_stderr[1], 2) == -1)
-            ::_exit(spawn_status::errno_dup());
+            fork_exit(spawn_status::errno_dup(), fds_fork_exit[1]);
         if (::close(fds_stderr[0]) == -1 || close(fds_stderr[1]) == -1)
-            ::_exit(spawn_status::errno_close());
+            fork_exit(spawn_status::errno_close(), fds_fork_exit[1]);
 
         char pid_message[1024];
         int pid_message_index = 0;
@@ -888,13 +931,13 @@ int32_t spawn(char protocol,
             pid_message_index = 4;
         unsigned long const pid_child = ::getpid();
         if (::ei_encode_version(pid_message, &pid_message_index))
-            ::_exit(GEPD::ExitStatus::ei_encode_error);
+            fork_exit(GEPD::ExitStatus::ei_encode_error, fds_fork_exit[1]);
         if (::ei_encode_tuple_header(pid_message, &pid_message_index, 2))
-            ::_exit(GEPD::ExitStatus::ei_encode_error);
+            fork_exit(GEPD::ExitStatus::ei_encode_error, fds_fork_exit[1]);
         if (::ei_encode_atom(pid_message, &pid_message_index, "pid"))
-            ::_exit(GEPD::ExitStatus::ei_encode_error);
+            fork_exit(GEPD::ExitStatus::ei_encode_error, fds_fork_exit[1]);
         if (::ei_encode_ulong(pid_message, &pid_message_index, pid_child))
-            ::_exit(GEPD::ExitStatus::ei_encode_error);
+            fork_exit(GEPD::ExitStatus::ei_encode_error, fds_fork_exit[1]);
         if (use_header)
         {
             int pid_message_length = pid_message_index - 4;
@@ -908,22 +951,22 @@ int32_t spawn(char protocol,
         {
             int sockfd = ::socket(domain, type, 0);
             if (sockfd == -1)
-                ::_exit(spawn_status::errno_socket());
+                fork_exit(spawn_status::errno_socket(), fds_fork_exit[1]);
             if (domain == PF_INET && type == SOCK_STREAM)
             {
                 int const tcp_nodelay_flag = 1;
                 // set TCP_NODELAY to turn off Nagle's algorithm
                 if (::setsockopt(sockfd, IPPROTO_TCP, TCP_NODELAY,
                                  &tcp_nodelay_flag, sizeof(int)) == -1)
-                    ::_exit(spawn_status::socket_unknown);
+                    fork_exit(spawn_status::socket_unknown, fds_fork_exit[1]);
             }
 
             if (static_cast<size_t>(sockfd) != i + 3)
             {
                 if (::dup2(sockfd, i + 3) == -1)
-                    ::_exit(spawn_status::errno_dup());
+                    fork_exit(spawn_status::errno_dup(), fds_fork_exit[1]);
                 if (::close(sockfd) == -1)
-                    ::_exit(spawn_status::errno_close());
+                    fork_exit(spawn_status::errno_close(), fds_fork_exit[1]);
                 sockfd = i + 3;
             }
             
@@ -939,7 +982,7 @@ int32_t spawn(char protocol,
                 if (::connect(sockfd,
                               reinterpret_cast<struct sockaddr *>(&localhost),
                               sizeof(localhost)) == -1)
-                    ::_exit(spawn_status::errno_connect());
+                    fork_exit(spawn_status::errno_connect(), fds_fork_exit[1]);
             }
             else if (domain == PF_LOCAL)
             {
@@ -958,7 +1001,7 @@ int32_t spawn(char protocol,
                 if (::connect(sockfd,
                               reinterpret_cast<struct sockaddr *>(&local),
                               sizeof(local)) == -1)
-                    ::_exit(spawn_status::errno_connect());
+                    fork_exit(spawn_status::errno_connect(), fds_fork_exit[1]);
             }
             else
             {
@@ -970,7 +1013,7 @@ int32_t spawn(char protocol,
                 // let the first connection get a pid message for attempting
                 // to kill the OS process when the Erlang process terminates
                 if (::write(sockfd, pid_message, pid_message_index) == -1)
-                    ::_exit(spawn_status::errno_write());
+                    fork_exit(spawn_status::errno_write(), fds_fork_exit[1]);
             }
         }
 
@@ -985,37 +1028,37 @@ int32_t spawn(char protocol,
 
         if (owner_get(user_i, user_str, user_str_len,
                       group_i, group_str, group_str_len))
-            ::_exit(spawn_status::invalid_input);
+            fork_exit(spawn_status::invalid_input, fds_fork_exit[1]);
         if (chroot_directory_len > 1)
         {
 #if defined(HAVE_CHROOT)
             if (::chroot(chroot_directory))
-                ::_exit(spawn_status::invalid_input);
+                fork_exit(spawn_status::invalid_input, fds_fork_exit[1]);
             if (directory_len == 1)
             {
                 if (::chdir("/"))
-                    ::_exit(spawn_status::invalid_input);
+                    fork_exit(spawn_status::invalid_input, fds_fork_exit[1]);
             }
 #else
             assert(chroot_directory);
-            ::_exit(spawn_status::invalid_input);
+            fork_exit(spawn_status::invalid_input, fds_fork_exit[1]);
 #endif
         }
         if (rlimit(rlimits, rlimits_len))
-            ::_exit(spawn_status::invalid_input);
+            fork_exit(spawn_status::invalid_input, fds_fork_exit[1]);
         if (nice != 0)
         {
             errno = 0;
             if (::nice(nice)) {} // ignore invalid -1 result
             if (errno != 0)
-                ::_exit(spawn_status::invalid_input);
+                fork_exit(spawn_status::invalid_input, fds_fork_exit[1]);
         }
         if (owner_set(user_i, group_i))
-            ::_exit(spawn_status::invalid_input);
+            fork_exit(spawn_status::invalid_input, fds_fork_exit[1]);
         if (directory_len > 1)
         {
             if (::chdir(directory))
-                ::_exit(spawn_status::invalid_input);
+                fork_exit(spawn_status::invalid_input, fds_fork_exit[1]);
         }
         if (syscall_lock_len > 1)
         {
@@ -1025,36 +1068,62 @@ int32_t spawn(char protocol,
             strings_set(syscall_names, syscall_lock_count,
                         syscall_lock, syscall_lock_len);
             if (syscall_lock_set(syscall_names))
-                ::_exit(spawn_status::invalid_input);
+                fork_exit(spawn_status::invalid_input, fds_fork_exit[1]);
             delete [] syscall_names;
         }
 
         ::execve(filename, execve_argv, execve_env);
-        ::_exit(spawn_status::errno_exec());
+        fork_exit(spawn_status::errno_exec(), fds_fork_exit[1]);
     }
     else
     {
+        if (::close(fds_fork_exit[1]) == -1)
+            ::exit(spawn_status::errno_close());
         if (::close(fds_stdout[1]) == -1)
-            return spawn_status::errno_close();
+            ::exit(spawn_status::errno_close());
         if (::close(fds_stderr[1]) == -1)
-            return spawn_status::errno_close();
+            ::exit(spawn_status::errno_close());
 
-        if (GEPD::fds.reserve(GEPD::nfds + 2) == false)
-            ::exit(spawn_status::out_of_memory);
-        size_t const index_stdout = GEPD::nfds;
-        size_t const index_stderr = GEPD::nfds + 1;
-        GEPD::fds[index_stdout].fd = fds_stdout[0];
-        GEPD::fds[index_stdout].events = POLLIN | POLLPRI;
-        GEPD::fds[index_stdout].revents = 0;
-        GEPD::fds[index_stderr].fd = fds_stderr[0];
-        GEPD::fds[index_stderr].events = POLLIN | POLLPRI;
-        GEPD::fds[index_stderr].revents = 0;
-        GEPD::nfds += 2;
+        // wait for execve to complete
+        unsigned char fork_status = 0;
+        if (::read(fds_fork_exit[0], &fork_status, 1) == 1)
+        {
+            // the forked process failed before or during execve
+            char const * const fork_error =
+                spawn_status::status_to_string(fork_status);
+            if (fork_error)
+            {
+                std::cerr << "OS pid " << pid << " spawn error " <<
+                    fork_error << std::endl;
+            }
+            else
+            {
+                std::cerr << "OS pid " << pid << " spawn error " <<
+                    static_cast<int>(fork_status) << std::endl;
+            }
+        }
+        else
+        {
+            // execve did not fail
+            if (GEPD::fds.reserve(GEPD::nfds + 2) == false)
+                ::exit(spawn_status::out_of_memory);
+            size_t const index_stdout = GEPD::nfds;
+            size_t const index_stderr = GEPD::nfds + 1;
+            GEPD::fds[index_stdout].fd = fds_stdout[0];
+            GEPD::fds[index_stdout].events = POLLIN | POLLPRI;
+            GEPD::fds[index_stdout].revents = 0;
+            GEPD::fds[index_stderr].fd = fds_stderr[0];
+            GEPD::fds[index_stderr].events = POLLIN | POLLPRI;
+            GEPD::fds[index_stderr].revents = 0;
+            GEPD::nfds += 2;
 
-        copy_ptr<process_data> P(new process_data(pid,
-                                                  index_stdout,
-                                                  index_stderr));
-        processes.push_back(P);
+            copy_ptr<process_data> P(new process_data(pid,
+                                                      index_stdout,
+                                                      index_stderr));
+            processes.push_back(P);
+        }
+        if (::close(fds_fork_exit[0]) == -1)
+            ::exit(spawn_status::errno_close());
     }
     return pid;
 }
