@@ -7,7 +7,7 @@
 //
 // MIT License
 //
-// Copyright (c) 2009-2017 Michael Truog <mjtruog at gmail dot com>
+// Copyright (c) 2009-2023 Michael Truog <mjtruog at protonmail dot com>
 //
 // Permission is hereby granted, free of charge, to any person obtaining a
 // copy of this software and associated documentation files (the "Software"),
@@ -42,7 +42,6 @@
 #include <boost/preprocessor/seq/size.hpp>
 #include <boost/preprocessor/seq/elem.hpp>
 #include <boost/preprocessor/seq/transform.hpp>
-//#include <boost/preprocessor/tuple/to_seq.hpp> // broken with boost >= 1.5?
 #include <boost/preprocessor/tuple/to_list.hpp>
 #include <boost/preprocessor/tuple/elem.hpp>
 #include <boost/preprocessor/list/for_each.hpp>
@@ -784,6 +783,12 @@ case BOOST_PP_DEC(I):\
         return GEPD::ExitStatus::success;
     }
 
+    bool orphaned(unsigned long const ppid)
+    {
+        unsigned long const ppid_now = getppid();
+        return (ppid_now != ppid);
+    }
+
     enum
     {
         INDEX_STDOUT = 0,
@@ -805,37 +810,46 @@ int GEPD::consume_stream(int fd, short & revents,
         return GEPD::ExitStatus::poll_NVAL;
     revents = 0;
 
+    bool flush = false;
+    size_t i_flush = 0;
     ssize_t left = stream.size() - i;
-    ssize_t readBytes;
-    while ((readBytes = read(fd, &stream[i], left)) == left &&
-           stream.grow())
+    if (left == 0)
     {
-        i += left;
-        left = stream.size() - i;
-        bool ready = true;
-        data_ready(fd, ready);
-        if (ready == false)
-            break;
+        // stream is full, flush it
+        flush = true;
+        i_flush = i - 1;
     }
-    if (readBytes == 0 && i == 0)
-        return GEPD::ExitStatus::success;
-    else if (readBytes == -1)
-        return errno_read();
-    i += readBytes; // i is the next index to read at, always
-
-    // only send stderr output before the last newline character
-    bool foundNewline = false;
-    size_t iNewline = 0;
-    for (ssize_t j = i - 1; ! foundNewline && j >= 0; --j)
+    else
     {
-        if (stream[j] == '\n')
+        ssize_t read_bytes;
+        while ((read_bytes = read(fd, &stream[i], left)) == left &&
+               stream.grow())
         {
-            foundNewline = true;
-            iNewline = j;
+            i += left;
+            left = stream.size() - i;
+            bool ready = true;
+            data_ready(fd, ready);
+            if (ready == false)
+                break;
+        }
+        if (read_bytes == 0 && i == 0)
+            return GEPD::ExitStatus::success;
+        else if (read_bytes == -1)
+            return errno_read();
+        i += read_bytes; // i is the next index to read at, always
+
+        // send stdout/stderr output before the last newline character
+        for (ssize_t j = i - 1; ! flush && j >= 0; --j)
+        {
+            if (stream[j] == '\n')
+            {
+                flush = true;
+                i_flush = j;
+            }
         }
     }
 
-    if (foundNewline)
+    if (flush)
     {
         int index = sizeof(OUTPUT_PREFIX_TYPE);
         if (ei_encode_version(send_buffer.get<char>(), &index))
@@ -846,25 +860,25 @@ int GEPD::consume_stream(int fd, short & revents,
             return GEPD::ExitStatus::ei_encode_error;
         if (ei_encode_ulong(send_buffer.get<char>(), &index, pid))
             return GEPD::ExitStatus::ei_encode_error;
-        if (send_buffer.reserve(index + (iNewline + 1) + 1) == false)
+        if (send_buffer.reserve(index + (i_flush + 1) + 1) == false)
             return GEPD::ExitStatus::write_overflow;
         if (ei_encode_string_len(send_buffer.get<char>(), &index,
-                                 stream.get<char>(), iNewline + 1))
+                                 stream.get<char>(), i_flush + 1))
             return GEPD::ExitStatus::ei_encode_error;
         int status;
         if ((status = write_cmd(send_buffer, index -
                                 sizeof(OUTPUT_PREFIX_TYPE))))
             return status;
         // keep any data not yet sent (waiting for a newline)
-        if (iNewline == i - 1)
+        if (i_flush == i - 1)
         {
             i = 0;
         }
         else
         {
-            size_t const remainingBytes = i - iNewline - 1;
-            stream.move(iNewline + 1, remainingBytes, 0);
-            i = remainingBytes;
+            size_t const remaining_bytes = i - i_flush - 1;
+            stream.move(i_flush + 1, remaining_bytes, 0);
+            i = remaining_bytes;
         }
     }
     return GEPD::ExitStatus::success;
@@ -879,8 +893,9 @@ int GEPD::flush_stream(int fd, short revents,
         return GEPD::ExitStatus::success;
 
     ssize_t left = stream.size() - i;
-    ssize_t readBytes;
-    while ((readBytes = read(fd, &stream[i], left)) == left &&
+    assert(left > 0);
+    ssize_t read_bytes;
+    while ((read_bytes = read(fd, &stream[i], left)) == left &&
            stream.grow())
     {
         i += left;
@@ -890,10 +905,10 @@ int GEPD::flush_stream(int fd, short revents,
         if (ready == false)
             break;
     }
-    if (readBytes == 0 && i == 0)
+    if (read_bytes == 0 && i == 0)
         return GEPD::ExitStatus::success;
-    else if (readBytes != -1)
-        i += readBytes; // i is the next index to read at, always
+    else if (read_bytes != -1)
+        i += read_bytes; // i is the next index to read at, always
 
     size_t const total = i - 1;
     i = 0;
@@ -930,7 +945,6 @@ nfds_t GEPD::nfds = 0;
 
 int GEPD::default_main()
 {
-    int const timeout = -1; // milliseconds
     // use the option {packet, 4} for open_port/2
     // (limited by 4MB buffer size below)
     realloc_ptr<unsigned char> buffer(32768, 4194304);
@@ -940,7 +954,7 @@ int GEPD::default_main()
     if ((status = GEPD::init()))
         return status;
     int count;
-    return GEPD::wait(count, timeout, buffer, stream1, stream2);
+    return GEPD::wait(count, buffer, stream1, stream2);
 }
 
 int GEPD::init()
@@ -964,6 +978,29 @@ int GEPD::init()
     return GEPD::ExitStatus::success;
 }
 
+int GEPD::wait(int & count,
+               realloc_ptr<unsigned char> & buffer,
+               realloc_ptr<unsigned char> & stream1,
+               realloc_ptr<unsigned char> & stream2)
+{
+    static unsigned long const ppid = getppid();
+    // based on TIMEOUT_TERMINATE_MAX in cloudi_core_i_constants.hrl
+    int const timeout = 60000; // milliseconds
+    int status;
+    while ((status = GEPD::wait(count, timeout, buffer,
+                                stream1, stream2)) == GEPD::ExitStatus::timeout)
+    {
+        if (orphaned(ppid))
+            return GEPD::ExitStatus::erlang_exit;
+    }
+    if (status == GEPD::ExitStatus::ready)
+    {
+        if (orphaned(ppid))
+            status = GEPD::ExitStatus::erlang_exit;
+    }
+    return status;
+}
+
 int GEPD::wait(int & count, int const timeout,
                realloc_ptr<unsigned char> & buffer,
                realloc_ptr<unsigned char> & stream1,
