@@ -9,7 +9,7 @@
 %%%
 %%% MIT License
 %%%
-%%% Copyright (c) 2015-2021 Michael Truog <mjtruog at protonmail dot com>
+%%% Copyright (c) 2015-2022 Michael Truog <mjtruog at protonmail dot com>
 %%%
 %%% Permission is hereby granted, free of charge, to any person obtaining a
 %%% copy of this software and associated documentation files (the "Software"),
@@ -30,8 +30,8 @@
 %%% DEALINGS IN THE SOFTWARE.
 %%%
 %%% @author Michael Truog <mjtruog at protonmail dot com>
-%%% @copyright 2015-2021 Michael Truog
-%%% @version 2.0.3 {@date} {@time}
+%%% @copyright 2015-2022 Michael Truog
+%%% @version 2.0.5 {@date} {@time}
 %%%------------------------------------------------------------------------
 
 -module(cloudi_args_type).
@@ -41,15 +41,28 @@
 -export([function_required/2,
          function_required_pick/2,
          function_optional/2,
+         period_to_milliseconds/3,
+         period_to_milliseconds/4,
+         period_to_seconds/2,
          priority/1,
          service_name/1,
-         service_name_suffix/2,
+         service_name_pattern/1,
          service_name_pattern_suffix/2,
-         timeout_milliseconds/1]).
+         service_name_suffix/2,
+         timeout_milliseconds/1,
+         timeout_period/1,
+         timeout_period_to_milliseconds/1]).
 
 -include("cloudi_logger.hrl").
 -include("cloudi_core_i_constants.hrl").
 
+-type period() :: non_neg_integer() | limit_min | limit_max |
+                  {pos_integer(), seconds | second} |
+                  {pos_integer(), minutes | minute} |
+                  {pos_integer(), hours | hour} |
+                  {pos_integer(), days | day}.
+-export_type([period/0]).
+
 %%%------------------------------------------------------------------------
 %%% External interface functions
 %%%------------------------------------------------------------------------
@@ -139,6 +152,115 @@ function_optional(undefined, _) ->
 function_optional(Function, Arity) ->
     function_required(Function, Arity).
 
+-spec period_to_milliseconds(Value :: period(),
+                             Min :: non_neg_integer(),
+                             Max :: pos_integer()) ->
+    non_neg_integer().
+
+period_to_milliseconds(_, Min, _)
+    when not is_integer(Min) ->
+    ?LOG_ERROR_SYNC("invalid period min: ~tp", [Min]),
+    erlang:exit(badarg);
+period_to_milliseconds(_, _, Max)
+    when not is_integer(Max) ->
+    ?LOG_ERROR_SYNC("invalid period max: ~tp", [Max]),
+    erlang:exit(badarg);
+period_to_milliseconds(_, Min, Max)
+    when not (Min < Max) ->
+    ?LOG_ERROR_SYNC("invalid period min/max: ~w >= ~w", [Min, Max]),
+    erlang:exit(badarg);
+period_to_milliseconds(limit_min, Min, _) ->
+    Min;
+period_to_milliseconds(limit_max, _, Max) ->
+    Max;
+period_to_milliseconds(Value, Min, Max)
+    when is_integer(Value), Value >= 0 ->
+    if
+        Value < Min ->
+            ?LOG_ERROR_SYNC("period ~w < ~w ", [Value, Min]),
+            erlang:exit(badarg);
+        Value > Max ->
+            ?LOG_ERROR_SYNC("period ~w > ~w ", [Value, Max]),
+            erlang:exit(badarg);
+        true ->
+            Value
+    end;
+period_to_milliseconds({Multiplier, Unit} = Value, Min, Max)
+    when is_integer(Multiplier), Multiplier >= 1 ->
+    ValueMilliSeconds = if
+        Unit =:= seconds orelse Unit =:= second ->
+            Multiplier * 1000;
+        Unit =:= minutes orelse Unit =:= minute ->
+            Multiplier * 60000;
+        Unit =:= hours orelse Unit =:= hour ->
+            Multiplier * 3600000;
+        Unit =:= days orelse Unit =:= day ->
+            Multiplier * 86400000;
+        true ->
+            ?LOG_ERROR_SYNC("invalid period unit: ~tp", [Unit]),
+            erlang:exit(badarg)
+    end,
+    if
+        ValueMilliSeconds < Min ->
+            ?LOG_ERROR_SYNC("period ~w < ~w ", [Value, Min]),
+            erlang:exit(badarg);
+        ValueMilliSeconds > Max ->
+            ?LOG_ERROR_SYNC("period ~w > ~w ", [Value, Max]),
+            erlang:exit(badarg);
+        true ->
+            ValueMilliSeconds
+    end;
+period_to_milliseconds(Value, _, _) ->
+    ?LOG_ERROR_SYNC("invalid period: ~tp", [Value]),
+    erlang:exit(badarg).
+
+-spec period_to_milliseconds(Value :: period() | undefined,
+                             Min :: non_neg_integer(),
+                             Max :: pos_integer(),
+                             Default :: period()) ->
+    non_neg_integer().
+
+period_to_milliseconds(undefined, Min, Max, Default) ->
+    period_to_milliseconds(Default, Min, Max);
+period_to_milliseconds(Value, Min, Max, _) ->
+    period_to_milliseconds(Value, Min, Max).
+
+-spec period_to_seconds(Value :: cloudi_service_api:period_gte(),
+                        Min :: non_neg_integer()) ->
+    cloudi_service_api:seconds().
+
+period_to_seconds(_, Min)
+    when not is_integer(Min) ->
+    ?LOG_ERROR_SYNC("invalid period min: ~tp", [Min]),
+    erlang:exit(badarg);
+period_to_seconds(limit_min, Min) ->
+    Min;
+period_to_seconds(Value, Min)
+    when is_integer(Value), Value >= 0 ->
+    if
+        Value < Min ->
+            ?LOG_ERROR_SYNC("period ~w < ~w ", [Value, Min]),
+            erlang:exit(badarg);
+        true ->
+            Value
+    end;
+period_to_seconds({Multiplier, Unit}, _)
+    when is_integer(Multiplier), Multiplier >= 1 ->
+    if
+        Unit =:= minutes orelse Unit =:= minute ->
+            Multiplier * 60;
+        Unit =:= hours orelse Unit =:= hour ->
+            Multiplier * 3600;
+        Unit =:= days orelse Unit =:= day ->
+            Multiplier * 86400;
+        true ->
+            ?LOG_ERROR_SYNC("invalid period unit: ~tp", [Unit]),
+            erlang:exit(badarg)
+    end;
+period_to_seconds(Value, _) ->
+    ?LOG_ERROR_SYNC("invalid period: ~tp", [Value]),
+    erlang:exit(badarg).
+
 -spec priority(Priority :: cloudi:priority()) ->
     true.
 
@@ -169,6 +291,42 @@ service_name(Name) ->
     ?LOG_ERROR_SYNC("invalid service name: ~tp", [Name]),
     erlang:exit(badarg).
 
+-spec service_name_pattern(Pattern :: cloudi:service_name_pattern()) ->
+    true.
+
+service_name_pattern([PatternC | _] = Pattern)
+    when is_integer(PatternC) ->
+    _ = cloudi_x_trie:is_pattern2_bytes(Pattern),
+    true;
+service_name_pattern(Pattern) ->
+    ?LOG_ERROR_SYNC("invalid service name pattern: ~tp", [Pattern]),
+    erlang:exit(badarg).
+
+-spec service_name_pattern_suffix(Prefix :: cloudi:service_name_pattern(),
+                                  Pattern :: cloudi:service_name_pattern()) ->
+    cloudi:service_name_pattern_suffix().
+
+service_name_pattern_suffix([PrefixC | _] = Prefix, [PatternC | _] = Pattern)
+    when is_integer(PrefixC), is_integer(PatternC) ->
+    case suffix_pattern_parse(Prefix, Pattern) of
+        error ->
+            ?LOG_ERROR_SYNC("prefix service name pattern mismatch: "
+                            "\"~ts\" \"~ts\"",
+                            [cloudi_service_name:utf8_forced(Prefix),
+                             cloudi_service_name:utf8_forced(Pattern)]),
+            erlang:exit(badarg);
+        Suffix ->
+            Suffix
+    end;
+service_name_pattern_suffix([PrefixC | _], Pattern)
+    when is_integer(PrefixC) ->
+    ?LOG_ERROR_SYNC("invalid service name pattern: ~tp", [Pattern]),
+    erlang:exit(badarg);
+service_name_pattern_suffix(Prefix, [PatternC | _])
+    when is_integer(PatternC) ->
+    ?LOG_ERROR_SYNC("invalid prefix: ~tp", [Prefix]),
+    erlang:exit(badarg).
+
 -spec service_name_suffix(Prefix :: cloudi:service_name_pattern(),
                           Name :: cloudi:service_name()) ->
     string().
@@ -201,31 +359,6 @@ service_name_suffix(Prefix, [NameC | _])
     ?LOG_ERROR_SYNC("invalid prefix: ~tp", [Prefix]),
     erlang:exit(badarg).
 
--spec service_name_pattern_suffix(Prefix :: cloudi:service_name_pattern(),
-                                  Pattern :: cloudi:service_name_pattern()) ->
-    cloudi:service_name_pattern_suffix().
-
-service_name_pattern_suffix([PrefixC | _] = Prefix, [PatternC | _] = Pattern)
-    when is_integer(PrefixC), is_integer(PatternC) ->
-    case suffix_pattern_parse(Prefix, Pattern) of
-        error ->
-            ?LOG_ERROR_SYNC("prefix service name pattern mismatch: "
-                            "\"~ts\" \"~ts\"",
-                            [cloudi_service_name:utf8_forced(Prefix),
-                             cloudi_service_name:utf8_forced(Pattern)]),
-            erlang:exit(badarg);
-        Suffix ->
-            Suffix
-    end;
-service_name_pattern_suffix([PrefixC | _], Pattern)
-    when is_integer(PrefixC) ->
-    ?LOG_ERROR_SYNC("invalid service name pattern: ~tp", [Pattern]),
-    erlang:exit(badarg);
-service_name_pattern_suffix(Prefix, [PatternC | _])
-    when is_integer(PatternC) ->
-    ?LOG_ERROR_SYNC("invalid prefix: ~tp", [Prefix]),
-    erlang:exit(badarg).
-
 -spec timeout_milliseconds(Timeout :: cloudi:timeout_milliseconds()) ->
     true.
 
@@ -242,6 +375,63 @@ timeout_milliseconds(Timeout) ->
     ?LOG_ERROR_SYNC("invalid timeout: ~tp", [Timeout]),
     erlang:exit(badarg).
 
+-spec timeout_period(Timeout :: cloudi:timeout_period()) ->
+    true.
+
+timeout_period(undefined) ->
+    true;
+timeout_period(limit_min) ->
+    true;
+timeout_period(limit_max) ->
+    true;
+timeout_period(Timeout)
+    when is_integer(Timeout), Timeout >= 0, Timeout =< ?TIMEOUT_MAX_ERLANG ->
+    true;
+timeout_period({Multiplier, Unit} = Timeout)
+    when is_integer(Multiplier), Multiplier >= 1 ->
+    if
+        Unit =:= seconds orelse Unit =:= second,
+        Multiplier =< ?TIMEOUT_MAX_ERLANG div 1000 ->
+            true;
+        Unit =:= minutes orelse Unit =:= minute,
+        Multiplier =< ?TIMEOUT_MAX_ERLANG div 60000 ->
+            true;
+        Unit =:= hours orelse Unit =:= hour,
+        Multiplier =< ?TIMEOUT_MAX_ERLANG div 3600000 ->
+            true;
+        Unit =:= days orelse Unit =:= day,
+        Multiplier =< ?TIMEOUT_MAX_ERLANG div 86400000 ->
+            true;
+        true ->
+            ?LOG_ERROR_SYNC("invalid timeout_period: ~tp", [Timeout]),
+            erlang:exit(badarg)
+    end;
+timeout_period(Timeout) ->
+    ?LOG_ERROR_SYNC("invalid timeout: ~tp", [Timeout]),
+    erlang:exit(badarg).
+
+-spec timeout_period_to_milliseconds(Timeout :: cloudi:timeout_period()) ->
+    cloudi:timeout_milliseconds().
+
+timeout_period_to_milliseconds(Timeout)
+    when Timeout =:= undefined; Timeout =:= limit_min; Timeout =:= limit_max ->
+    Timeout;
+timeout_period_to_milliseconds(Timeout)
+    when is_integer(Timeout), Timeout >= 0, Timeout =< ?TIMEOUT_MAX_ERLANG ->
+    Timeout;
+timeout_period_to_milliseconds({Multiplier, Unit})
+    when is_integer(Multiplier), Multiplier >= 1 ->
+    if
+        Unit =:= seconds orelse Unit =:= second ->
+            Multiplier * 1000;
+        Unit =:= minutes orelse Unit =:= minute ->
+            Multiplier * 60000;
+        Unit =:= hours orelse Unit =:= hour ->
+            Multiplier * 3600000;
+        Unit =:= days orelse Unit =:= day ->
+            Multiplier * 86400000
+    end.
+
 %%%------------------------------------------------------------------------
 %%% Private functions
 %%%------------------------------------------------------------------------
