@@ -9,7 +9,7 @@
 %%%
 %%% MIT License
 %%%
-%%% Copyright (c) 2011-2022 Michael Truog <mjtruog at protonmail dot com>
+%%% Copyright (c) 2011-2023 Michael Truog <mjtruog at protonmail dot com>
 %%%
 %%% Permission is hereby granted, free of charge, to any person obtaining a
 %%% copy of this software and associated documentation files (the "Software"),
@@ -30,8 +30,8 @@
 %%% DEALINGS IN THE SOFTWARE.
 %%%
 %%% @author Michael Truog <mjtruog at protonmail dot com>
-%%% @copyright 2011-2022 Michael Truog
-%%% @version 2.0.5 {@date} {@time}
+%%% @copyright 2011-2023 Michael Truog
+%%% @version 2.0.6 {@date} {@time}
 %%%------------------------------------------------------------------------
 
 -module(cloudi_core_i_configurator).
@@ -71,7 +71,7 @@
          code_path_remove/2,
          code_path/1,
          code_status/3,
-         service_start/2,
+         service_start/3,
          service_stop/3,
          service_restart/2,
          service_suspend/2,
@@ -278,17 +278,19 @@ code_status(TimeNative, TimeOffset, Timeout) ->
 
 -spec service_start(#config_service_internal{} |
                     #config_service_external{},
+                    Concurrency :: cloudi_core_i_concurrency:state(),
                     Timeout :: pos_integer() | infinity) ->
-    {ok, #config_service_internal{} |
-         #config_service_external{}} |
-    stopped |
+    {ok,
+     #config_service_internal{} | #config_service_external{},
+     cloudi_core_i_concurrency:state()} |
+    {stopped, cloudi_core_i_concurrency:state()} |
     {error, error_reason_service_start()}.
 
 service_start(#config_service_internal{
                   count_process = CountProcess,
                   options = #config_service_options{
                       reload = Reload}} = Service,
-              Timeout) ->
+              Concurrency, Timeout) ->
     case service_start_find_internal(Service, Timeout) of
         {ok, Application, #config_service_internal{
                               module = Module} = FoundService} ->
@@ -305,20 +307,23 @@ service_start(#config_service_internal{
                 Reload =:= false ->
                     ok
             end,
-            CountProcessNew = cloudi_concurrency:count(CountProcess),
+            ProcessCount = cloudi_core_i_concurrency:count(CountProcess,
+                                                           Concurrency),
             service_start_internal(FoundService, GroupLeader,
-                                   CountProcessNew, Timeout);
+                                   ProcessCount, Concurrency, Timeout);
         {error, _} = Error ->
             Error
     end;
 
 service_start(#config_service_external{count_process = CountProcess,
                                        count_thread = CountThread} = Service,
-              Timeout) ->
-    CountProcessNew = cloudi_concurrency:count(CountProcess),
-    CountThreadNew = cloudi_concurrency:count(CountThread),
-    service_start_external(Service, CountThreadNew,
-                           CountProcessNew, Timeout).
+              Concurrency, Timeout) ->
+    ProcessCount = cloudi_core_i_concurrency:count(CountProcess,
+                                                   Concurrency),
+    ThreadCount = cloudi_core_i_concurrency:count(CountThread,
+                                                  Concurrency),
+    service_start_external(Service, ThreadCount, ProcessCount,
+                           Concurrency, Timeout).
 
 -spec service_stop(#config_service_internal{} |
                    #config_service_external{},
@@ -395,9 +400,9 @@ service_update(#config_service_update{
     end.
 
 service_update_external(Pids, Ports, Arguments,
-                        CountThread, CountProcess) ->
+                        ThreadCount, ProcessCount) ->
     service_update_external(0, Pids, Ports, Arguments,
-                            CountThread, CountProcess).
+                            ThreadCount, ProcessCount).
 
 service_process_init_end(Pid)
     when is_pid(Pid) ->
@@ -735,22 +740,26 @@ code_change(_, State, _) ->
 %%% Private functions
 %%%------------------------------------------------------------------------
 
-configure(#config{services = Services} = Config, Timeout) ->
-    case configure_service(Services, Timeout) of
-        {ok, ServicesNew} ->
-            {ok, Config#config{services = ServicesNew}};
+configure(#config{concurrency = Concurrency,
+                  services = Services} = Config, Timeout) ->
+    case configure_service(Services, Concurrency, Timeout) of
+        {ok, ServicesNew, ConcurrencyNew} ->
+            {ok, Config#config{concurrency = ConcurrencyNew,
+                               services = ServicesNew}};
         {error, _} = Error ->
             Error
     end.
 
-configure_service([], Configured, _) ->
-    {ok, lists:reverse(Configured)};
-configure_service([Service | Services], Configured, Timeout) ->
-    case service_start(Service, Timeout) of
-        {ok, ServiceNew} ->
-            configure_service(Services, [ServiceNew | Configured], Timeout);
-        stopped ->
-            configure_service(Services, Configured, Timeout);
+configure_service([], Configured, Concurrency, _) ->
+    {ok, lists:reverse(Configured), Concurrency};
+configure_service([Service | Services], Configured, Concurrency, Timeout) ->
+    case service_start(Service, Concurrency, Timeout) of
+        {ok, ServiceNew, ConcurrencyNew} ->
+            configure_service(Services, [ServiceNew | Configured],
+                              ConcurrencyNew, Timeout);
+        {stopped, ConcurrencyNew} ->
+            configure_service(Services, Configured,
+                              ConcurrencyNew, Timeout);
         {error, Reason} = Error ->
             % wait for logging statements to be logged before crashing
             ?LOG_FATAL_SYNC("configure failed: ~tp~n~tp",
@@ -758,8 +767,8 @@ configure_service([Service | Services], Configured, Timeout) ->
             Error
     end.
 
-configure_service(Services, Timeout) ->
-    configure_service(Services, [], Timeout).
+configure_service(Services, Concurrency, Timeout) ->
+    configure_service(Services, [], Concurrency, Timeout).
 
 service_start_find_internal(#config_service_internal{
                                 module = Module,
@@ -1090,24 +1099,25 @@ service_start_wait_pid([{MonitorRef, Pid} | M], Death) ->
             service_start_wait_pid(M, service_start_wait_pid_death(Death, Info))
     end.
 
-service_start_wait_pids([], undefined, Service) ->
-    {ok, Service};
-service_start_wait_pids([], {stopped, _}, _) ->
+service_start_wait_pids([], undefined, Service, Concurrency) ->
+    {ok, Service, Concurrency};
+service_start_wait_pids([], {stopped, _}, _, Concurrency) ->
     % shutdown | {shutdown, _} is logged in cloudi_core_i_services_monitor and
     % is not helpful as a return value because it will be ignored
-    stopped;
-service_start_wait_pids([], {error, Reason}, Service) ->
+    {stopped, Concurrency};
+service_start_wait_pids([], {error, Reason}, Service, _) ->
     case Service of
         #config_service_internal{} ->
             {error, {service_internal_start_failed, Reason}};
         #config_service_external{} ->
             {error, {service_external_start_failed, Reason}}
     end;
-service_start_wait_pids([M | MonitorPids], Death, Service) ->
+service_start_wait_pids([M | MonitorPids], Death, Service, Concurrency) ->
     service_start_wait_pids(MonitorPids,
-                            service_start_wait_pid(M, Death), Service).
+                            service_start_wait_pid(M, Death),
+                            Service, Concurrency).
 
-service_start_wait(Pids, Service) ->
+service_start_wait(Pids, Service, Concurrency) ->
     % create the monitors on all the service's receiver pids
     MonitorPids = [[{erlang:monitor(process, Pid), Pid} ||
                     Pid <- P] || P <- Pids],
@@ -1117,11 +1127,12 @@ service_start_wait(Pids, Service) ->
     % or the cloudi_service_api:services_add/2 function when a service
     % starts for the first time
     ok = cloudi_core_i_services_monitor:process_init_begin(Pids),
-    service_start_wait_pids(MonitorPids, undefined, Service).
+    service_start_wait_pids(MonitorPids, undefined, Service, Concurrency).
 
-service_start_internal(CountProcess, Pids, Service, _, CountProcess, _) ->
-    service_start_wait(lists:reverse(Pids), Service);
-service_start_internal(IndexProcess, Pids,
+service_start_internal(ProcessCount, Pids, Service, _,
+                       ProcessCount, Concurrency, _) ->
+    service_start_wait(lists:reverse(Pids), Service, Concurrency);
+service_start_internal(ProcessIndex, Pids,
                        #config_service_internal{
                            module = Module,
                            args = Args,
@@ -1136,36 +1147,44 @@ service_start_internal(IndexProcess, Pids,
                            options = #config_service_options{
                                restart_all = RestartAll,
                                restart_delay = RestartDelay,
-                               scope = Scope} = Options,
+                               critical = Critical,
+                               scope = Scope,
+                               bind = Bind} = Options,
                            max_r = MaxR,
                            max_t = MaxT,
                            uuid = ID} = Service, GroupLeader,
-                       CountProcess, Timeout) ->
+                       ProcessCount, Concurrency, Timeout) ->
+    {BindNew,
+     ConcurrencyNew} = cloudi_core_i_concurrency:
+                       bind_assign_process(Bind, ProcessIndex, Concurrency),
     case cloudi_core_i_services_monitor:
          monitor(cloudi_core_i_spawn, start_internal,
                  [GroupLeader,
                   Module, Args, TimeoutInit,
                   Prefix, TimeoutAsync, TimeoutSync, TimeoutTerm,
-                  DestRefresh, DestListDeny,
-                  DestListAllow, Options, ID],
-                 IndexProcess, CountProcess, 1, Scope,
+                  DestRefresh, DestListDeny, DestListAllow,
+                  Options#config_service_options{bind = BindNew}, ID],
+                 ProcessIndex, ProcessCount, 1, Scope,
                  TimeoutTerm, RestartAll, RestartDelay,
-                 MaxR, MaxT, ID, Timeout) of
+                 Critical, MaxR, MaxT, ID, Timeout) of
         {ok, P} ->
             service_format_log(Service, P),
-            service_start_internal(IndexProcess + 1, [P | Pids], Service,
-                                   GroupLeader, CountProcess, Timeout);
+            service_start_internal(ProcessIndex + 1, [P | Pids],
+                                   Service, GroupLeader, ProcessCount,
+                                   ConcurrencyNew, Timeout);
         {error, Reason} ->
             {error, {service_internal_start_failed, Reason}}
     end.
 
-service_start_internal(Service, GroupLeader, CountProcess, Timeout) ->
+service_start_internal(Service, GroupLeader,
+                       ProcessCount, Concurrency, Timeout) ->
     service_start_internal(0, [], Service, GroupLeader,
-                           CountProcess, timeout_decr(Timeout)).
+                           ProcessCount, Concurrency, timeout_decr(Timeout)).
 
-service_start_external(CountProcess, Pids, Service, _, CountProcess, _) ->
-    service_start_wait(lists:reverse(Pids), Service);
-service_start_external(IndexProcess, Pids,
+service_start_external(ProcessCount, Pids, Service, _,
+                       ProcessCount, Concurrency, _) ->
+    service_start_wait(lists:reverse(Pids), Service, Concurrency);
+service_start_external(ProcessIndex, Pids,
                        #config_service_external{
                            file_path = FilePath,
                            args = Args,
@@ -1183,33 +1202,41 @@ service_start_external(IndexProcess, Pids,
                            options = #config_service_options{
                                restart_all = RestartAll,
                                restart_delay = RestartDelay,
-                               scope = Scope} = Options,
+                               critical = Critical,
+                               scope = Scope,
+                               bind = Bind} = Options,
                            max_r = MaxR,
                            max_t = MaxT,
                            uuid = ID} = Service,
-                       CountThread, CountProcess, Timeout) ->
+                       ThreadCount, ProcessCount, Concurrency, Timeout) ->
+    {BindNew,
+     ConcurrencyNew} = cloudi_core_i_concurrency:
+                       bind_assign_process(Bind, ProcessIndex, ThreadCount,
+                                           Concurrency),
     case cloudi_core_i_services_monitor:
          monitor(cloudi_core_i_spawn, start_external,
-                 [CountThread,
+                 [ThreadCount,
                   FilePath, Args, Env,
                   Protocol, BufferSize, TimeoutInit,
                   Prefix, TimeoutAsync, TimeoutSync, TimeoutTerm,
-                  DestRefresh, DestListDeny,
-                  DestListAllow, Options, ID],
-                 IndexProcess, CountProcess, CountThread, Scope,
+                  DestRefresh, DestListDeny, DestListAllow,
+                  Options#config_service_options{bind = BindNew}, ID],
+                 ProcessIndex, ProcessCount, ThreadCount, Scope,
                  TimeoutTerm, RestartAll, RestartDelay,
-                 MaxR, MaxT, ID, Timeout) of
+                 Critical, MaxR, MaxT, ID, Timeout) of
         {ok, P} ->
             service_format_log(Service, P),
-            service_start_external(IndexProcess + 1, [P | Pids], Service,
-                                   CountThread, CountProcess, Timeout);
+            service_start_external(ProcessIndex + 1, [P | Pids], Service,
+                                   ThreadCount, ProcessCount,
+                                   ConcurrencyNew, Timeout);
         {error, Reason} ->
             {error, {service_external_start_failed, Reason}}
     end.
 
-service_start_external(Service, CountThread, CountProcess, Timeout) ->
-    service_start_external(0, [], Service, CountThread,
-                           CountProcess, timeout_decr(Timeout)).
+service_start_external(Service, ThreadCount, ProcessCount,
+                       Concurrency, Timeout) ->
+    service_start_external(0, [], Service, ThreadCount, ProcessCount,
+                           Concurrency, timeout_decr(Timeout)).
 
 service_stop_internal(#config_service_internal{
                           module = Module,
@@ -1318,19 +1345,19 @@ service_resume_external(#config_service_external{
             {error, {service_external_resume_failed, Reason}}
     end.
 
-service_update_external(CountProcess, [], [], _, _, CountProcess) ->
+service_update_external(ProcessCount, [], [], _, _, ProcessCount) ->
     ok;
-service_update_external(IndexProcess, Pids, Ports, Arguments,
-                        CountThread, CountProcess) ->
-    {ProcessPids, RemainingPids} = lists:split(CountThread, Pids),
-    {ProcessPorts, RemainingPorts} = lists:split(CountThread, Ports),
+service_update_external(ProcessIndex, Pids, Ports, Arguments,
+                        ThreadCount, ProcessCount) ->
+    {ProcessPids, RemainingPids} = lists:split(ThreadCount, Pids),
+    {ProcessPorts, RemainingPorts} = lists:split(ThreadCount, Ports),
     case cloudi_core_i_spawn:
          update_external(ProcessPids, ProcessPorts,
-                         [IndexProcess, CountProcess | Arguments]) of
+                         [ProcessIndex, ProcessCount | Arguments]) of
         ok ->
-            service_update_external(IndexProcess + 1,
+            service_update_external(ProcessIndex + 1,
                                     RemainingPids, RemainingPorts, Arguments,
-                                    CountThread, CountProcess);
+                                    ThreadCount, ProcessCount);
         {error, Reason} ->
             {error, {service_external_update_failed, Reason}}
     end.
