@@ -9,7 +9,7 @@
 %%%
 %%% MIT License
 %%%
-%%% Copyright (c) 2011-2021 Michael Truog <mjtruog at protonmail dot com>
+%%% Copyright (c) 2011-2022 Michael Truog <mjtruog at protonmail dot com>
 %%%
 %%% Permission is hereby granted, free of charge, to any person obtaining a
 %%% copy of this software and associated documentation files (the "Software"),
@@ -30,8 +30,8 @@
 %%% DEALINGS IN THE SOFTWARE.
 %%%
 %%% @author Michael Truog <mjtruog at protonmail dot com>
-%%% @copyright 2011-2021 Michael Truog
-%%% @version 2.0.3 {@date} {@time}
+%%% @copyright 2011-2022 Michael Truog
+%%% @version 2.0.5 {@date} {@time}
 %%%------------------------------------------------------------------------
 
 -module(cloudi_core_i_nodes).
@@ -54,6 +54,7 @@
          handle_call/3, handle_cast/2, handle_info/2,
          terminate/2, code_change/3]).
 
+-include("cloudi_availability.hrl").
 -include("cloudi_logger.hrl").
 -include("cloudi_core_i_constants.hrl").
 -include("cloudi_core_i_configuration.hrl").
@@ -72,8 +73,8 @@
         nodes_dead :: list(node()),
         nodes_all :: list(node()),
         nodes_state :: nodes_state(),
-        nodes_down_durations = cloudi_core_i_status:durations_new()
-            :: cloudi_core_i_status:durations(node()),
+        nodes_down_durations = cloudi_availability:durations_new()
+            :: cloudi_availability:durations(node()),
         logging_redirect :: node() | undefined,
         reconnect_interval :: pos_integer(),
         reconnect_timer :: reference(),
@@ -401,7 +402,7 @@ reconfigure_nodes(NodeName, NodesNew,
     NodesDeadNew = reconfigure_nodes_dead(ListenNodes, NodesAllNew),
     NodesAliveNew = reconfigure_nodes_alive(NodesDeadNew, NodesAllNew),
     NodesStateNew = maps:with([node() | NodesAllNew], NodesStateOld),
-    NodesDownDurationsNew = cloudi_core_i_status:
+    NodesDownDurationsNew = cloudi_availability:
                             durations_copy(NodesAllNew, NodesDownDurationsOld),
     {NodesAliveNew, NodesDeadNew, NodesAllNew,
      NodesStateNew, NodesDownDurationsNew}.
@@ -450,7 +451,7 @@ track_nodeup(Node, ConnectNode,
             NodeState = {TimeStart, ConnectNode, undefined, Disconnects},
             Duration = {TimeDisconnect, TimeConnect},
             {maps:put(Node, NodeState, NodesState),
-             cloudi_core_i_status:
+             cloudi_availability:
              durations_store(NodeL, Duration, NodesDownDurations)};
         error ->
             NodeState = {TimeConnect, ConnectNode, undefined, 0},
@@ -486,16 +487,18 @@ nodes_status(NodesSelection, TimeNow, NodesDownDurations, NodesState,
     TimeWeekStart = TimeNow - ?NATIVE_TIME_IN_WEEK,
     TimeMonthStart = TimeNow - ?NATIVE_TIME_IN_MONTH,
     TimeYearStart = TimeNow - ?NATIVE_TIME_IN_YEAR,
+    TimeOffset = erlang:time_offset(),
     CostDefault = maps:get(default, Cost, undefined),
     nodes_status(NodesSelection, [], TimeNow, TimeDayStart, TimeWeekStart,
-                 TimeMonthStart, TimeYearStart, NodesDownDurations, NodesState,
+                 TimeMonthStart, TimeYearStart, TimeOffset,
+                 NodesDownDurations, NodesState,
                  Cost, CostDefault, CostPrecision).
 
-nodes_status([], StatusList, _, _, _, _, _, _, _, _, _, _) ->
+nodes_status([], StatusList, _, _, _, _, _, _, _, _, _, _, _) ->
     {ok, lists:reverse(StatusList)};
 nodes_status([Node | NodesSelection], StatusList, TimeNow,
              TimeDayStart, TimeWeekStart,
-             TimeMonthStart, TimeYearStart,
+             TimeMonthStart, TimeYearStart, TimeOffset,
              NodesDownDurations, NodesState,
              Cost, CostDefault, CostPrecision) ->
     case maps:find(Node, NodesState) of
@@ -508,34 +511,50 @@ nodes_status([Node | NodesSelection], StatusList, TimeNow,
                 is_integer(TimeDisconnect) ->
                     % track ongoing downtime with a temporary duration
                     {true,
-                     cloudi_core_i_status:
+                     cloudi_availability:
                      durations_store([Node], {TimeDisconnect, TimeNow},
                                      NodesDownDurations)}
             end,
-            DurationsStateDown = cloudi_core_i_status:
+            DurationsStateDown = cloudi_availability:
                                  durations_state(Node, NodesDownDurationsTmp),
             NanoSeconds = cloudi_timestamp:
                           convert(TimeNow - TimeStart, native, nanosecond),
             Uptime = cloudi_timestamp:
                      nanoseconds_to_string(NanoSeconds),
             {ApproximateYearDisconnect,
-             NanoSecondsYearDisconnect} = cloudi_core_i_status:
-                                          durations_sum(DurationsStateDown,
-                                                        TimeYearStart),
+             NanoSecondsYearDisconnect,
+             ViewYearDisconnect} = cloudi_availability:
+                                   durations_sum_with_view(DurationsStateDown,
+                                                           TimeYearStart,
+                                                           TimeNow,
+                                                           year,
+                                                           TimeOffset),
             {ApproximateMonthDisconnect,
-             NanoSecondsMonthDisconnect} = cloudi_core_i_status:
-                                           durations_sum(DurationsStateDown,
-                                                         TimeMonthStart),
+             NanoSecondsMonthDisconnect,
+             ViewMonthDisconnect} = cloudi_availability:
+                                    durations_sum_with_view(DurationsStateDown,
+                                                            TimeMonthStart,
+                                                            TimeNow,
+                                                            month,
+                                                            TimeOffset),
             {ApproximateWeekDisconnect,
-             NanoSecondsWeekDisconnect} = cloudi_core_i_status:
-                                          durations_sum(DurationsStateDown,
-                                                        TimeWeekStart),
+             NanoSecondsWeekDisconnect,
+             ViewWeekDisconnect} = cloudi_availability:
+                                   durations_sum_with_view(DurationsStateDown,
+                                                           TimeWeekStart,
+                                                           TimeNow,
+                                                           week,
+                                                           TimeOffset),
             {ApproximateDayDisconnect,
-             NanoSecondsDayDisconnect} = cloudi_core_i_status:
-                                         durations_sum(DurationsStateDown,
-                                                       TimeDayStart),
+             NanoSecondsDayDisconnect,
+             ViewDayDisconnect} = cloudi_availability:
+                                  durations_sum_with_view(DurationsStateDown,
+                                                          TimeDayStart,
+                                                          TimeNow,
+                                                          day,
+                                                          TimeOffset),
             Status0 = [],
-            Status1 = case cloudi_core_i_status:
+            Status1 = case cloudi_availability:
                            nanoseconds_to_availability_year(
                                NanoSeconds,
                                ApproximateYearDisconnect,
@@ -546,7 +565,7 @@ nodes_status([Node | NodesSelection], StatusList, TimeNow,
                     [{availability_year,
                       AvailabilityYear} | Status0]
             end,
-            Status2 = case cloudi_core_i_status:
+            Status2 = case cloudi_availability:
                            nanoseconds_to_availability_month(
                                NanoSeconds,
                                ApproximateMonthDisconnect,
@@ -557,7 +576,7 @@ nodes_status([Node | NodesSelection], StatusList, TimeNow,
                     [{availability_month,
                       AvailabilityMonth} | Status1]
             end,
-            Status3 = case cloudi_core_i_status:
+            Status3 = case cloudi_availability:
                            nanoseconds_to_availability_week(
                                NanoSeconds,
                                ApproximateWeekDisconnect,
@@ -569,81 +588,118 @@ nodes_status([Node | NodesSelection], StatusList, TimeNow,
                       AvailabilityWeek} | Status2]
             end,
             Status4 = [{availability_day,
-                        cloudi_core_i_status:
+                        cloudi_availability:
                         nanoseconds_to_availability_day(
                             NanoSeconds,
                             ApproximateDayDisconnect,
                             NanoSecondsDayDisconnect)} | Status3],
+            DowntimeYearDisconnect = TimeStart =< TimeMonthStart andalso
+                                     NanoSecondsYearDisconnect > 0,
+            DowntimeMonthDisconnect = TimeStart =< TimeWeekStart andalso
+                                      (NanoSecondsMonthDisconnect > 0 orelse
+                                       NanoSecondsYearDisconnect > 0),
+            DowntimeWeekDisconnect = TimeStart =< TimeDayStart andalso
+                                     (NanoSecondsWeekDisconnect > 0 orelse
+                                      NanoSecondsMonthDisconnect > 0 orelse
+                                      NanoSecondsYearDisconnect > 0),
+            DowntimeDayDisconnect = NanoSecondsDayDisconnect > 0 orelse
+                                    NanoSecondsWeekDisconnect > 0 orelse
+                                    NanoSecondsMonthDisconnect > 0 orelse
+                                    NanoSecondsYearDisconnect > 0,
             Status5 = if
-                TimeStart =< TimeMonthStart,
-                NanoSecondsYearDisconnect > 0 ->
-                    [{downtime_year_disconnected,
-                      cloudi_core_i_status:
-                      nanoseconds_to_string_gt(NanoSecondsYearDisconnect,
-                                               ApproximateYearDisconnect)} |
+                DowntimeYearDisconnect =:= true ->
+                    [{outages_year_disconnected,
+                      ViewYearDisconnect} |
                      Status4];
-                true ->
+                DowntimeYearDisconnect =:= false ->
                     Status4
             end,
             Status6 = if
-                TimeStart =< TimeWeekStart,
-                NanoSecondsMonthDisconnect > 0 orelse
-                NanoSecondsYearDisconnect > 0 ->
-                    [{downtime_month_disconnected,
-                      cloudi_core_i_status:
-                      nanoseconds_to_string_gt(NanoSecondsMonthDisconnect,
-                                               ApproximateMonthDisconnect)} |
+                DowntimeMonthDisconnect =:= true ->
+                    [{outages_month_disconnected,
+                      ViewMonthDisconnect} |
                      Status5];
-                true ->
+                DowntimeMonthDisconnect =:= false ->
                     Status5
             end,
             Status7 = if
-                TimeStart =< TimeDayStart,
-                NanoSecondsWeekDisconnect > 0 orelse
-                NanoSecondsMonthDisconnect > 0 orelse
-                NanoSecondsYearDisconnect > 0 ->
-                    [{downtime_week_disconnected,
-                      cloudi_core_i_status:
-                      nanoseconds_to_string_gt(NanoSecondsWeekDisconnect,
-                                               ApproximateWeekDisconnect)} |
+                DowntimeWeekDisconnect =:= true ->
+                    [{outages_week_disconnected,
+                      ViewWeekDisconnect} |
                      Status6];
-                true ->
+                DowntimeWeekDisconnect =:= false ->
                     Status6
             end,
             Status8 = if
-                NanoSecondsDayDisconnect > 0 orelse
-                NanoSecondsWeekDisconnect > 0 orelse
-                NanoSecondsMonthDisconnect > 0 ->
-                    [{downtime_day_disconnected,
-                      cloudi_core_i_status:
-                      nanoseconds_to_string_gt(NanoSecondsDayDisconnect,
-                                               ApproximateDayDisconnect)} |
+                DowntimeDayDisconnect =:= true ->
+                    [{outages_day_disconnected,
+                      ViewDayDisconnect} |
                      Status7];
-                true ->
+                DowntimeDayDisconnect =:= false ->
                     Status7
             end,
             Status9 = if
+                DowntimeYearDisconnect =:= true ->
+                    [{downtime_year_disconnected,
+                      cloudi_availability:
+                      nanoseconds_to_string_gt(NanoSecondsYearDisconnect,
+                                               ApproximateYearDisconnect)} |
+                     Status8];
+                DowntimeYearDisconnect =:= false ->
+                    Status8
+            end,
+            Status10 = if
+                DowntimeMonthDisconnect =:= true ->
+                    [{downtime_month_disconnected,
+                      cloudi_availability:
+                      nanoseconds_to_string_gt(NanoSecondsMonthDisconnect,
+                                               ApproximateMonthDisconnect)} |
+                     Status9];
+                DowntimeMonthDisconnect =:= false ->
+                    Status9
+            end,
+            Status11 = if
+                DowntimeWeekDisconnect =:= true ->
+                    [{downtime_week_disconnected,
+                      cloudi_availability:
+                      nanoseconds_to_string_gt(NanoSecondsWeekDisconnect,
+                                               ApproximateWeekDisconnect)} |
+                     Status10];
+                DowntimeWeekDisconnect =:= false ->
+                    Status10
+            end,
+            Status12 = if
+                DowntimeDayDisconnect =:= true ->
+                    [{downtime_day_disconnected,
+                      cloudi_availability:
+                      nanoseconds_to_string_gt(NanoSecondsDayDisconnect,
+                                               ApproximateDayDisconnect)} |
+                     Status11];
+                DowntimeDayDisconnect =:= false ->
+                    Status11
+            end,
+            Status13 = if
                 LocalNode =:= true ->
-                    Status8;
+                    Status12;
                 LocalNode =:= false ->
                     [{connection, ConnectNode},
                      {tracked_disconnects,
                       erlang:integer_to_list(Disconnects)},
-                     {disconnected, Disconnected} | Status8]
+                     {disconnected, Disconnected} | Status12]
             end,
-            Status10 = node_status_cost(maps:get(Node, Cost, CostDefault),
+            Status14 = node_status_cost(maps:get(Node, Cost, CostDefault),
                                         NanoSeconds, CostPrecision,
-                                        LocalNode, Status9),
+                                        LocalNode, Status13),
             StatusN = if
                 LocalNode =:= true ->
-                    [{uptime, Uptime} | Status10];
+                    [{uptime, Uptime} | Status14];
                 LocalNode =:= false ->
-                    [{tracked, Uptime} | Status10]
+                    [{tracked, Uptime} | Status14]
             end,
             nodes_status(NodesSelection,
                          [{Node, StatusN} | StatusList], TimeNow,
                          TimeDayStart, TimeWeekStart,
-                         TimeMonthStart, TimeYearStart,
+                         TimeMonthStart, TimeYearStart, TimeOffset,
                          NodesDownDurations, NodesState,
                          Cost, CostDefault, CostPrecision);
         error ->
