@@ -10,7 +10,7 @@
 %%%
 %%% MIT License
 %%%
-%%% Copyright (c) 2015-2020 Michael Truog <mjtruog at protonmail dot com>
+%%% Copyright (c) 2015-2022 Michael Truog <mjtruog at protonmail dot com>
 %%%
 %%% Permission is hereby granted, free of charge, to any person obtaining a
 %%% copy of this software and associated documentation files (the "Software"),
@@ -31,8 +31,8 @@
 %%% DEALINGS IN THE SOFTWARE.
 %%%
 %%% @author Michael Truog <mjtruog at protonmail dot com>
-%%% @copyright 2015-2020 Michael Truog
-%%% @version 1.8.1 {@date} {@time}
+%%% @copyright 2015-2022 Michael Truog
+%%% @version 2.0.5 {@date} {@time}
 %%%------------------------------------------------------------------------
 
 -module(cloudi_core_i_os_process).
@@ -48,9 +48,11 @@
          cgroup_set/2,
          cgroup_unset/2,
          chroot_format/2,
+         syscall_lock_validate/1,
+         syscall_lock_format/1,
          directory_format/2]).
 
--include("cloudi_core_i_constants.hrl").
+-include("cloudi_environment.hrl").
 -ifdef(CLOUDI_CORE_STANDALONE).
 -define(OS_RLIMIT_DEFAULTS, [{invalid, undefined}]).
 -else.
@@ -251,20 +253,23 @@ limit_format_list_value(Value)
 owner_validate(Values)
     when is_list(Values) ->
     Defaults = [
-        {user, "user"},
-        {group, "group"}],
+        {user, "string"},
+        {group, "string"}],
     case cloudi_proplists:take_values(Defaults, Values) of
         [User, _]
-            when not ((is_list(User) andalso is_integer(hd(User))) orelse
+            when not ((is_list(User) andalso is_integer(hd(User)) andalso
+                       (User /= "root")) orelse
                       (is_integer(User) andalso
                        (User > 0) andalso (User =< 16#ffffffffffffffff))) ->
             {error, {service_options_owner_invalid, [{user, User}]}};
         [_, Group]
-            when not ((is_list(Group) andalso is_integer(hd(Group))) orelse
+            when not ((is_list(Group) andalso is_integer(hd(Group)) andalso
+                       (Group /= "root")) orelse
                       (is_integer(Group) andalso
                        (Group > 0) andalso (Group =< 16#ffffffffffffffff))) ->
             {error, {service_options_owner_invalid, [{group, Group}]}};
         [_, _] ->
+            % only basic validation has passed
             {ok, Values};
         [_, _ | Extra] ->
             {error, {service_options_owner_invalid, Extra}}
@@ -286,7 +291,7 @@ owner_format(Values, EnvironmentLookup) ->
             case cloudi_environment:transform(User, EnvironmentLookup) of
                 [] ->
                     % make user lookup fail
-                    {0, User};
+                    {0, [0]};
                 [_ | _] = UserFinal ->
                     {0, UserFinal}
             end;
@@ -301,7 +306,7 @@ owner_format(Values, EnvironmentLookup) ->
             case cloudi_environment:transform(Group, EnvironmentLookup) of
                 [] ->
                     % make group lookup fail
-                    {0, Group};
+                    {0, [0]};
                 [_ | _] = GroupFinal ->
                     {0, GroupFinal}
             end;
@@ -336,7 +341,12 @@ cgroup_validate(Values)
             {error, {service_options_cgroup_invalid,
                      [{update_or_create, UpdateOrCreate}]}};
         [_, _, _] ->
-            {ok, Values};
+            case cloudi_x_cgroups:new() of
+                {ok, _} ->
+                    {ok, Values};
+                {error, _} ->
+                    {error, {service_options_cgroup_invalid, does_not_exist}}
+            end;
         [_, _, _ | Extra] ->
             {error, {service_options_cgroup_invalid, Extra}}
     end;
@@ -398,8 +408,7 @@ cgroup_set(OSPid, Values)
 
 -spec cgroup_unset(OSPid :: pos_integer(),
                    Values :: cloudi_service_api:cgroup_external()) ->
-    ok |
-    {error, any()}.
+    ok.
 
 cgroup_unset(_, undefined) ->
     ok;
@@ -439,6 +448,58 @@ chroot_format(Chroot, EnvironmentLookup) ->
             {error, {service_options_chroot_invalid, NewChroot}}
     end.
 
+-spec syscall_lock_validate(cloudi_service_api:syscall_lock_external()) ->
+    {ok, cloudi_service_api:syscall_lock_external()} |
+    {error, {service_options_syscall_lock_invalid, any()}}.
+
+syscall_lock_validate(undefined) ->
+    {ok, undefined};
+syscall_lock_validate(Values)
+    when is_list(Values) ->
+    Defaults = [
+        {type, ?SYSCALL_LOCK_TYPE},
+        {names, []}],
+    case cloudi_proplists:take_values(Defaults, Values) of
+        [Type, _]
+            when not (Type =:= ?SYSCALL_LOCK_TYPE) ->
+            Invalid = if
+                Type =:= undefined ->
+                    does_not_exist;
+                true ->
+                    [{type, Type}]
+            end,
+            {error, {service_options_syscall_lock_invalid,
+                     Invalid}};
+        [_, Names]
+            when not (is_list(Names) andalso
+                      is_integer(hd(hd(Names))))->
+            {error, {service_options_syscall_lock_invalid,
+                     [{names, Names}]}};
+        [_, Names] ->
+            NamesValid = syscall_names_valid(Names),
+            if
+                NamesValid =:= true ->
+                    {ok, Values};
+                NamesValid =:= false ->
+                    {error, {service_options_syscall_lock_invalid,
+                             [{names, Names}]}}
+            end;
+        [_, _ | Extra] ->
+            {error, {service_options_syscall_lock_invalid, Extra}}
+    end;
+syscall_lock_validate(Invalid) ->
+    {error, {service_options_syscall_lock_invalid, Invalid}}.
+
+-spec syscall_lock_format(cloudi_service_api:syscall_lock_external()) ->
+    binary().
+
+syscall_lock_format(undefined) ->
+    <<0>>;
+syscall_lock_format(Values)
+    when is_list(Values) ->
+    {names, Names} = lists:keyfind(names, 1, Values),
+    erlang:iolist_to_binary(syscall_names_format(Names)).
+
 -spec directory_format(Directory :: cloudi_service_api:directory_external(),
                        EnvironmentLookup :: cloudi_environment:lookup()) ->
     {ok, NewDirectory :: string()} |
@@ -467,3 +528,28 @@ absolute_path([_ | _] = Path) ->
     (AbsolutePath == Path) orelse
     (AbsolutePath ++ [H] == Path).
 
+syscall_name_valid([]) ->
+    true;
+syscall_name_valid([H | T])
+    when (H >= $a) andalso (H =< $z);
+         (H == $_);
+         (H >= $0) andalso (H =< $9) ->
+    syscall_name_valid(T);
+syscall_name_valid(_) ->
+    false.
+
+syscall_names_valid([]) ->
+    true;
+syscall_names_valid([H | T]) ->
+    Valid = syscall_name_valid(H),
+    if
+        Valid =:= true ->
+            syscall_names_valid(T);
+        Valid =:= false ->
+            false
+    end.
+
+syscall_names_format([] = L) ->
+    L;
+syscall_names_format([H | T]) ->
+    [[H, 0] | syscall_names_format(T)].
