@@ -8,7 +8,7 @@
 %%%
 %%% MIT License
 %%%
-%%% Copyright (c) 2011-2022 Michael Truog <mjtruog at protonmail dot com>
+%%% Copyright (c) 2011-2023 Michael Truog <mjtruog at protonmail dot com>
 %%%
 %%% Permission is hereby granted, free of charge, to any person obtaining a
 %%% copy of this software and associated documentation files (the "Software"),
@@ -29,8 +29,8 @@
 %%% DEALINGS IN THE SOFTWARE.
 %%%
 %%% @author Michael Truog <mjtruog at protonmail dot com>
-%%% @copyright 2011-2022 Michael Truog
-%%% @version 2.0.5 {@date} {@time}
+%%% @copyright 2011-2023 Michael Truog
+%%% @version 2.0.6 {@date} {@time}
 %%%------------------------------------------------------------------------
 
 -module(cloudi_core_i_spawn).
@@ -55,13 +55,19 @@
         cloudi_core_i_services_external_sup:process_start).
 
 % environmental variables used by CloudI API initialization
--define(ENVIRONMENT_THREAD_COUNT,  "CLOUDI_API_INIT_THREAD_COUNT").
--define(ENVIRONMENT_PROTOCOL,      "CLOUDI_API_INIT_PROTOCOL").
--define(ENVIRONMENT_BUFFER_SIZE,   "CLOUDI_API_INIT_BUFFER_SIZE").
+-define(ENVIRONMENT_THREAD_COUNT,       "CLOUDI_API_INIT_THREAD_COUNT").
+-define(ENVIRONMENT_PROTOCOL,           "CLOUDI_API_INIT_PROTOCOL").
+-define(ENVIRONMENT_BUFFER_SIZE,        "CLOUDI_API_INIT_BUFFER_SIZE").
+% environmental variable process constants provided before initialization
+-define(ENVIRONMENT_TIMEOUT_INITIALIZE, "CLOUDI_API_INIT_TIMEOUT_INITIALIZE").
+-define(ENVIRONMENT_TIMEOUT_TERMINATE,  "CLOUDI_API_INIT_TIMEOUT_TERMINATE").
+-define(ENVIRONMENT_PROCESS_INDEX,      "CLOUDI_API_INIT_PROCESS_INDEX").
+-define(ENVIRONMENT_PROCESS_COUNT_MAX,  "CLOUDI_API_INIT_PROCESS_COUNT_MAX").
+-define(ENVIRONMENT_PROCESS_COUNT_MIN,  "CLOUDI_API_INIT_PROCESS_COUNT_MIN").
 
 -ifdef(CLOUDI_CORE_STANDALONE).
 -dialyzer({no_match,
-           [start_external_spawn/20,
+           [start_external_spawn/25,
             update_external/3]}).
 -endif.
 
@@ -101,7 +107,7 @@
                   #config_service_options{} | % ConfigOptions
                   cloudi_service_api:service_id()). % ID
 -type arguments_execution_external() ::
-    nonempty_list(pos_integer() | % ThreadsPerProcess
+    nonempty_list(pos_integer() | % ThreadCount
                   cloudi_service_api:file_path() | % Filename
                   cloudi_service_api:args_external() | % Arguments
                   cloudi_service_api:env_external() | % Environment
@@ -224,7 +230,7 @@ start_internal(ProcessIndex, ProcessCount, TimeStart, TimeRestart, Restarts,
                      TimeRestart :: undefined |
                                     cloudi_timestamp:native_monotonic(),
                      Restarts :: non_neg_integer(),
-                     ThreadsPerProcess :: pos_integer(),
+                     ThreadCount :: pos_integer(),
                      Filename :: cloudi_service_api:file_path(),
                      Arguments :: cloudi_service_api:args_external(),
                      Environment :: cloudi_service_api:env_external(),
@@ -252,12 +258,12 @@ start_internal(ProcessIndex, ProcessCount, TimeStart, TimeRestart, Restarts,
     {error, error_reason_start_external() | any()}.
 
 start_external(ProcessIndex, ProcessCount, TimeStart, TimeRestart, Restarts,
-               ThreadsPerProcess, Filename, Arguments, Environment,
+               ThreadCount, Filename, Arguments, Environment,
                Protocol, BufferSize, Timeout, Prefix,
                TimeoutAsync, TimeoutSync, TimeoutTerm, DestRefresh,
                DestListDeny, DestListAllow, ConfigOptions, ID) ->
     case start_external_spawn_params(ProcessIndex, ProcessCount,
-                                     ThreadsPerProcess,
+                                     ThreadCount,
                                      Filename, Arguments, Environment,
                                      Protocol, BufferSize, Timeout, Prefix,
                                      TimeoutAsync, TimeoutSync, TimeoutTerm,
@@ -265,12 +271,13 @@ start_external(ProcessIndex, ProcessCount, TimeStart, TimeRestart, Restarts,
         {ok,
          SpawnProcess, SpawnProtocol, SocketPath,
          Rlimits, Owner, Nice, CGroup, Chroot, SyscallLock, Directory,
-         CommandLine, FilenameNew, ArgumentsNew, EnvironmentLookup} ->
+         CommandLine, FilenameNew, ArgumentsNew, EnvironmentLookup,
+         ProcessCountMax, ProcessCountMin} ->
             {ok, DestDeny, DestAllow} =
                 start_external_threads_params(DestListDeny, DestListAllow),
             ConfigOptionsNew = ConfigOptions#config_service_options{
                                    cgroup = CGroup},
-            case start_external_threads(ThreadsPerProcess,
+            case start_external_threads(ThreadCount,
                                         ProcessIndex,
                                         ProcessCount,
                                         TimeStart,
@@ -294,13 +301,15 @@ start_external(ProcessIndex, ProcessCount, TimeStart, TimeRestart, Restarts,
                                          Rlimits, Owner,
                                          Nice, CGroup, Chroot, SyscallLock,
                                          Directory,
-                                         ThreadsPerProcess,
+                                         ThreadCount,
                                          CommandLine,
                                          FilenameNew,
                                          ArgumentsNew,
                                          Environment,
                                          EnvironmentLookup,
-                                         Protocol, BufferSize);
+                                         Protocol, BufferSize,
+                                         Timeout, TimeoutTerm, ProcessIndex,
+                                         ProcessCountMax, ProcessCountMin);
                 {error, _} = Error ->
                     Error
             end;
@@ -308,13 +317,13 @@ start_external(ProcessIndex, ProcessCount, TimeStart, TimeRestart, Restarts,
             Error
     end.
 
--spec status_internal(CountProcess :: pos_integer(),
+-spec status_internal(ProcessCount :: pos_integer(),
                       PidsOrdered :: list(pid()),
                       arguments_execution_internal(),
                       Status :: nonempty_list()) ->
     cloudi_service_api:service_status_internal().
 
-status_internal(CountProcess, PidsOrdered,
+status_internal(ProcessCount, PidsOrdered,
                 [_GroupLeader,
                  Module, _Args, _TimeoutInit, Prefix,
                  _TimeoutAsync, _TimeoutSync, _TimeoutTerm,
@@ -324,19 +333,19 @@ status_internal(CountProcess, PidsOrdered,
     [{type, internal},
      {prefix, Prefix},
      {module, Module},
-     {count_process, CountProcess},
+     {count_process, ProcessCount},
      {pids_erlang, PidsOrdered} | Status].
 
--spec status_external(CountProcess :: pos_integer(),
-                      CountThread :: pos_integer(),
+-spec status_external(ProcessCount :: pos_integer(),
+                      ThreadCount :: pos_integer(),
                       OSPidsOrdered :: list(pos_integer()),
                       PidsOrdered :: list(pid()),
                       arguments_execution_external(),
                       Status :: nonempty_list()) ->
     cloudi_service_api:service_status_external().
 
-status_external(CountProcess, CountThread, OSPidsOrdered, PidsOrdered,
-                [_ThreadsPerProcess,
+status_external(ProcessCount, ThreadCount, OSPidsOrdered, PidsOrdered,
+                [_ThreadCount,
                  Filename, _Arguments, _Environment,
                  _Protocol, _BufferSize, _TimeoutInit, Prefix,
                  _TimeoutAsync, _TimeoutSync, _TimeoutTerm,
@@ -346,8 +355,8 @@ status_external(CountProcess, CountThread, OSPidsOrdered, PidsOrdered,
     [{type, external},
      {prefix, Prefix},
      {file_path, Filename},
-     {count_process, CountProcess},
-     {count_thread, CountThread},
+     {count_process, ProcessCount},
+     {count_thread, ThreadCount},
      {pids_os, OSPidsOrdered},
      {pids_erlang, PidsOrdered} | Status].
 
@@ -358,13 +367,13 @@ status_external(CountProcess, CountThread, OSPidsOrdered, PidsOrdered,
     {error, error_reason_start_external() | any()}.
 
 update_external(Pids, Ports,
-                [ProcessIndex, ProcessCount, ThreadsPerProcess,
+                [ProcessIndex, ProcessCount, ThreadCount,
                  Filename, Arguments, Environment,
                  Protocol, BufferSize, Timeout, Prefix,
                  TimeoutAsync, TimeoutSync, TimeoutTerm, DestRefresh,
                  _DestListDeny, _DestListAllow, ConfigOptions, ID]) ->
     case start_external_spawn_params(ProcessIndex, ProcessCount,
-                                     ThreadsPerProcess,
+                                     ThreadCount,
                                      Filename, Arguments, Environment,
                                      Protocol, BufferSize, Timeout, Prefix,
                                      TimeoutAsync, TimeoutSync, TimeoutTerm,
@@ -372,15 +381,18 @@ update_external(Pids, Ports,
         {ok,
          SpawnProcess, SpawnProtocol, SocketPath,
          Rlimits, Owner, Nice, CGroup, Chroot, SyscallLock, Directory,
-         CommandLine, FilenameNew, ArgumentsNew, EnvironmentLookup} ->
+         CommandLine, FilenameNew, ArgumentsNew, EnvironmentLookup,
+         ProcessCountMax, ProcessCountMin} ->
             case start_external_spawn(SpawnProcess, SpawnProtocol, SocketPath,
                                       Pids, Ports,
                                       Rlimits, Owner,
                                       Nice, CGroup, Chroot, SyscallLock,
-                                      Directory, ThreadsPerProcess,
+                                      Directory, ThreadCount,
                                       CommandLine, FilenameNew, ArgumentsNew,
                                       Environment, EnvironmentLookup,
-                                      Protocol, BufferSize) of
+                                      Protocol, BufferSize,
+                                      Timeout, TimeoutTerm, ProcessIndex,
+                                      ProcessCountMax, ProcessCountMin) of
                 {ok, Pids} ->
                     _ = [Pid ! {'cloudi_service_update_state', CommandLine}
                          || Pid <- Pids],
@@ -497,13 +509,13 @@ update_external_f(FilenameNew, ArgumentsNew, EnvironmentNew,
                   TimeoutInitNew, TimeoutAsyncNew, TimeoutSyncNew,
                   DestListDenyNew, DestListAllowNew,
                   OptionsKeys, ConfigOptionsNew,
-                  [ThreadsPerProcess,
+                  [ThreadCount,
                    FilenameOld, ArgumentsOld, EnvironmentOld,
                    Protocol, BufferSize, TimeoutInitOld, Prefix,
                    TimeoutAsyncOld, TimeoutSyncOld, TimeoutTerm,
                    DestRefreshOld, DestListDenyOld, DestListAllowOld,
                    ConfigOptionsOld, ID]) ->
-    [ThreadsPerProcess,
+    [ThreadCount,
      if
         is_list(FilenameNew) ->
             FilenameNew;
@@ -591,6 +603,17 @@ syscall_lock(#config_service_options{syscall_lock = SyscallLock}) ->
 directory(#config_service_options{directory = Directory}, EnvironmentLookup) ->
     cloudi_core_i_os_process:directory_format(Directory, EnvironmentLookup).
 
+count_process_limits(ProcessCount,
+                     #config_service_options{
+                         count_process_dynamic = CountProcessDynamic}) ->
+    case cloudi_core_i_rate_based_configuration:
+         count_process_dynamic_limits(CountProcessDynamic) of
+        undefined ->
+            {ProcessCount, ProcessCount};
+        {_, _} = MinMax ->
+            MinMax
+    end.
+
 create_socket_path(TemporaryDirectory, ID)
     when is_binary(ID) ->
     Path = filename:join([TemporaryDirectory,
@@ -599,13 +622,13 @@ create_socket_path(TemporaryDirectory, ID)
     false = filelib:is_file(Path),
     Path.
 
-start_external_spawn_params(ProcessIndex, ProcessCount, ThreadsPerProcess,
+start_external_spawn_params(ProcessIndex, ProcessCount, ThreadCount,
                             Filename, Arguments, Environment,
                             Protocol, BufferSize, Timeout, Prefix,
                             TimeoutAsync, TimeoutSync, TimeoutTerm, DestRefresh,
                             ConfigOptions, ID)
     when is_integer(ProcessIndex), is_integer(ProcessCount),
-         is_integer(ThreadsPerProcess), ThreadsPerProcess > 0,
+         is_integer(ThreadCount), ThreadCount > 0,
          is_list(Filename), is_list(Arguments), is_list(Environment),
          is_integer(BufferSize), is_integer(Timeout), is_list(Prefix),
          is_integer(TimeoutAsync), is_integer(TimeoutSync),
@@ -656,12 +679,15 @@ start_external_spawn_params(ProcessIndex, ProcessCount, ThreadsPerProcess,
                     Nice = nice(ConfigOptions),
                     CGroup = cgroup(ConfigOptions, EnvironmentLookup),
                     SyscallLock = syscall_lock(ConfigOptions),
+                    {ProcessCountMin,
+                     ProcessCountMax} = count_process_limits(ProcessCount,
+                                                             ConfigOptions),
                     {ok,
                      SpawnProcess, SpawnProtocol, SocketPath,
                      Rlimits, Owner, Nice, CGroup, Chroot, SyscallLock,
                      Directory,
-                     CommandLine, FilenameNew, ArgumentsNew,
-                     EnvironmentLookup};
+                     CommandLine, FilenameNew, ArgumentsNew, EnvironmentLookup,
+                     ProcessCountMax, ProcessCountMin};
                 undefined ->
                     {error, noproc}
             end;
@@ -714,20 +740,21 @@ start_external_spawn_params_parse(Filename, Arguments, ConfigOptions,
             Error
     end.
 
-start_external_thread(ThreadsPerProcess, Pids, Ports, ThreadsPerProcess,
+start_external_thread(ThreadCount, Pids, Ports, ThreadCount,
                       _, _, _, _, _, _, _, _, _, _,
                       _, _, _, _, _, _, _, _, _) ->
     {ok, lists:reverse(Pids), lists:reverse(Ports)};
 
-start_external_thread(I, Pids, Ports, ThreadsPerProcess,
+start_external_thread(I, Pids, Ports, ThreadCount,
                       ProcessIndex, ProcessCount,
                       TimeStart, TimeRestart, Restarts,
                       CommandLine, Protocol, SocketPath, BufferSize, Timeout,
                       Prefix, TimeoutAsync, TimeoutSync, TimeoutTerm,
                       DestRefresh, DestDeny, DestAllow,
-                      ConfigOptions, ID) ->
+                      #config_service_options{
+                          bind = Bind} = ConfigOptions, ID) ->
     case ?PROCESS_START_EXTERNAL(Protocol, SocketPath,
-                                 I + ThreadsPerProcess * ProcessIndex,
+                                 I + ThreadCount * ProcessIndex,
                                  ProcessIndex, ProcessCount,
                                  TimeStart, TimeRestart, Restarts,
                                  CommandLine, BufferSize,
@@ -736,8 +763,9 @@ start_external_thread(I, Pids, Ports, ThreadsPerProcess,
                                  DestRefresh, DestDeny, DestAllow,
                                  ConfigOptions, ID) of
         {ok, Pid, Port} ->
+            BindNew = cloudi_core_i_concurrency:bind_increment_thread(Bind),
             start_external_thread(I + 1, [Pid | Pids], [Port | Ports],
-                                  ThreadsPerProcess,
+                                  ThreadCount,
                                   ProcessIndex, ProcessCount,
                                   TimeStart, TimeRestart, Restarts,
                                   CommandLine, Protocol, SocketPath,
@@ -745,20 +773,21 @@ start_external_thread(I, Pids, Ports, ThreadsPerProcess,
                                   Prefix, TimeoutAsync, TimeoutSync,
                                   TimeoutTerm,
                                   DestRefresh, DestDeny, DestAllow,
-                                  ConfigOptions, ID);
+                                  ConfigOptions#config_service_options{
+                                      bind = BindNew}, ID);
         {error, _} = Error ->
             lists:foreach(fun(P) -> erlang:exit(P, kill) end, Pids),
             Error
     end.
 
-start_external_threads(ThreadsPerProcess,
+start_external_threads(ThreadCount,
                        ProcessIndex, ProcessCount,
                        TimeStart, TimeRestart, Restarts,
                        CommandLine, Protocol, SocketPath, BufferSize, Timeout,
                        Prefix, TimeoutAsync, TimeoutSync, TimeoutTerm,
                        DestRefresh, DestDeny, DestAllow,
                        ConfigOptions, ID) ->
-    start_external_thread(0, [], [], ThreadsPerProcess,
+    start_external_thread(0, [], [], ThreadCount,
                           ProcessIndex, ProcessCount,
                           TimeStart, TimeRestart, Restarts,
                           CommandLine, Protocol, SocketPath,
@@ -770,11 +799,15 @@ start_external_threads(ThreadsPerProcess,
 start_external_spawn(SpawnProcess, SpawnProtocol, SocketPath,
                      Pids, Ports, Rlimits, Owner,
                      Nice, CGroup, Chroot, SyscallLock, Directory,
-                     ThreadsPerProcess, CommandLine,
+                     ThreadCount, CommandLine,
                      Filename, Arguments, Environment,
-                     EnvironmentLookup, Protocol, BufferSize) ->
-    case environment_parse(Environment, ThreadsPerProcess,
-                           Protocol, BufferSize, EnvironmentLookup) of
+                     EnvironmentLookup, Protocol, BufferSize,
+                     Timeout, TimeoutTerm, ProcessIndex,
+                     ProcessCountMax, ProcessCountMin) ->
+    case environment_parse(Environment, ThreadCount, Protocol, BufferSize,
+                           Timeout, TimeoutTerm, ProcessIndex,
+                           ProcessCountMax, ProcessCountMin,
+                           EnvironmentLookup) of
         {ok, SpawnEnvironment} ->
             {UserI, UserStr, GroupI, GroupStr} = Owner,
             case cloudi_core_i_os_spawn:spawn(SpawnProcess,
@@ -886,22 +919,51 @@ arguments_parse(ArgumentsBinary, ArgumentsList, Argument, Delim, [H | T]) ->
 
 % add CloudI API environmental variables and format into a single
 % string that is easy to use in C/C++
-environment_parse(Environment0, ThreadsPerProcess0,
-                  Protocol0, BufferSize0, EnvironmentLookup0) ->
-    ThreadsPerProcess1 = erlang:integer_to_list(ThreadsPerProcess0),
+environment_parse(Environment0, ThreadCount0,
+                  Protocol0, BufferSize0, Timeout0, TimeoutTerm0, ProcessIndex0,
+                  ProcessCountMax0, ProcessCountMin0, EnvironmentLookup0) ->
+    ThreadCount1 = erlang:integer_to_list(ThreadCount0),
     Protocol1 = erlang:atom_to_list(Protocol0),
     BufferSize1 = erlang:integer_to_list(BufferSize0),
-    Environment1 = lists:keystore(?ENVIRONMENT_THREAD_COUNT, 1, Environment0,
+    Timeout1 = erlang:integer_to_list(Timeout0),
+    TimeoutTerm1 = erlang:integer_to_list(TimeoutTerm0),
+    ProcessIndex1 = erlang:integer_to_list(ProcessIndex0),
+    ProcessCountMax1 = erlang:integer_to_list(ProcessCountMax0),
+    ProcessCountMin1 = erlang:integer_to_list(ProcessCountMin0),
+    Environment1 = lists:keystore(?ENVIRONMENT_THREAD_COUNT, 1,
+                                  Environment0,
                                   {?ENVIRONMENT_THREAD_COUNT,
-                                   ThreadsPerProcess1}),
-    Environment2 = lists:keystore(?ENVIRONMENT_PROTOCOL, 1, Environment1,
+                                   ThreadCount1}),
+    Environment2 = lists:keystore(?ENVIRONMENT_PROTOCOL, 1,
+                                  Environment1,
                                   {?ENVIRONMENT_PROTOCOL,
                                    Protocol1}),
-    Environment3 = lists:keystore(?ENVIRONMENT_BUFFER_SIZE, 1, Environment2,
+    Environment3 = lists:keystore(?ENVIRONMENT_BUFFER_SIZE, 1,
+                                  Environment2,
                                   {?ENVIRONMENT_BUFFER_SIZE,
                                    BufferSize1}),
+    Environment4 = lists:keystore(?ENVIRONMENT_TIMEOUT_INITIALIZE, 1,
+                                  Environment3,
+                                  {?ENVIRONMENT_TIMEOUT_INITIALIZE,
+                                   Timeout1}),
+    Environment5 = lists:keystore(?ENVIRONMENT_TIMEOUT_TERMINATE, 1,
+                                  Environment4,
+                                  {?ENVIRONMENT_TIMEOUT_TERMINATE,
+                                   TimeoutTerm1}),
+    Environment6 = lists:keystore(?ENVIRONMENT_PROCESS_INDEX, 1,
+                                  Environment5,
+                                  {?ENVIRONMENT_PROCESS_INDEX,
+                                   ProcessIndex1}),
+    Environment7 = lists:keystore(?ENVIRONMENT_PROCESS_COUNT_MAX, 1,
+                                  Environment6,
+                                  {?ENVIRONMENT_PROCESS_COUNT_MAX,
+                                   ProcessCountMax1}),
+    Environment8 = lists:keystore(?ENVIRONMENT_PROCESS_COUNT_MIN, 1,
+                                  Environment7,
+                                  {?ENVIRONMENT_PROCESS_COUNT_MIN,
+                                   ProcessCountMin1}),
     EnvironmentLookup1 = cloudi_x_trie:store(?ENVIRONMENT_THREAD_COUNT,
-                                             ThreadsPerProcess1,
+                                             ThreadCount1,
                                              EnvironmentLookup0),
     EnvironmentLookup2 = cloudi_x_trie:store(?ENVIRONMENT_PROTOCOL,
                                              Protocol1,
@@ -909,7 +971,22 @@ environment_parse(Environment0, ThreadsPerProcess0,
     EnvironmentLookup3 = cloudi_x_trie:store(?ENVIRONMENT_BUFFER_SIZE,
                                              BufferSize1,
                                              EnvironmentLookup2),
-    environment_format(Environment3, EnvironmentLookup3).
+    EnvironmentLookup4 = cloudi_x_trie:store(?ENVIRONMENT_TIMEOUT_INITIALIZE,
+                                             Timeout1,
+                                             EnvironmentLookup3),
+    EnvironmentLookup5 = cloudi_x_trie:store(?ENVIRONMENT_TIMEOUT_TERMINATE,
+                                             TimeoutTerm1,
+                                             EnvironmentLookup4),
+    EnvironmentLookup6 = cloudi_x_trie:store(?ENVIRONMENT_PROCESS_INDEX,
+                                             ProcessIndex1,
+                                             EnvironmentLookup5),
+    EnvironmentLookup7 = cloudi_x_trie:store(?ENVIRONMENT_PROCESS_COUNT_MAX,
+                                             ProcessCountMax1,
+                                             EnvironmentLookup6),
+    EnvironmentLookup8 = cloudi_x_trie:store(?ENVIRONMENT_PROCESS_COUNT_MIN,
+                                             ProcessCountMin1,
+                                             EnvironmentLookup7),
+    environment_format(Environment8, EnvironmentLookup8).
 
 environment_format_value([], _, K, _) ->
     {error, {service_external_env_invalid_expanded, K}};
