@@ -8,7 +8,7 @@
 %%%
 %%% MIT License
 %%%
-%%% Copyright (c) 2009-2018 Michael Truog <mjtruog at protonmail dot com>
+%%% Copyright (c) 2009-2022 Michael Truog <mjtruog at protonmail dot com>
 %%%
 %%% Permission is hereby granted, free of charge, to any person obtaining a
 %%% copy of this software and associated documentation files (the "Software"),
@@ -29,8 +29,8 @@
 %%% DEALINGS IN THE SOFTWARE.
 %%%
 %%% @author Michael Truog <mjtruog at protonmail dot com>
-%%% @copyright 2009-2018 Michael Truog
-%%% @version 1.7.4 {@date} {@time}
+%%% @copyright 2009-2022 Michael Truog
+%%% @version 2.0.5 {@date} {@time}
 %%%------------------------------------------------------------------------
 
 -module(cloudi_string).
@@ -73,18 +73,16 @@
          triml/2,
          trimr/1,
          trimr/2,
-         uppercase/1]).
+         uppercase/1,
+         uri_encode/1,
+         uri_decode/1]).
+
+-include("cloudi_core_i_constants.hrl").
 
 % based on unicode_util:whitespace/0
 -define(WHITESPACE, [13,9,10,11,12,13,32,133,8206,8207,8232,8233]).
 % keep output in a single line when using ~p (printable) formatting
--ifdef(ERLANG_OTP_VERSION_21_FEATURES).
 -define(COMPACT_LIMIT, "0").
--else.
--define(COMPACT_LIMIT, "1000000000000").
--endif.
-
--include("cloudi_core_i_constants.hrl").
 
 %%%------------------------------------------------------------------------
 %%% External interface functions
@@ -280,17 +278,8 @@ binary_to_term(B) ->
                        Correct :: nonempty_string()) ->
     boolean().
 
-compare_constant(Test, [_ | _] = Correct) ->
-    compare_constant(Test, Correct, 0) =:= 0.
-
-compare_constant([], [], Bits) ->
-    Bits;
-compare_constant([], [_ | _], _) ->
-    1;
-compare_constant([C | Test], [] = Correct, Bits) ->
-    compare_constant(Test, Correct, Bits bor (C bxor -1));
-compare_constant([C1 | Test], [C2 | Correct], Bits) ->
-    compare_constant(Test, Correct, Bits bor (C1 bxor C2)).
+compare_constant(Test, Correct) ->
+    compare_constant_list(Test, Correct).
 
 %%-------------------------------------------------------------------------
 %% @doc
@@ -304,8 +293,24 @@ compare_constant([C1 | Test], [C2 | Correct], Bits) ->
                             Correct :: nonempty_string()) ->
     boolean().
 
-compare_constant_list(Test, Correct) ->
-    compare_constant(Test, Correct).
+compare_constant_list(_, []) ->
+    erlang:exit(badarg);
+compare_constant_list(Test, [_ | _] = Correct) ->
+    compare_constant_list(Test, Correct,
+                          [-1 || _ <- Test], [], 0) =:= 0.
+
+compare_constant_list([], [], _, _, Bits) ->
+    Bits;
+compare_constant_list([], [_ | _], _, _, _) ->
+    -1;
+compare_constant_list([C1 | Test], [] = Correct,
+                      [C2 | Busy], BusyEmpty, Bits) ->
+    compare_constant_list(Test, Correct,
+                          Busy, BusyEmpty, Bits bor (C1 bxor C2));
+compare_constant_list([C1 | Test], [C2 | Correct],
+                      Busy, [] = BusyEmpty, Bits) ->
+    compare_constant_list(Test, Correct,
+                          Busy, BusyEmpty, Bits bor (C1 bxor C2)).
 
 %%-------------------------------------------------------------------------
 %% @doc
@@ -315,13 +320,28 @@ compare_constant_list(Test, Correct) ->
 %% @end
 %%-------------------------------------------------------------------------
 
--spec compare_constant_binary(Test :: binary(),
-                              Correct :: binary()) ->
+-spec compare_constant_binary(Test :: <<_:_*8>>,
+                              Correct :: <<_:8, _:_*8>>) ->
     boolean().
 
+compare_constant_binary(_, <<>>) ->
+    erlang:exit(badarg);
 compare_constant_binary(Test, Correct) ->
-    compare_constant(erlang:binary_to_list(Test),
-                     erlang:binary_to_list(Correct)).
+    compare_constant_binary(Test, Correct,
+                            <<<<255>> || <<_>> <= Test>>, <<>>, 0) =:= 0.
+
+compare_constant_binary(<<>>, <<>>, _, _, Bits) ->
+    Bits;
+compare_constant_binary(<<>>, <<_:8, _/binary>>, _, _, _) ->
+    -1;
+compare_constant_binary(<<C1:8, Test/binary>>, <<>> = Correct,
+                        <<C2:8/signed, Busy/binary>>, BusyEmpty, Bits) ->
+    compare_constant_binary(Test, Correct,
+                            Busy, BusyEmpty, Bits bor (C1 bxor C2));
+compare_constant_binary(<<C1:8, Test/binary>>, <<C2:8, Correct/binary>>,
+                        Busy, <<>> = BusyEmpty, Bits) ->
+    compare_constant_binary(Test, Correct,
+                            Busy, BusyEmpty, Bits bor (C1 bxor C2)).
 
 %%-------------------------------------------------------------------------
 %% @doc
@@ -333,7 +353,6 @@ compare_constant_binary(Test, Correct) ->
             String :: string() | binary()) ->
     string() | binary() | false.
 
--ifdef(ERLANG_OTP_VERSION_20_FEATURES).
 findl(SearchPattern, String) ->
     case string:find(String, SearchPattern, leading) of
         nomatch ->
@@ -341,34 +360,6 @@ findl(SearchPattern, String) ->
         Result ->
             Result
     end.
--else.
-findl(SearchPattern, String)
-    when is_list(SearchPattern) ->
-    StringList = if
-        is_binary(String) ->
-            erlang:binary_to_list(String);
-        is_list(String) ->
-            lists:flatten(String)
-    end,
-    SearchPatternList = if
-        is_binary(SearchPattern) ->
-            erlang:binary_to_list(SearchPattern);
-        is_list(SearchPattern) ->
-            lists:flatten(SearchPattern)
-    end,
-    case string:str(StringList, SearchPatternList) of
-        0 ->
-            false;
-        Index ->
-            Result = lists:nthtail(Index - 1, StringList),
-            if
-                is_binary(String) ->
-                    erlang:list_to_binary(Result);
-                is_list(String) ->
-                    Result
-            end
-    end.
--endif.
 
 %%-------------------------------------------------------------------------
 %% @doc
@@ -380,7 +371,6 @@ findl(SearchPattern, String)
             String :: string() | binary()) ->
     string() | binary() | false.
 
--ifdef(ERLANG_OTP_VERSION_20_FEATURES).
 findr(SearchPattern, String) ->
     case string:find(String, SearchPattern, trailing) of
         nomatch ->
@@ -388,33 +378,6 @@ findr(SearchPattern, String) ->
         Result ->
             Result
     end.
--else.
-findr(SearchPattern, String) ->
-    StringList = if
-        is_binary(String) ->
-            erlang:binary_to_list(String);
-        is_list(String) ->
-            lists:flatten(String)
-    end,
-    SearchPatternList = if
-        is_binary(SearchPattern) ->
-            erlang:binary_to_list(SearchPattern);
-        is_list(SearchPattern) ->
-            lists:flatten(SearchPattern)
-    end,
-    case string:rstr(StringList, SearchPatternList) of
-        0 ->
-            false;
-        Index ->
-            Result = lists:nthtail(Index - 1, StringList),
-            if
-                is_binary(String) ->
-                    erlang:list_to_binary(Result);
-                is_list(String) ->
-                    Result
-            end
-    end.
--endif.
 
 %%-------------------------------------------------------------------------
 %% @doc
@@ -518,17 +481,8 @@ list_to_term(L) ->
 -spec lowercase(String :: string() | binary()) ->
     string() | binary().
 
--ifdef(ERLANG_OTP_VERSION_20_FEATURES).
 lowercase(String) ->
     string:lowercase(String).
--else.
-lowercase(String)
-    when is_list(String) ->
-    string:to_lower(String);
-lowercase(String)
-    when is_binary(String) ->
-    erlang:list_to_binary(string:to_lower(erlang:binary_to_list(String))).
--endif.
 
 %%-------------------------------------------------------------------------
 %% @doc
@@ -540,26 +494,8 @@ lowercase(String)
             String :: string() | binary()) ->
     list(string() | binary()).
 
--ifdef(ERLANG_OTP_VERSION_20_FEATURES).
 split(SearchPattern, String) ->
     string:split(String, SearchPattern, all).
--else.
-split(SearchPattern, String)
-    when is_list(String) ->
-    [erlang:binary_to_list(S)
-     || S <- split(SearchPattern, erlang:list_to_binary(String))];
-split(SearchPattern, String)
-    when is_binary(String) ->
-    Pattern = if
-        is_binary(SearchPattern) ->
-            [SearchPattern];
-        is_integer(hd(SearchPattern)) ->
-            [erlang:list_to_binary(SearchPattern)];
-        is_list(SearchPattern) ->
-            [unicode:characters_to_binary(S) || S <- SearchPattern]
-    end,
-    binary:split(String, Pattern, [global]).
--endif.
 
 %%-------------------------------------------------------------------------
 %% @doc
@@ -665,13 +601,8 @@ splitr_input(L1, L2, Char, [C | Rest], Input) ->
 -spec term_to_binary(T :: any()) ->
     binary().
 
--ifdef(ERLANG_OTP_VERSION_20_FEATURES).
 term_to_binary(T) ->
     unicode:characters_to_binary(io_lib:format("~tw", [T])).
--else.
-term_to_binary(T) ->
-    unicode:characters_to_binary(io_lib:format("~w", [T])).
--endif.
 
 %%-------------------------------------------------------------------------
 %% @doc
@@ -684,7 +615,7 @@ term_to_binary(T) ->
     binary().
 
 term_to_binary_compact(T) ->
-    unicode:characters_to_binary(io_lib:format("~" ?COMPACT_LIMIT "p", [T])).
+    unicode:characters_to_binary(io_lib:format("~" ?COMPACT_LIMIT "tp", [T])).
 
 %%-------------------------------------------------------------------------
 %% @doc
@@ -695,13 +626,8 @@ term_to_binary_compact(T) ->
 -spec term_to_list(T :: any()) ->
     string().
 
--ifdef(ERLANG_OTP_VERSION_20_FEATURES).
 term_to_list(T) ->
     format("~tw", [T]).
--else.
-term_to_list(T) ->
-    format("~w", [T]).
--endif.
 
 %%-------------------------------------------------------------------------
 %% @doc
@@ -713,7 +639,7 @@ term_to_list(T) ->
     string().
 
 term_to_list_compact(T) ->
-    format("~" ?COMPACT_LIMIT "p", [T]).
+    format("~" ?COMPACT_LIMIT "tp", [T]).
 
 %%-------------------------------------------------------------------------
 %% @doc
@@ -724,14 +650,8 @@ term_to_list_compact(T) ->
 -spec titlecase(String :: string() | binary()) ->
     string() | binary().
 
--ifdef(ERLANG_OTP_VERSION_20_FEATURES).
 titlecase(String) ->
     string:titlecase(String).
--else.
-titlecase(String) ->
-    % functionality was not present in any form
-    String.
--endif.
 
 %%-------------------------------------------------------------------------
 %% @doc
@@ -742,13 +662,8 @@ titlecase(String) ->
 -spec trim(String :: string() | binary()) ->
     string() | binary().
 
--ifdef(ERLANG_OTP_VERSION_20_FEATURES).
 trim(String) ->
     string:trim(String).
--else.
-trim(String) ->
-    trim(?WHITESPACE, String).
--endif.
 
 %%-------------------------------------------------------------------------
 %% @doc
@@ -760,23 +675,8 @@ trim(String) ->
            String :: string() | binary()) ->
     string() | binary().
 
--ifdef(ERLANG_OTP_VERSION_20_FEATURES).
 trim(Characters, String) ->
     string:trim(String, both, Characters).
--else.
-trim(Characters, String)
-    when is_list(String) ->
-    trim_list(lists:flatten(Characters), String);
-trim(Characters, String)
-    when is_binary(String) ->
-    erlang:list_to_binary(trim_list(lists:flatten(Characters),
-                                    erlang:binary_to_list(String))).
-
-trim_list([], String) ->
-    String;
-trim_list([H | T], String) ->
-    trim_list(T, string:strip(String, both, H)).
--endif.
 
 %%-------------------------------------------------------------------------
 %% @doc
@@ -787,13 +687,8 @@ trim_list([H | T], String) ->
 -spec triml(String :: string() | binary()) ->
     string() | binary().
 
--ifdef(ERLANG_OTP_VERSION_20_FEATURES).
 triml(String) ->
     string:trim(String, leading).
--else.
-triml(String) ->
-    triml(?WHITESPACE, String).
--endif.
 
 %%-------------------------------------------------------------------------
 %% @doc
@@ -805,23 +700,8 @@ triml(String) ->
             String :: string() | binary()) ->
     string() | binary().
 
--ifdef(ERLANG_OTP_VERSION_20_FEATURES).
 triml(Characters, String) ->
     string:trim(String, leading, Characters).
--else.
-triml(Characters, String)
-    when is_list(String) ->
-    triml_list(lists:flatten(Characters), String);
-triml(Characters, String)
-    when is_binary(String) ->
-    erlang:list_to_binary(triml_list(lists:flatten(Characters),
-                                     erlang:binary_to_list(String))).
-
-triml_list([], String) ->
-    String;
-triml_list([H | T], String) ->
-    triml_list(T, string:strip(String, left, H)).
--endif.
 
 %%-------------------------------------------------------------------------
 %% @doc
@@ -832,13 +712,8 @@ triml_list([H | T], String) ->
 -spec trimr(String :: string() | binary()) ->
     string() | binary().
 
--ifdef(ERLANG_OTP_VERSION_20_FEATURES).
 trimr(String) ->
     string:trim(String, trailing).
--else.
-trimr(String) ->
-    trimr(?WHITESPACE, String).
--endif.
 
 %%-------------------------------------------------------------------------
 %% @doc
@@ -850,23 +725,8 @@ trimr(String) ->
             String :: string() | binary()) ->
     string() | binary().
 
--ifdef(ERLANG_OTP_VERSION_20_FEATURES).
 trimr(Characters, String) ->
     string:trim(String, trailing, Characters).
--else.
-trimr(Characters, String)
-    when is_list(String) ->
-    trimr_list(lists:flatten(Characters), String);
-trimr(Characters, String)
-    when is_binary(String) ->
-    erlang:list_to_binary(trimr_list(lists:flatten(Characters),
-                                     erlang:binary_to_list(String))).
-
-trimr_list([], String) ->
-    String;
-trimr_list([H | T], String) ->
-    trimr_list(T, string:strip(String, right, H)).
--endif.
 
 %%-------------------------------------------------------------------------
 %% @doc
@@ -877,16 +737,45 @@ trimr_list([H | T], String) ->
 -spec uppercase(String :: string() | binary()) ->
     string() | binary().
 
--ifdef(ERLANG_OTP_VERSION_20_FEATURES).
 uppercase(String) ->
     string:uppercase(String).
+
+%%-------------------------------------------------------------------------
+%% @doc
+%% ===Return the string with URI encoding.===
+%% @end
+%%-------------------------------------------------------------------------
+
+-spec uri_encode(URI :: string() | binary()) ->
+    string() | binary().
+
+-ifdef(ERLANG_OTP_VERSION_25_FEATURES).
+uri_encode(URI) ->
+    uri_string:quote(URI).
 -else.
-uppercase(String)
-    when is_list(String) ->
-    string:to_upper(String);
-uppercase(String)
-    when is_binary(String) ->
-    erlang:list_to_binary(string:to_upper(erlang:binary_to_list(String))).
+-compile({nowarn_deprecated_function,
+          [{http_uri, encode, 1}]}).
+uri_encode(URI) ->
+    http_uri:encode(URI).
+-endif.
+
+%%-------------------------------------------------------------------------
+%% @doc
+%% ===Return the string without URI encoding.===
+%% @end
+%%-------------------------------------------------------------------------
+
+-spec uri_decode(String :: string() | binary()) ->
+    string() | binary().
+
+-ifdef(ERLANG_OTP_VERSION_25_FEATURES).
+uri_decode(String) ->
+    uri_string:unquote(String).
+-else.
+-compile({nowarn_deprecated_function,
+          [{http_uri, decode, 1}]}).
+uri_decode(String) ->
+    http_uri:decode(String).
 -endif.
 
 %%%------------------------------------------------------------------------
@@ -896,7 +785,28 @@ uppercase(String)
 -ifdef(TEST).
 -include_lib("eunit/include/eunit.hrl").
 
-afterl_test() ->
+-include("cloudi_core_i_test.hrl").
+
+module_test_() ->
+    {timeout, ?TEST_TIMEOUT, [
+        {"afterl tests", ?_assertOk(t_afterl())},
+        {"afterr tests", ?_assertOk(t_afterr())},
+        {"beforel tests", ?_assertOk(t_beforel())},
+        {"beforer tests", ?_assertOk(t_beforer())},
+        {"compare_constant_binary", ?_assertOk(t_compare_constant_binary())},
+        {"compare_constant_list", ?_assertOk(t_compare_constant_list())},
+        {"findl tests", ?_assertOk(t_findl())},
+        {"findr tests", ?_assertOk(t_findr())},
+        {"join tests", ?_assertOk(t_join())},
+        {"split tests", ?_assertOk(t_split())},
+        {"splitl tests", ?_assertOk(t_splitl())},
+        {"splitr tests", ?_assertOk(t_splitr())},
+        {"trim tests", ?_assertOk(t_trim())},
+        {"triml tests", ?_assertOk(t_triml())},
+        {"trimr tests", ?_assertOk(t_trimr())}
+    ]}.
+
+t_afterl() ->
     "this-is-all-input" = afterl($/, "this-is-all-input", input),
     "" = afterl($/, "this-is-all-input", empty),
     "" = afterl($/, "this-is-all-input"),
@@ -905,7 +815,7 @@ afterl_test() ->
     "is-all-input" = afterl($-, "this-is-all-input"),
     ok.
 
-afterr_test() ->
+t_afterr() ->
     "this-is-all-input" = afterr($/, "this-is-all-input", input),
     "" = afterr($/, "this-is-all-input", empty),
     "" = afterr($/, "this-is-all-input"),
@@ -914,7 +824,7 @@ afterr_test() ->
     "input" = afterr($-, "this-is-all-input"),
     ok.
 
-beforel_test() ->
+t_beforel() ->
     "this-is-all-input" = beforel($/, "this-is-all-input", input),
     "" = beforel($/, "this-is-all-input", empty),
     "" = beforel($/, "this-is-all-input"),
@@ -923,7 +833,7 @@ beforel_test() ->
     "this" = beforel($-, "this-is-all-input"),
     ok.
 
-beforer_test() ->
+t_beforer() ->
     "this-is-all-input" = beforer($/, "this-is-all-input", input),
     "" = beforer($/, "this-is-all-input", empty),
     "" = beforer($/, "this-is-all-input"),
@@ -932,7 +842,23 @@ beforer_test() ->
     "this-is-all" = beforer($-, "this-is-all-input"),
     ok.
 
-findl_test() ->
+t_compare_constant_binary() ->
+    true = compare_constant_binary(<<"abc">>, <<"abc">>),
+    false = compare_constant_binary(<<"abc">>, <<"abd">>),
+    false = compare_constant_binary(<<"abc">>, <<"abcd">>),
+    false = compare_constant_binary(<<"abcde">>, <<"abcd">>),
+    true = compare_constant_binary(<<"abcd">>, <<"abcd">>),
+    ok.
+
+t_compare_constant_list() ->
+    true = compare_constant_list("abc", "abc"),
+    false = compare_constant_list("abc", "abd"),
+    false = compare_constant_list("abc", "abcd"),
+    false = compare_constant_list("abcde", "abcd"),
+    true = compare_constant_list("abcd", "abcd"),
+    ok.
+
+t_findl() ->
     "..cd..ef" = findl(".", "ab..cd..ef"),
     <<"..cd..ef">> = findl(".", <<"ab..cd..ef">>),
     <<"..cd..ef">> = findl("..", <<"ab..cd..ef">>),
@@ -941,7 +867,7 @@ findl_test() ->
     false = findl("x", "ab..cd..ef"),
     ok.
 
-findr_test() ->
+t_findr() ->
     ".ef" = findr(".", "ab..cd..ef"),
     <<".ef">> = findr(".", <<"ab..cd..ef">>),
     <<"..ef">> = findr("..", <<"ab..cd..ef">>),
@@ -950,19 +876,19 @@ findr_test() ->
     false = findr("x", "ab..cd..ef"),
     ok.
 
-join_test() ->
+t_join() ->
     "ab..bc..cd" = join("..", ["ab","bc","cd"]),
     <<"ab..bc..cd">> = join(<<"..">>, [<<"ab">>,<<"bc">>,<<"cd">>]),
     <<"ab..bc....cd">> = join(<<"..">>, [<<"ab">>,<<"bc">>,<<>>,<<"cd">>]),
     ok.
 
-split_test() ->
+t_split() ->
     ["ab","bc","cd"] = split("..", "ab..bc..cd"),
     [<<"ab">>,<<"bc">>,<<"cd">>] = split(<<"..">>, <<"ab..bc..cd">>),
     [<<"ab">>,<<"bc">>,<<>>,<<"cd">>] = split(<<"..">>, <<"ab..bc....cd">>),
     ok.
 
-splitl_test() ->
+t_splitl() ->
     {"this-is-all-input", ""} = splitl($/, "this-is-all-input", input),
     {"", ""} = splitl($/, "this-is-all-input", empty),
     {"", ""} = splitl($/, "this-is-all-input"),
@@ -971,7 +897,7 @@ splitl_test() ->
     {"this", "is-all-input"} = splitl($-, "this-is-all-input"),
     ok.
 
-splitr_test() ->
+t_splitr() ->
     {"", "this-is-all-input"} = splitr($/, "this-is-all-input", input),
     {"", ""} = splitr($/, "this-is-all-input", empty),
     {"", ""} = splitr($/, "this-is-all-input"),
@@ -980,21 +906,21 @@ splitr_test() ->
     {"this-is-all", "input"} = splitr($-, "this-is-all-input"),
     ok.
 
-trim_test() ->
+t_trim() ->
     <<"Hello">> = trim(<<"\t  Hello  \n">>),
     "Hello" = trim("\t  Hello  \n"),
     <<".Hello.">> = trim(<<".Hello.\n">>),
     ".Hello." = trim(".Hello.\n"),
     ok.
 
-triml_test() ->
+t_triml() ->
     <<"Hello  \n">> = triml(<<"\t  Hello  \n">>),
     "Hello  \n" = triml("\t  Hello  \n"),
     <<"Hello.\n">> = triml("\n.", <<".Hello.\n">>),
     "Hello.\n" = triml("\n.", ".Hello.\n"),
     ok.
 
-trimr_test() ->
+t_trimr() ->
     <<"\t  Hello">> = trimr(<<"\t  Hello  \n">>),
     "\t  Hello" = trimr("\t  Hello  \n"),
     <<".Hello">> = trimr("\n.", <<".Hello.\n">>),
