@@ -7,7 +7,7 @@
 %%%
 %%% MIT License
 %%%
-%%% Copyright (c) 2014-2022 Michael Truog <mjtruog at protonmail dot com>
+%%% Copyright (c) 2014-2023 Michael Truog <mjtruog at protonmail dot com>
 %%%
 %%% Permission is hereby granted, free of charge, to any person obtaining a
 %%% copy of this software and associated documentation files (the "Software"),
@@ -28,8 +28,8 @@
 %%% DEALINGS IN THE SOFTWARE.
 %%%
 %%% @author Michael Truog <mjtruog at protonmail dot com>
-%%% @copyright 2014-2022 Michael Truog
-%%% @version 2.0.5 {@date} {@time}
+%%% @copyright 2014-2023 Michael Truog
+%%% @version 2.0.6 {@date} {@time}
 %%%------------------------------------------------------------------------
 
 -module(cloudi_service_SUITE).
@@ -67,6 +67,8 @@
          t_service_internal_terminate_3/1,
          t_service_internal_terminate_4/1,
          t_service_internal_update_1/1,
+         t_service_internal_update_pid_options_1/1,
+         t_service_internal_update_pid_options_2/1,
          t_service_internal_log_1/1,
          t_service_internal_idle_1/1,
          t_service_internal_stop_error_1/1,
@@ -246,6 +248,13 @@ cloudi_service_handle_request(_RequestType, _Name, _Pattern,
                               #state{mode = pid} = State,
                               _Dispatcher) ->
     {stop, Reason, State};
+cloudi_service_handle_request(_RequestType, _Name, _Pattern,
+                              <<>>, service_pids,
+                              _Timeout, _Priority, _TransId, _Pid,
+                              #state{mode = pid} = State,
+                              Dispatcher) ->
+    Service = cloudi_service:self(Dispatcher),
+    {reply, {Dispatcher, Service}, State};
 cloudi_service_handle_request(_RequestType, _Name, _Pattern,
                               ?REQUEST_INFO7, ?REQUEST7,
                               _Timeout, _Priority, _TransId, _Pid,
@@ -326,6 +335,8 @@ groups() ->
        t_service_internal_terminate_3,
        t_service_internal_terminate_4,
        t_service_internal_update_1,
+       t_service_internal_update_pid_options_1,
+       t_service_internal_update_pid_options_2,
        t_service_internal_log_1,
        t_service_internal_idle_1,
        t_service_internal_stop_error_1,
@@ -557,7 +568,9 @@ init_per_testcase(TestCase, Config)
         ], infinity),
     [{service_ids, ServiceIds} | Config];
 init_per_testcase(TestCase, Config)
-    when (TestCase =:= t_service_internal_stop_error_1) orelse
+    when (TestCase =:= t_service_internal_update_pid_options_1) orelse
+         (TestCase =:= t_service_internal_update_pid_options_2) orelse
+         (TestCase =:= t_service_internal_stop_error_1) orelse
          (TestCase =:= t_service_internal_stop_error_2) orelse
          (TestCase =:= t_service_internal_stop_shutdown_1) orelse
          (TestCase =:= t_service_internal_stop_shutdown_2) orelse
@@ -565,6 +578,7 @@ init_per_testcase(TestCase, Config)
          (TestCase =:= t_service_internal_stop_shutdown_6) orelse
          (TestCase =:= t_service_internal_stop_normal_1) orelse
          (TestCase =:= t_service_internal_stop_normal_2) ->
+    DuoMode = (TestCase =:= t_service_internal_update_pid_options_2),
     ok = init_per_testcase(TestCase),
     {ok, ServiceIds} = cloudi_service_api:services_add([
         % using the record configuration format (from cloudi_service_api.hrl)
@@ -573,7 +587,8 @@ init_per_testcase(TestCase, Config)
                   args = [{mode, pid}],
                   options = [{automatic_loading, false},
                              {request_pid_uses, infinity},
-                             {info_pid_uses, infinity}]}
+                             {info_pid_uses, infinity},
+                             {duo_mode, DuoMode}]}
         ], infinity),
     [{service_ids, ServiceIds} | Config];
 init_per_testcase(TestCase, Config)
@@ -662,8 +677,8 @@ t_service_internal_sync_1(_Config) ->
                                    ServiceName,
                                    ?REQUEST2),
     true = cloudi_x_uuid:is_v1(TransId2),
-    true = (cloudi_x_uuid:get_v1_time(TransId2) >
-            cloudi_x_uuid:get_v1_time(TransId1)),
+    true = (cloudi_x_uuid:get_v1_time_ns(TransId2) >
+            cloudi_x_uuid:get_v1_time_ns(TransId1)),
     {error,
      {service_internal_start_failed,
       {error, invalid_state}}} = cloudi_service_api:services_add([
@@ -748,8 +763,8 @@ t_service_internal_async_1(_Config) ->
                                    ?REQUEST2),
     true = (Timeout2 > (TimeoutMax - 1000)) andalso (Timeout2 =< TimeoutMax),
     true = cloudi_x_uuid:is_v1(TransId2),
-    true = (cloudi_x_uuid:get_v1_time(TransId2) >
-            cloudi_x_uuid:get_v1_time(TransId1)),
+    true = (cloudi_x_uuid:get_v1_time_ns(TransId2) >
+            cloudi_x_uuid:get_v1_time_ns(TransId1)),
     {error,
      {service_internal_start_failed,
       {error, invalid_state}}} = cloudi_service_api:services_add([
@@ -783,12 +798,12 @@ t_service_internal_async_2(_Config) ->
      Context4} = cloudi:recv_async(Context3, <<0:128>>),
     {{ok, ?RESPONSE_INFO1, ?RESPONSE1, TransId4},
      Context5} = cloudi:recv_async(Context4, <<0:128>>),
-    true = (cloudi_x_uuid:get_v1_time(TransId1) <
-            cloudi_x_uuid:get_v1_time(TransId2)) andalso
-           (cloudi_x_uuid:get_v1_time(TransId2) <
-            cloudi_x_uuid:get_v1_time(TransId3)) andalso
-           (cloudi_x_uuid:get_v1_time(TransId3) <
-            cloudi_x_uuid:get_v1_time(TransId4)),
+    true = (cloudi_x_uuid:get_v1_time_ns(TransId1) <
+            cloudi_x_uuid:get_v1_time_ns(TransId2)) andalso
+           (cloudi_x_uuid:get_v1_time_ns(TransId2) <
+            cloudi_x_uuid:get_v1_time_ns(TransId3)) andalso
+           (cloudi_x_uuid:get_v1_time_ns(TransId3) <
+            cloudi_x_uuid:get_v1_time_ns(TransId4)),
     {{ok,
       [{'send_async', ServiceName, ServiceName, ?REQUEST_INFO1, ?REQUEST1,
         Timeout1, 0, TransId1, Self},
@@ -821,12 +836,12 @@ t_service_internal_async_2(_Config) ->
        {?RESPONSE_INFO1, ?RESPONSE1, TransId7},
        {?RESPONSE_INFO1, ?RESPONSE1, TransId8}]},
      Context8} = cloudi:recv_asyncs(Context7, TransIds),
-    true = (cloudi_x_uuid:get_v1_time(TransId5) <
-            cloudi_x_uuid:get_v1_time(TransId6)) andalso
-           (cloudi_x_uuid:get_v1_time(TransId6) <
-            cloudi_x_uuid:get_v1_time(TransId7)) andalso
-           (cloudi_x_uuid:get_v1_time(TransId7) <
-            cloudi_x_uuid:get_v1_time(TransId8)),
+    true = (cloudi_x_uuid:get_v1_time_ns(TransId5) <
+            cloudi_x_uuid:get_v1_time_ns(TransId6)) andalso
+           (cloudi_x_uuid:get_v1_time_ns(TransId6) <
+            cloudi_x_uuid:get_v1_time_ns(TransId7)) andalso
+           (cloudi_x_uuid:get_v1_time_ns(TransId7) <
+            cloudi_x_uuid:get_v1_time_ns(TransId8)),
     {{ok,
       [{'send_async', ServiceName, ServiceName, ?REQUEST_INFO1, ?REQUEST1,
         Timeout5, 0, TransId5, Self},
@@ -864,12 +879,12 @@ t_service_internal_async_3(_Config) ->
        {?RESPONSE_INFO1, ?RESPONSE1, TransId3},
        {?RESPONSE_INFO1, ?RESPONSE1, TransId4}]},
      _Context1} = cloudi:send_sync(Context0, ServiceName, ?REQUEST3),
-    true = (cloudi_x_uuid:get_v1_time(TransId1) <
-            cloudi_x_uuid:get_v1_time(TransId2)) andalso
-           (cloudi_x_uuid:get_v1_time(TransId2) <
-            cloudi_x_uuid:get_v1_time(TransId3)) andalso
-           (cloudi_x_uuid:get_v1_time(TransId3) <
-            cloudi_x_uuid:get_v1_time(TransId4)),
+    true = (cloudi_x_uuid:get_v1_time_ns(TransId1) <
+            cloudi_x_uuid:get_v1_time_ns(TransId2)) andalso
+           (cloudi_x_uuid:get_v1_time_ns(TransId2) <
+            cloudi_x_uuid:get_v1_time_ns(TransId3)) andalso
+           (cloudi_x_uuid:get_v1_time_ns(TransId3) <
+            cloudi_x_uuid:get_v1_time_ns(TransId4)),
     true = cloudi_x_uuid:is_v1(TransId1),
     true = cloudi_x_uuid:is_v1(TransId2),
     true = cloudi_x_uuid:is_v1(TransId3),
@@ -1030,6 +1045,215 @@ t_service_internal_update_1(Config) ->
                                    undefined, undefined),
     ok.
 
+t_service_internal_update_pid_options_1(Config) ->
+    t_service_internal_update_pid_options(Config, false).
+
+t_service_internal_update_pid_options_2(Config) ->
+    t_service_internal_update_pid_options(Config, true).
+
+t_service_internal_update_pid_options(Config, DuoMode) ->
+    Context0 = cloudi:new(),
+    ServiceName = ?SERVICE_PREFIX1 ++ ?SERVICE_SUFFIX1,
+    {{ok, {Dispatcher, Service}},
+     Context1} = cloudi:send_sync(Context0,
+                                  ServiceName,
+                                  <<>>, service_pids,
+                                  undefined, undefined),
+    {{ok, RequestPid0},
+     Context2} = cloudi:send_sync(Context1,
+                                  ServiceName,
+                                  <<>>),
+    Service ! {pid, self()},
+    InfoPid0 = receive
+        {pid, InfoPidValue0} ->
+            InfoPidValue0
+    end,
+    if
+        DuoMode =:= true ->
+            true = (Dispatcher /= Service),
+            true = (InfoPid0 =:= Service);
+        DuoMode =:= false ->
+            true = (Dispatcher =:= Service)
+    end,
+    true = (InfoPid0 /= RequestPid0),
+    [{max_heap_size,
+      MaxHeapSizeDefault},
+     {message_queue_data,
+      MessageQueueDataDefault},
+     {priority,
+      PriorityDefault}] = erlang:process_info(Dispatcher,
+                                              [max_heap_size,
+                                               message_queue_data,
+                                               priority]),
+    on_heap = MessageQueueDataDefault,
+    normal = PriorityDefault,
+    [{max_heap_size,
+      MaxHeapSizeDefault},
+     {message_queue_data,
+      MessageQueueDataDefault},
+     {priority,
+      PriorityDefault}] = erlang:process_info(InfoPid0,
+                                              [max_heap_size,
+                                               message_queue_data,
+                                               priority]),
+    [{max_heap_size,
+      MaxHeapSizeDefault},
+     {message_queue_data,
+      MessageQueueDataDefault},
+     {priority,
+      PriorityDefault}] = erlang:process_info(RequestPid0,
+                                              [max_heap_size,
+                                               message_queue_data,
+                                               priority]),
+    DispatcherPidOptions1 = [{max_heap_size,
+                              #{error_logger => true,
+                                kill => true,
+                                size => 1048577}},
+                             {priority, high}],
+    InfoPidOptions1 = [{max_heap_size,
+                        #{error_logger => true,
+                          kill => true,
+                          size => 1048578}},
+                       {message_queue_data, off_heap},
+                       {priority, low}],
+    RequestPidOptions1 = [{max_heap_size,
+                           #{error_logger => true,
+                             kill => true,
+                             size => 1048579}},
+                          {priority, normal}],
+    Options0 = [{dispatcher_pid_options,
+                 [{sensitive, true} | DispatcherPidOptions1]},
+                {info_pid_options, InfoPidOptions1},
+                {request_pid_options,
+                 [{sensitive, false} | RequestPidOptions1]}],
+    [ServiceId] = ?config(service_ids, Config),
+    {ok, [[ServiceId]]} = cloudi_service_api:
+                          services_update([{ServiceId,
+                                            [{module, ?MODULE},
+                                             {options, Options0}]}],
+                                          infinity),
+    {{ok, RequestPid1},
+     Context3} = cloudi:send_sync(Context2,
+                                  ServiceName,
+                                  <<>>),
+    Service ! {pid, self()},
+    InfoPid1 = receive
+        {pid, InfoPidValue1} ->
+            InfoPidValue1
+    end,
+    false = erlang:is_process_alive(RequestPid0),
+    true = erlang:is_process_alive(RequestPid1),
+    if
+        DuoMode =:= true ->
+            % ensure dispatcher_pid_options have been applied
+            _ = sys:get_state(Dispatcher, infinity),
+
+            true = erlang:is_process_alive(InfoPid0),
+            true = (InfoPid0 == InfoPid1);
+        DuoMode =:= false ->
+            false = erlang:is_process_alive(InfoPid0),
+            true = (InfoPid0 /= InfoPid1),
+            true = erlang:is_process_alive(InfoPid1)
+    end,
+    [{max_heap_size,
+      MaxHeapSize1},
+     {priority,
+      high}] = erlang:process_info(Dispatcher,
+                                   [max_heap_size,
+                                    priority]),
+    #{error_logger := true,
+      kill := true,
+      size := 1048577} = MaxHeapSize1,
+    [{max_heap_size,
+      MaxHeapSize2},
+     {message_queue_data,
+      off_heap},
+     {priority,
+      low}] = erlang:process_info(InfoPid1,
+                                  [max_heap_size,
+                                   message_queue_data,
+                                   priority]),
+    #{error_logger := true,
+      kill := true,
+      size := 1048578} = MaxHeapSize2,
+    [{max_heap_size,
+      MaxHeapSize3},
+     {priority,
+      normal}] = erlang:process_info(RequestPid1,
+                                     [max_heap_size,
+                                      priority]),
+    #{error_logger := true,
+      kill := true,
+      size := 1048579} = MaxHeapSize3,
+    DispatcherPidOptions2 = [{priority, normal}],
+    InfoPidOptions2 = [{message_queue_data, off_heap}],
+    RequestPidOptions2 = [{max_heap_size,
+                           #{error_logger => true,
+                             kill => true,
+                             size => 1048579}}],
+    Options1 = [{dispatcher_pid_options, DispatcherPidOptions2},
+                {info_pid_options, InfoPidOptions2},
+                {request_pid_options, RequestPidOptions2}],
+    {ok, [[ServiceId]]} = cloudi_service_api:
+                          services_update([{ServiceId,
+                                            [{module, ?MODULE},
+                                             {options, Options1}]}],
+                                          infinity),
+    {{ok, RequestPid2},
+     _} = cloudi:send_sync(Context3,
+                           ServiceName,
+                           <<>>),
+    Service ! {pid, self()},
+    InfoPid2 = receive
+        {pid, InfoPidValue2} ->
+            InfoPidValue2
+    end,
+    false = erlang:is_process_alive(RequestPid1),
+    true = erlang:is_process_alive(RequestPid2),
+    if
+        DuoMode =:= true ->
+            % ensure dispatcher_pid_options have been applied
+            _ = sys:get_state(Dispatcher, infinity),
+
+            true = erlang:is_process_alive(InfoPid1),
+            true = (InfoPid1 == InfoPid2);
+        DuoMode =:= false ->
+            false = erlang:is_process_alive(InfoPid1),
+            true = (InfoPid1 /= InfoPid2),
+            true = erlang:is_process_alive(InfoPid2)
+    end,
+    [{max_heap_size,
+      MaxHeapSizeDefault},
+     {message_queue_data,
+      MessageQueueDataDefault},
+     {priority,
+      PriorityDefault}] = erlang:process_info(Dispatcher,
+                                              [max_heap_size,
+                                               message_queue_data,
+                                               priority]),
+    [{max_heap_size,
+      MaxHeapSizeDefault},
+     {message_queue_data,
+      off_heap},
+     {priority,
+      PriorityDefault}] = erlang:process_info(InfoPid2,
+                                              [max_heap_size,
+                                               message_queue_data,
+                                               priority]),
+    [{message_queue_data,
+      MessageQueueDataDefault},
+     {priority,
+      PriorityDefault},
+     {max_heap_size,
+      MaxHeapSize4}] = erlang:process_info(RequestPid2,
+                                           [message_queue_data,
+                                            priority,
+                                            max_heap_size]),
+    #{error_logger := true,
+      kill := true,
+      size := 1048579} = MaxHeapSize4,
+    ok.
+
 t_service_internal_log_1(_Config) ->
     ?LOG_METADATA_SET([{test, t_service_internal_log_1},
                        {pid, self()} | ?LOG_METADATA_GET()]),
