@@ -9,7 +9,7 @@
 %%%
 %%% MIT License
 %%%
-%%% Copyright (c) 2019-2021 Michael Truog <mjtruog at protonmail dot com>
+%%% Copyright (c) 2019-2022 Michael Truog <mjtruog at protonmail dot com>
 %%%
 %%% Permission is hereby granted, free of charge, to any person obtaining a
 %%% copy of this software and associated documentation files (the "Software"),
@@ -30,8 +30,8 @@
 %%% DEALINGS IN THE SOFTWARE.
 %%%
 %%% @author Michael Truog <mjtruog at protonmail dot com>
-%%% @copyright 2019-2021 Michael Truog
-%%% @version 2.0.3 {@date} {@time}
+%%% @copyright 2019-2022 Michael Truog
+%%% @version 2.0.5 {@date} {@time}
 %%%------------------------------------------------------------------------
 
 -module(cloudi_service_api_batch).
@@ -65,6 +65,9 @@
         % (n.b., if a service configuration is unable
         %  to start (initialization fails) the queue is
         %  always purged with the error logged).
+        % Any termination of any process (including restarts)
+        % due to an error (Reason is not {shutdown, _} or shutdown)
+        % will cause a purge when the service terminates.
 -define(DEFAULT_QUEUES,                        []).
         % List of {QueueName, Configs} to be used for services_add
 -define(DEFAULT_QUEUE_DEPENDENCIES,            []).
@@ -104,7 +107,9 @@
         terminate = false
             :: boolean(),
         terminate_timer = undefined
-            :: undefined | reference()
+            :: undefined | reference(),
+        terminate_purge = false
+            :: boolean()
     }).
 
 -record(state,
@@ -143,7 +148,7 @@
     nonempty_list(cloudi_service_api:service_internal() |
                   cloudi_service_api:service_external() |
                   cloudi_service_api:service_proplist()).
--type timeout_milliseconds() :: cloudi:timeout_milliseconds().
+-type timeout_period() :: cloudi:timeout_period().
 -type module_response(Result) ::
     {{ok, Result}, NewAgent :: agent()} |
     {{error, cloudi:error_reason()}, NewAgent :: agent()}.
@@ -161,7 +166,7 @@ queue_clear(Agent, Prefix, [I | _] = QueueName)
 -spec queue_clear(Agent :: agent(),
                   Prefix :: service_name(),
                   QueueName :: nonempty_string(),
-                  Timeout :: timeout_milliseconds()) ->
+                  Timeout :: timeout_period()) ->
     module_response(ok | {error, not_found}).
 
 queue_clear(Agent, Prefix, [I | _] = QueueName, Timeout)
@@ -184,7 +189,7 @@ services_add(Agent, Prefix, [I | _] = QueueName, [_ | _] = Configs)
                    Prefix :: service_name(),
                    QueueName :: nonempty_string(),
                    Configs :: service_configurations(),
-                   Timeout :: timeout_milliseconds()) ->
+                   Timeout :: timeout_period()) ->
     module_response(CountQueued :: non_neg_integer() | {error, purged}).
 
 services_add(Agent, Prefix, [I | _] = QueueName, [_ | _] = Configs, Timeout)
@@ -205,7 +210,7 @@ services_remove(Agent, Prefix, [I | _] = QueueName)
 -spec services_remove(Agent :: agent(),
                       Prefix :: service_name(),
                       QueueName :: nonempty_string(),
-                      Timeout :: timeout_milliseconds()) ->
+                      Timeout :: timeout_period()) ->
     module_response(ok | {error, not_found}).
 
 services_remove(Agent, Prefix, [I | _] = QueueName, Timeout)
@@ -226,7 +231,7 @@ services_restart(Agent, Prefix, [I | _] = QueueName)
 -spec services_restart(Agent :: agent(),
                        Prefix :: service_name(),
                        QueueName :: nonempty_string(),
-                       Timeout :: timeout_milliseconds()) ->
+                       Timeout :: timeout_period()) ->
     module_response(ok | {error, not_found | not_running}).
 
 services_restart(Agent, Prefix, [I | _] = QueueName, Timeout)
@@ -302,7 +307,7 @@ cloudi_service_init(Args, Prefix, _Timeout, Dispatcher) ->
 
 cloudi_service_handle_request(_RequestType, Name, Pattern,
                               _RequestInfo, Request,
-                              _Timeout, _Priority, _TransId, _Pid,
+                              _Timeout, _Priority, _TransId, _Source,
                               State, _Dispatcher) ->
     {Response,
      StateNew} = case cloudi_service_name:parse_with_suffix(Name, Pattern) of
@@ -350,13 +355,13 @@ cloudi_service_handle_info({aspects_terminate_before,
                             QueueName, Reason, TimeoutTerminate},
                            State, _Dispatcher) ->
     {noreply, running_terminate(QueueName, Reason, TimeoutTerminate, State)};
-cloudi_service_handle_info({terminate, QueueName, Reason, Timeout},
+cloudi_service_handle_info({terminate, QueueName, Timeout},
                            State, _Dispatcher) ->
-    {noreply, running_stopping(QueueName, Reason, Timeout, State)};
-cloudi_service_handle_info({terminated, QueueName, Reason},
-                           State, _Dispatcher) ->
-    QueuePurge = batch_queue_purge(Reason, State),
-    case running_stopped(QueuePurge, QueueName, State) of
+    {noreply, running_stopping(QueueName, Timeout, State)};
+cloudi_service_handle_info({terminated, QueueName},
+                           #state{queues = Queues} = State, _Dispatcher) ->
+    Queue = cloudi_x_trie:fetch(QueueName, Queues),
+    case running_stopped(Queue, QueueName, State) of
         #state{stop_when_done = true,
                queue_count = 0} = StateNew ->
             {stop, shutdown, StateNew};
@@ -484,7 +489,7 @@ batch_queue_resume_list([QueueName | QueueNameList],
                         true ->
                             State;
                         false ->
-                            batch_queue_run_old(QueueName, Queue, State)
+                            batch_queue_run_old(Queue, QueueName, State)
                     end;
                 Suspended =:= false ->
                     State
@@ -673,32 +678,52 @@ running_init(QueueName, TimeoutInit,
 
 running_terminate(QueueName, Reason, TimeoutTerminate,
                   #state{service = Service,
+                         purge_on_error = PurgeOnError,
                          queues = Queues} = State) ->
+    TerminatePurge = if
+        PurgeOnError =:= true ->
+            case Reason of
+                {shutdown, _} ->
+                    false;
+                shutdown ->
+                    false;
+                _ ->
+                    true
+            end;
+        PurgeOnError =:= false ->
+            false
+    end,
     case cloudi_x_trie:find(QueueName, Queues) of
-        {ok, #queue{terminate = true}} ->
-            State;
+        {ok, #queue{terminate = true,
+                    terminate_purge = TerminatePurgeOld} = Queue} ->
+            TerminatePurgeNew = TerminatePurge or TerminatePurgeOld,
+            QueueNew = Queue#queue{terminate_purge = TerminatePurgeNew},
+            State#state{queues = cloudi_x_trie:store(QueueName,
+                                                     QueueNew, Queues)};
         {ok, #queue{timeout_init = TimeoutInit,
-                    terminate = false} = Queue} ->
+                    terminate = false,
+                    terminate_purge = TerminatePurgeOld} = Queue} ->
             Timeout = TimeoutTerminate + TimeoutInit + ?TIMEOUT_DELTA,
             TimeoutNew = Timeout - ?TERMINATE_INTERVAL,
             TerminateTimer = if
                 TimeoutNew > 0 ->
                     erlang:send_after(?TERMINATE_INTERVAL, Service,
-                                      {terminate, QueueName, Reason,
-                                       TimeoutNew});
+                                      {terminate, QueueName, TimeoutNew});
                 true ->
                     erlang:send_after(Timeout, Service,
-                                      {terminated, QueueName, Reason})
+                                      {terminated, QueueName})
             end,
+            TerminatePurgeNew = TerminatePurge or TerminatePurgeOld,
             QueueNew = Queue#queue{terminate = true,
-                                   terminate_timer = TerminateTimer},
+                                   terminate_timer = TerminateTimer,
+                                   terminate_purge = TerminatePurgeNew},
             State#state{queues = cloudi_x_trie:store(QueueName,
                                                      QueueNew, Queues)};
         error ->
             State
     end.
 
-running_stopping(QueueName, Reason, Timeout,
+running_stopping(QueueName, Timeout,
                  #state{service = Service,
                         queues = Queues} = State) ->
     Queue = cloudi_x_trie:fetch(QueueName, Queues),
@@ -718,22 +743,18 @@ running_stopping(QueueName, Reason, Timeout,
     end,
     if
         Running =:= false ->
-            QueuePurge = batch_queue_purge(Reason, State),
-            QueueNew = Queue#queue{terminate = true,
-                                   terminate_timer = undefined},
-            QueuesNew = cloudi_x_trie:store(QueueName, QueueNew, Queues),
-            running_stopped(QueuePurge, QueueName,
-                            State#state{queues = QueuesNew});
+            running_stopped(Queue#queue{terminate = true,
+                                        terminate_timer = undefined},
+                            QueueName, State);
         Terminate =:= true ->
             TimeoutNew = Timeout - ?TERMINATE_INTERVAL,
             TerminateTimer = if
                 TimeoutNew > 0 ->
                     erlang:send_after(?TERMINATE_INTERVAL, Service,
-                                      {terminate, QueueName, Reason,
-                                       TimeoutNew});
+                                      {terminate, QueueName, TimeoutNew});
                 true ->
                     erlang:send_after(Timeout, Service,
-                                      {terminated, QueueName, Reason})
+                                      {terminated, QueueName})
             end,
             QueueNew = Queue#queue{terminate_timer = TerminateTimer},
             State#state{queues = cloudi_x_trie:store(QueueName,
@@ -743,25 +764,19 @@ running_stopping(QueueName, Reason, Timeout,
             State
     end.
 
-running_stopped(true, QueueName,
+running_stopped(#queue{terminate = false} = Queue, QueueName,
+                #state{queues = Queues} = State) ->
+    % terminate_timer was cancelled after the message was queued
+    State#state{queues = cloudi_x_trie:store(QueueName, Queue, Queues)};
+running_stopped(#queue{terminate_purge = true}, QueueName, State) ->
+    batch_queue_erase(QueueName, State);
+running_stopped(#queue{count = Count,
+                       terminate_purge = false} = Queue, QueueName,
                 #state{queues = Queues} = State) ->
-    #queue{terminate = Terminate} = cloudi_x_trie:fetch(QueueName, Queues),
     if
-        Terminate =:= true ->
+        Count == 0 ->
             batch_queue_erase(QueueName, State);
-        Terminate =:= false ->
-            % terminate_timer was cancelled after the message was queued
-            State
-    end;
-running_stopped(false, QueueName,
-                #state{queues = Queues} = State) ->
-    case cloudi_x_trie:fetch(QueueName, Queues) of
-        #queue{terminate = false} ->
-            % terminate_timer was cancelled after the message was queued
-            State;
-        #queue{count = 0} ->
-            batch_queue_erase(QueueName, State);
-        #queue{} = Queue ->
+        true ->
             case batch_queue_suspended(QueueName, State) of
                 true ->
                     QueueNew = Queue#queue{service_id = undefined,
@@ -770,22 +785,10 @@ running_stopped(false, QueueName,
                                                     QueueNew, Queues),
                     State#state{queues = QueuesNew};
                 false ->
-                    batch_queue_run_old(QueueName, Queue, State)
+                    batch_queue_run_old(Queue, QueueName, State)
             end
     end.
 
-batch_queue_purge(Reason, #state{purge_on_error = true}) ->
-    case Reason of
-        {shutdown, _} ->
-            false;
-        shutdown ->
-            false;
-        _ ->
-            true
-    end;
-batch_queue_purge(_, #state{purge_on_error = false}) ->
-    false.
-
 batch_queue_run_new(QueueName, [Config | ConfigsTail],
                     #state{service = Service,
                            queue_count = QueueCount,
@@ -805,10 +808,9 @@ batch_queue_run_new(QueueName, [Config | ConfigsTail],
             {{error, purged}, State}
     end.
 
-batch_queue_run_old(QueueName,
-                    #queue{count = Count,
+batch_queue_run_old(#queue{count = Count,
                            service_id = ServiceId,
-                           suspended = true} = Queue,
+                           suspended = true} = Queue, QueueName,
                     #state{queues = Queues} = State)
     when is_binary(ServiceId) ->
     case cloudi_service_api:services_resume([ServiceId], infinity) of
@@ -821,14 +823,12 @@ batch_queue_run_old(QueueName,
                 Count == 0 ->
                     batch_queue_erase(QueueName, State);
                 Count > 0 ->
-                    batch_queue_run_old(QueueName,
-                                        Queue#queue{service_id = undefined},
-                                        State)
+                    batch_queue_run_old(Queue#queue{service_id = undefined},
+                                        QueueName, State)
             end
     end;
-batch_queue_run_old(QueueName,
-                    #queue{count = Count,
-                           data = Data} = Queue,
+batch_queue_run_old(#queue{count = Count,
+                           data = Data} = Queue, QueueName,
                     #state{service = Service,
                            queues = Queues} = State) ->
     {{value, Config}, DataNew} = queue:out(Data),
