@@ -9,7 +9,7 @@
 %%%
 %%% MIT License
 %%%
-%%% Copyright (c) 2015-2017 Michael Truog <mjtruog at gmail dot com>
+%%% Copyright (c) 2015-2020 Michael Truog <mjtruog at protonmail dot com>
 %%%
 %%% Permission is hereby granted, free of charge, to any person obtaining a
 %%% copy of this software and associated documentation files (the "Software"),
@@ -29,13 +29,13 @@
 %%% FROM, OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER
 %%% DEALINGS IN THE SOFTWARE.
 %%%
-%%% @author Michael Truog <mjtruog [at] gmail (dot) com>
-%%% @copyright 2015-2017 Michael Truog
-%%% @version 1.7.1 {@date} {@time}
+%%% @author Michael Truog <mjtruog at protonmail dot com>
+%%% @copyright 2015-2020 Michael Truog
+%%% @version 2.0.1 {@date} {@time}
 %%%------------------------------------------------------------------------
 
 -module(cloudi_service_monitoring_erlang).
--author('mjtruog [at] gmail (dot) com').
+-author('mjtruog at protonmail dot com').
 
 %% external interface
 -export([basic_update/0,
@@ -55,13 +55,37 @@
 %%%------------------------------------------------------------------------
 
 basic_update() ->
-    {_, ErrorLoggerN} = erlang:process_info(whereis(error_logger),
-                                            message_queue_len),
+    MetricValues0 = case erlang:whereis(error_logger) of
+        undefined ->
+            [];
+        ErrorLoggerPid ->
+            ErrorLoggerLength = element(2,
+                erlang:process_info(ErrorLoggerPid, message_queue_len)),
+            [metric(gauge, [error_logger, message_queue_len],
+                    ErrorLoggerLength)]
+    end,
+    MetricValues1 = case erlang:whereis(logger_proxy) of
+        undefined ->
+            MetricValues0;
+        LoggerProxyPid ->
+            LoggerProxyLength = element(2,
+                erlang:process_info(LoggerProxyPid, message_queue_len)),
+            [metric(gauge, [logger_proxy, message_queue_len],
+                    LoggerProxyLength) | MetricValues0]
+    end,
+    MetricValuesN = case erlang:whereis(logger) of
+        undefined ->
+            MetricValues1;
+        LoggerPid ->
+            LoggerLength = element(2,
+                erlang:process_info(LoggerPid, message_queue_len)),
+            [metric(gauge, [logger, message_queue_len],
+                    LoggerLength) | MetricValues1]
+    end,
     [metric(gauge, [code, modules],
             erlang:length(code:all_loaded())),
      metric(gauge, [ets, tables],
-            erlang:length(ets:all())),
-     metric(gauge, [error_logger, message_queue_len], ErrorLoggerN)].
+            erlang:length(ets:all())) | MetricValuesN].
 
 memory_init(Options) ->
     Options.
@@ -75,7 +99,7 @@ memory_update(Options) ->
     {MetricValues, Options}.
 
 system_info_init(Options) ->
-    [system_info_value(Option) || Option <- Options],
+    _ = [system_info_value(Option) || Option <- Options],
     Options.
 
 system_info_update(Options) ->
