@@ -73,9 +73,8 @@ cloudi_service_terminate(_Reason, _Timeout, _State) ->
 %%%------------------------------------------------------------------------
 
 all() ->
-    test_conditions_merge([
-        %test_condition_riak([{group, riak}]),
-        test_condition_pgsql([{group, pgsql}])]).
+    test_conditions_merge([%test_condition_riak([{group, riak}]),
+                           test_condition_pgsql([{group, pgsql}])]).
 
 groups() ->
     [{pgsql, [],
@@ -216,7 +215,7 @@ t_example_without_db_1(_Config) ->
     %    oauth_signature="74KNZJeDHnMBp0EMJ9ZHt%2FXKycU%3D"
     Name1 = "/initiate/post",
     RequestInfo1 = [
-        {<<"url-path">>, <<"/initiate">>}, % <- from cloudi_service_http_cowboy1
+        {<<"url-path">>, <<"/initiate">>}, % <- from cloudi_service_http_cowboy
         {<<"host">>, <<"photos.example.net">>},
         {<<"authorization">>,
          <<"OAuth realm=\"Photos\","
@@ -247,7 +246,7 @@ t_example_without_db_1(_Config) ->
     % https://photos.example.net/authorize?oauth_token=hh5s93j4hdidpola
     Name2 = "/authorize/get",
     RequestInfo2 = [
-        {<<"url-path">>, <<"/authorize">>}, % <- from cloudi_service_http_cowboy1
+        {<<"url-path">>, <<"/authorize">>}, % <- from cloudi_service_http_cowboy
         {<<"host">>, <<"photos.example.net">>}],
     Request2 = [{<<"oauth_token">>, <<"hh5s93j4hdidpola">>}],
 
@@ -277,7 +276,7 @@ t_example_without_db_1(_Config) ->
     %    oauth_signature="gKgrFCywp7rO0OXSjdot%2FIHF7IU%3D"
     Name3 = "/token/post",
     RequestInfo3 = [
-        {<<"url-path">>, <<"/token">>}, % <- from cloudi_service_http_cowboy1
+        {<<"url-path">>, <<"/token">>}, % <- from cloudi_service_http_cowboy
         {<<"host">>, <<"photos.example.net">>},
         {<<"authorization">>,
          <<"OAuth realm=\"Photos\","
@@ -361,7 +360,7 @@ t_example_with_db_1(_Config) ->
     %    oauth_signature="74KNZJeDHnMBp0EMJ9ZHt%2FXKycU%3D"
     Name1 = "/initiate/post",
     RequestInfo1 = [
-        {<<"url-path">>, <<"/initiate">>}, % <- from cloudi_service_http_cowboy1
+        {<<"url-path">>, <<"/initiate">>}, % <- from cloudi_service_http_cowboy
         {<<"host">>, <<"photos.example.net">>},
         {<<"authorization">>,
          <<"OAuth realm=\"Photos\","
@@ -390,7 +389,7 @@ t_example_with_db_1(_Config) ->
     % https://photos.example.net/authorize?oauth_token=hh5s93j4hdidpola
     Name2 = "/authorize/get",
     RequestInfo2 = [
-        {<<"url-path">>, <<"/authorize">>}, % <- from cloudi_service_http_cowboy1
+        {<<"url-path">>, <<"/authorize">>}, % <- from cloudi_service_http_cowboy
         {<<"host">>, <<"photos.example.net">>}],
     Request2 = [{<<"oauth_token">>, TokenRequest}],
 
@@ -433,7 +432,7 @@ t_example_with_db_1(_Config) ->
     %    oauth_signature="gKgrFCywp7rO0OXSjdot%2FIHF7IU%3D"
     Name3 = "/token/post",
     RequestInfo3 = [
-        {<<"url-path">>, <<"/token">>}, % <- from cloudi_service_http_cowboy1
+        {<<"url-path">>, <<"/token">>}, % <- from cloudi_service_http_cowboy
         {<<"host">>, <<"photos.example.net">>},
         {<<"authorization">>,
          <<(<<"OAuth realm=\"Photos\","
@@ -488,7 +487,7 @@ t_example_with_db_1(_Config) ->
     Name4 = "/verify/photos/get",
     RequestInfo4 = [
         {<<"url-path">>,
-         <<"/verify/photos">>}, % <- from cloudi_service_http_cowboy1
+         <<"/verify/photos">>}, % <- from cloudi_service_http_cowboy
         {<<"host">>, <<"photos.example.net">>},
         {<<"authorization">>,
          <<(<<"OAuth realm=\"Photos\","
@@ -577,6 +576,14 @@ location_verifier(Location, Callback, TokenRequest) ->
      <<"oauth_verifier=", Verifier/binary>>] = binary:split(Location, <<"&">>),
     Verifier.
 
+test_conditions_merge(L) ->
+    case test_conditions_merge(L, []) of
+        [] ->
+            {skip, L};
+        [_ | _] = NewL ->
+            NewL
+    end.
+
 test_conditions_merge([], Output) ->
     lists:reverse(Output);
 test_conditions_merge([{skip, _} | L], Output) ->
@@ -584,20 +591,10 @@ test_conditions_merge([{skip, _} | L], Output) ->
 test_conditions_merge([Group | L], Output) ->
     test_conditions_merge(L, [Group | Output]).
 
-test_conditions_merge(L) ->
-    if
-        ?CLOUDI_LONG_TEST_TIMEOUT > 0 ->
-            case test_conditions_merge(L, []) of
-                [] ->
-                    {skip, L};
-                [_ | _] = NewL ->
-                    NewL
-            end;
-        ?CLOUDI_LONG_TEST_TIMEOUT =:= 0 ->
-            {skip, long_tests_disabled}
-    end.
-
-test_condition(L, Host, Port, ErrorReason) ->
+test_condition_db(_, _, _, _, 0) ->
+    {skip, long_tests_disabled};
+test_condition_db(L, Host, Port, ErrorReason, LongTestTimeout)
+    when LongTestTimeout > 0 ->
     case gen_tcp:connect(Host, Port, []) of
         {ok, Socket} ->
             catch gen_tcp:close(Socket),
@@ -613,8 +610,10 @@ test_condition(L, Host, Port, ErrorReason) ->
     end.
 
 test_condition_pgsql(L) ->
-    test_condition(L, ?DEFAULT_PGSQL_HOST, ?DEFAULT_PGSQL_PORT, pgsql_dead).
+    test_condition_db(L, ?DEFAULT_PGSQL_HOST, ?DEFAULT_PGSQL_PORT,
+                      pgsql_dead, ?CLOUDI_LONG_TEST_TIMEOUT).
 
 test_condition_riak(L) ->
-    test_condition(L, ?DEFAULT_RIAK_HOST, ?DEFAULT_RIAK_PORT, riak_dead).
+    test_condition_db(L, ?DEFAULT_RIAK_HOST, ?DEFAULT_RIAK_PORT,
+                      riak_dead, ?CLOUDI_LONG_TEST_TIMEOUT).
 
