@@ -8,7 +8,7 @@
 %%%
 %%% MIT License
 %%%
-%%% Copyright (c) 2013-2020 Michael Truog <mjtruog at protonmail dot com>
+%%% Copyright (c) 2013-2021 Michael Truog <mjtruog at protonmail dot com>
 %%%
 %%% Permission is hereby granted, free of charge, to any person obtaining a
 %%% copy of this software and associated documentation files (the "Software"),
@@ -29,8 +29,8 @@
 %%% DEALINGS IN THE SOFTWARE.
 %%%
 %%% @author Michael Truog <mjtruog at protonmail dot com>
-%%% @copyright 2013-2020 Michael Truog
-%%% @version 1.8.1 {@date} {@time}
+%%% @copyright 2013-2021 Michael Truog
+%%% @version 2.0.3 {@date} {@time}
 %%%------------------------------------------------------------------------
 
 -module(cloudi_service_tcp).
@@ -358,10 +358,10 @@ cloudi_service_handle_info({inet_async, Listener, Acceptor, {ok, Socket}},
             SourceAddressFormatted = cloudi_ip_address:to_binary(SourceAddress),
             SourcePortFormatted = erlang:integer_to_binary(SourcePort),
             RequestInfo = cloudi_request_info:key_value_new(
-                [{<<"source_address">>, SourceAddressFormatted},
-                 {<<"source_port">>, SourcePortFormatted},
-                 {<<"destination_address">>, DestinationAddressFormatted},
-                 {<<"destination_port">>, DestinationPortFormatted}]),
+                [{<<"source-address">>, SourceAddressFormatted},
+                 {<<"source-port">>, SourcePortFormatted},
+                 {<<"destination-address">>, DestinationAddressFormatted},
+                 {<<"destination-port">>, DestinationPortFormatted}]),
             SocketPid = proc_lib:spawn_opt(fun() ->
                 StateSocket = #state_socket{
                                   socket = Socket,
@@ -384,9 +384,11 @@ cloudi_service_handle_info({inet_async, Listener, Acceptor, {ok, Socket}},
             end, [link]),
             case gen_tcp:controlling_process(Socket, SocketPid) of
                 ok = InitSuccess ->
-                    SocketPid ! {init, InitSuccess};
+                    SocketPid ! {init, InitSuccess},
+                    ok;
                 {error, _} = InitError ->
-                    SocketPid ! {init, InitError}
+                    SocketPid ! {init, InitError},
+                    ok
             end,
             ConnectionCount + 1;
         {error, Reason} ->
@@ -757,6 +759,10 @@ socket_loop(#state_socket{
             socket_loop_terminate(normal, State)
     end.
 
+-spec socket_loop_terminate(Reason :: atom(),
+                            #state_socket{}) ->
+    no_return().
+
 socket_loop_terminate(Reason,
                       #state_socket{
                           socket = Socket,
@@ -768,9 +774,10 @@ socket_loop_terminate(Reason,
                           debug_level = DebugLevel}) ->
     if
         is_list(DestinationDisconnect) ->
-            send_async_minimal(Dispatcher, DestinationDisconnect,
-                               RequestInfo, <<"DISCONNECT">>,
-                               TimeoutAsync, self());
+            _ = send_async_minimal(Dispatcher, DestinationDisconnect,
+                                   RequestInfo, <<"DISCONNECT">>,
+                                   TimeoutAsync, self()),
+            ok;
         true ->
             ok
     end,
