@@ -8,7 +8,7 @@
 %%%
 %%% MIT License
 %%%
-%%% Copyright (c) 2014-2017 Michael Truog <mjtruog at gmail dot com>
+%%% Copyright (c) 2014-2022 Michael Truog <mjtruog at protonmail dot com>
 %%%
 %%% Permission is hereby granted, free of charge, to any person obtaining a
 %%% copy of this software and associated documentation files (the "Software"),
@@ -28,13 +28,13 @@
 %%% FROM, OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER
 %%% DEALINGS IN THE SOFTWARE.
 %%%
-%%% @author Michael Truog <mjtruog [at] gmail (dot) com>
-%%% @copyright 2014-2017 Michael Truog
-%%% @version 1.7.1 {@date} {@time}
+%%% @author Michael Truog <mjtruog at protonmail dot com>
+%%% @copyright 2014-2022 Michael Truog
+%%% @version 2.0.5 {@date} {@time}
 %%%------------------------------------------------------------------------
 
 -module(erlang_term).
--author('mjtruog [at] gmail (dot) com').
+-author('mjtruog at protonmail dot com').
 
 %% external interface
 -export([byte_size/1,
@@ -49,6 +49,11 @@
 -ifdef(ERLANG_OTP_VERSION_16).
 -else.
 -define(ERLANG_OTP_VERSION_17_FEATURES, undefined).
+-ifdef(OTP_RELEASE). % Erlang/OTP >= 21.0
+-if(?OTP_RELEASE >= 25).
+-define(ERLANG_OTP_VERSION_25_FEATURES, true).
+-endif.
+-endif.
 -endif.
 -ifdef(ERLANG_OTP_VERSION_17_FEATURES).
 -define(BYTE_SIZE_TERMS_MAP,
@@ -71,6 +76,23 @@ byte_size_terms(Term)
     ok,
     ).
 -endif.
+-ifdef(ERLANG_OTP_VERSION_25_FEATURES).
+-define(INTERNAL_TEST_TUPLE_EMPTY,
+    8 = byte_size({}, 8),
+    32 = byte_size(#{}, 8),
+    ).
+-else.
+-ifdef(ERLANG_OTP_VERSION_17_FEATURES).
+-define(INTERNAL_TEST_TUPLE_EMPTY,
+    16 = byte_size({}, 8),
+    40 = byte_size(#{}, 8),
+    ).
+-else.
+-define(INTERNAL_TEST_TUPLE_EMPTY,
+    16 = byte_size({}, 8),
+    ).
+-endif.
+-endif.
 
 %%%------------------------------------------------------------------------
 %%% External interface functions
@@ -141,7 +163,18 @@ byte_size_term_global(_) ->
 -ifdef(TEST).
 -include_lib("eunit/include/eunit.hrl").
 
-internal_test() ->
+-ifdef(CLOUDI_TEST_TIMEOUT).
+-define(TEST_TIMEOUT, ?CLOUDI_TEST_TIMEOUT). % seconds
+-else.
+-define(TEST_TIMEOUT, 10). % seconds
+-endif.
+
+module_test_() ->
+    {timeout, ?TEST_TIMEOUT, [
+        {"internal tests", ?_assertEqual(ok, t_basic())}
+    ]}.
+
+t_basic() ->
     RefcBinary = <<1:((?HEAP_BINARY_LIMIT + 1) * 8)>>,
     HeapBinary = <<1:(?HEAP_BINARY_LIMIT * 8)>>,
     true = (7 == (1 + erts_debug:flat_size(RefcBinary))),
@@ -157,11 +190,12 @@ internal_test() ->
     8 = byte_size([], 8),
     24 = byte_size([0|[]], 8),
     24 = byte_size([1|2], 8), % improper list
-    16 = byte_size({}, 8),
     24 = byte_size({0}, 8),
+    32 = byte_size({0,1}, 8),
     8 = byte_size(0, 8),
     8 = byte_size(erlang:self(), 8),
     8 = byte_size(atom, 8),
+    ?INTERNAL_TEST_TUPLE_EMPTY
     ?INTERNAL_TEST_MAP
     ok.
 
