@@ -20,7 +20,7 @@
 %%%
 %%% MIT License
 %%%
-%%% Copyright (c) 2010-2021 Michael Truog <mjtruog at protonmail dot com>
+%%% Copyright (c) 2010-2022 Michael Truog <mjtruog at protonmail dot com>
 %%%
 %%% Permission is hereby granted, free of charge, to any person obtaining a
 %%% copy of this software and associated documentation files (the "Software"),
@@ -41,8 +41,8 @@
 %%% DEALINGS IN THE SOFTWARE.
 %%%
 %%% @author Michael Truog <mjtruog at protonmail dot com>
-%%% @copyright 2010-2021 Michael Truog
-%%% @version 2.0.3 {@date} {@time}
+%%% @copyright 2010-2022 Michael Truog
+%%% @version 2.0.5 {@date} {@time}
 %%%------------------------------------------------------------------------
 
 -module(trie).
@@ -74,6 +74,7 @@
          foreach/2,
          from_list/1,
          is_bytestring/1,
+         is_bytestring_nonempty/1,
          is_key/2,
          is_pattern/1,
          is_pattern_bytes/1,
@@ -738,6 +739,21 @@ is_bytestring([C | L])
 is_bytestring(_) ->
     false.
 
+%%-------------------------------------------------------------------------
+%% @doc
+%% ===Test if the parameter is a nonempty byte string.===
+%% @end
+%%-------------------------------------------------------------------------
+
+-spec is_bytestring_nonempty(L :: nonempty_list(byte())) -> 'true' | 'false'.
+
+is_bytestring_nonempty([C | L])
+    when C >= 0 andalso C =< 255 ->
+    is_bytestring(L);
+
+is_bytestring_nonempty(_) ->
+    false.
+
 %%-------------------------------------------------------------------------
 %% @doc
 %% ===Test to determine if a string is a pattern.===
@@ -2235,6 +2251,10 @@ test() ->
                                                     [2, 4, 5, 1], true),
     {error, {FillError2, 6}} = trie:pattern2_fill(FillPattern1, Parameters1,
                                                   [2, 4, 6], true),
+    error = trie:pattern2_parse("?/","//"),
+    error = trie:find_match2("//", trie:new(["?/"])),
+    error = trie:find_match2("//", trie:new(["?a","?/"])),
+    error = trie:find_match2("//b", trie:new(["?a","?/b"])),
     ok.
 
 %%%------------------------------------------------------------------------
@@ -2368,9 +2388,14 @@ wildcard_match2_lists([_ | _], [C | _])
 wildcard_match2_lists([$?], [_ | _]) ->
     erlang:exit(badarg);
 
-wildcard_match2_lists([$?, C | Pattern], [_ | L]) ->
+wildcard_match2_lists([$?, C | Pattern], [H | L]) ->
     true = (C =/= $*) andalso (C =/= $?),
-    wildcard_match2_lists_pattern0(Pattern, C, L);
+    if
+        C == H ->
+            false;
+        true ->
+            wildcard_match2_lists_pattern0(Pattern, C, L)
+    end;
 
 wildcard_match2_lists([$*], [_ | L]) ->
     wildcard_match2_lists_valid(L, true);
