@@ -3,7 +3,7 @@
 #
 # MIT License
 #
-# Copyright (c) 2017-2020 Michael Truog <mjtruog at protonmail dot com>
+# Copyright (c) 2017-2022 Michael Truog <mjtruog at protonmail dot com>
 #
 # Permission is hereby granted, free of charge, to any person obtaining a
 # copy of this software and associated documentation files (the "Software"),
@@ -35,9 +35,9 @@ require CloudI::TerminateException;
 sub new
 {
     my $class = shift;
-    my ($api, $thread_index) = @_;
+    my ($thread_index) = @_;
     my $self = bless {
-        api => $api,
+        api => '',
         thread_index => $thread_index,
         count => 0,
     }, $class;
@@ -49,6 +49,7 @@ sub run
     my $self = shift;
     eval
     {
+        $self->{api} = CloudI::API->new($self->{thread_index});
         $self->{api}->subscribe('perl/get', $self, '_request');
         my $result = $self->{api}->poll();
         assert($result == 0);
@@ -62,7 +63,7 @@ sub run
         }
         else
         {
-            print "$e";
+            print STDERR "$e";
         }
     }
     print "terminate count perl\n";
@@ -71,14 +72,14 @@ sub run
 sub assert
 {
     my ($test) = @_;
-    CloudI::API->assert($test);
+    CloudI::API::assert($test);
 }
 
 sub _request
 {
     my $self = shift;
     my ($request_type, $name, $pattern, $request_info, $request,
-        $timeout, $priority, $trans_id, $pid) = @_;
+        $timeout, $priority, $trans_id, $source) = @_;
     if ($self->{count} == 4294967295)
     {
         $self->{count} = 0;
@@ -92,18 +93,18 @@ sub _request
     my $response_info = CloudI::API::info_key_value_new({});
     $self->{api}->return_($request_type, $name, $pattern,
                           $response_info, $response,
-                          $timeout, $trans_id, $pid);
+                          $timeout, $trans_id, $source);
 }
 
 {
     assert($use_threads);
     my $thread_count = CloudI::API::thread_count();
     my @threads = ();
-    for my $i (0 .. ($thread_count - 1))
+    for my $thread_index (0 .. ($thread_count - 1))
     {
         my $t = threads->create(sub
         {
-            my $task = CountTask->new(CloudI::API->new($i), $i);
+            my $task = CountTask->new($thread_index);
             return $task->run();
         });
         assert(defined($t));
