@@ -4,7 +4,7 @@
 #
 # MIT License
 #
-# Copyright (c) 2017-2020 Michael Truog <mjtruog at protonmail dot com>
+# Copyright (c) 2017-2022 Michael Truog <mjtruog at protonmail dot com>
 #
 # Permission is hereby granted, free of charge, to any person obtaining a
 # copy of this software and associated documentation files (the "Software"),
@@ -38,9 +38,10 @@ class Task(threading.Thread):
     """
     count thread task
     """
-    def __init__(self, api, name, exception):
+    def __init__(self, thread_index, name, exception):
         threading.Thread.__init__(self)
-        self.__api = api
+        self.__api = None
+        self.__thread_index = thread_index
         self.__name = name
         self.__terminate_exception = exception
         self.__count = 0
@@ -50,6 +51,7 @@ class Task(threading.Thread):
         run the count thread
         """
         try:
+            self.__api = API(self.__thread_index)
             self.__api.subscribe(self.__name + '/get', self.__request)
 
             result = self.__api.poll()
@@ -61,7 +63,7 @@ class Task(threading.Thread):
         print('terminate count %s' % self.__name)
 
     def __request(self, request_type, name, pattern, request_info, request,
-                  timeout, priority, trans_id, pid):
+                  timeout, priority, trans_id, source):
         # pylint: disable=unused-argument
         # pylint: disable=too-many-arguments
         if self.__count == 4294967295:
@@ -73,13 +75,13 @@ class Task(threading.Thread):
         response_info = API.info_key_value_new({})
         self.__api.return_(request_type, name, pattern,
                            response_info, response,
-                           timeout, trans_id, pid)
+                           timeout, trans_id, source)
 
 def _main():
     thread_count = API.thread_count()
     assert thread_count >= 1
 
-    threads = [Task(API(thread_index), 'python', TerminateException)
+    threads = [Task(thread_index, 'python', TerminateException)
                for thread_index in range(thread_count)]
     for thread in threads:
         thread.start()
