@@ -4,7 +4,7 @@
 //
 // MIT License
 //
-// Copyright (c) 2014-2020 Michael Truog <mjtruog at protonmail dot com>
+// Copyright (c) 2014-2022 Michael Truog <mjtruog at protonmail dot com>
 //
 // Permission is hereby granted, free of charge, to any person obtaining a
 // copy of this software and associated documentation files (the "Software"),
@@ -27,19 +27,22 @@
 
 require 'CloudI.php';
 
-class Task //extends \Thread
+class Task
 {
     private $api;
+    private $thread_index;
 
-    public function __construct($api)
+    public function __construct($thread_index)
     {
-        $this->api = $api;
+        $this->api = null;
+        $this->thread_index = $thread_index;
     }
 
     public function run()
     {
         try
         {
+            $this->api = new \CloudI\API($this->thread_index);
             assert($this->api->subscribe_count('php.xml/get') == 0);
             $this->api->subscribe('php.xml/get', $this, 'request');
             assert($this->api->subscribe_count('php.xml/get') == 1);
@@ -58,7 +61,7 @@ class Task //extends \Thread
 
     public function request($request_type, $name, $pattern,
                             $request_info, $request,
-                            $timeout, $priority, $trans_id, $pid)
+                            $timeout, $priority, $trans_id, $source)
     {
         $http_qs = \CloudI\API::info_key_value_parse($request);
         if (! isset($http_qs['value']))
@@ -80,30 +83,36 @@ class Task //extends \Thread
         ));
         $this->api->return_($request_type, $name, $pattern,
                             $response_info, $response,
-                            $timeout, $trans_id, $pid);
+                            $timeout, $trans_id, $source);
     }
 }
 
 $thread_count = \CloudI\API::thread_count();
 assert($thread_count == 1);
-$main_thread = new Task(new \CloudI\API(0));
+$main_thread = new Task(0);
 $main_thread->run();
 
 /*
-// commented out due to PHP threads not having
+// commented out due to PHP ZTS (Zend Thread Safety) not having
 // readily available installation packages
+assert(PHP_ZTS == 1);
+use parallel\Runtime;
+
 $thread_count = \CloudI\API::thread_count();
 assert($thread_count >= 1);
-    
+
 $threads = array();
-for ($i = 0; $i < $thread_count; $i++)
+for ($thread_index = 0; $thread_index < $thread_count; $thread_index++)
 {
-    $threads[$i] = new Task(new \CloudI\API($i));
+    $t = new Runtime();
+    $t->run(function () use ($thread_index) {
+        $task = new Task($thread_index);
+        $task->run();
+    });
+    $threads[$thread_index] = $t;
 }
 foreach ($threads as $t)
-    $t->start();
-foreach ($threads as $t)
-    $t->join();
+    $t->close();
 */
 
 ?>
