@@ -3,7 +3,7 @@
 #
 # MIT License
 #
-# Copyright (c) 2014-2020 Michael Truog <mjtruog at protonmail dot com>
+# Copyright (c) 2014-2022 Michael Truog <mjtruog at protonmail dot com>
 #
 # Permission is hereby granted, free of charge, to any person obtaining a
 # copy of this software and associated documentation files (the "Software"),
@@ -35,9 +35,9 @@ require CloudI::TerminateException;
 sub new
 {
     my $class = shift;
-    my ($api, $thread_index) = @_;
+    my ($thread_index) = @_;
     my $self = bless {
-        api => $api,
+        api => '',
         thread_index => $thread_index,
     }, $class;
     return $self;
@@ -48,6 +48,7 @@ sub run
     my $self = shift;
     eval
     {
+        $self->{api} = CloudI::API->new($self->{thread_index});
         $self->{api}->subscribe('a/b/c/d', $self, '_sequence1_abcd');
         $self->{api}->subscribe('a/b/c/*', $self, '_sequence1_abc_');
         $self->{api}->subscribe('a/b/*/d', $self, '_sequence1_ab_d');
@@ -106,128 +107,128 @@ sub _sequence1_abcd
 {
     my $self = shift;
     my ($request_type, $name, $pattern, $request_info, $request,
-        $timeout, $priority, $trans_id, $pid) = @_;
+        $timeout, $priority, $trans_id, $source) = @_;
     assert($pattern eq ($self->{api}->prefix() . 'a/b/c/d'));
     assert($request eq 'test1');
     $self->{api}->return_($request_type, $name, $pattern,
-                          '', $request, $timeout, $trans_id, $pid);
+                          '', $request, $timeout, $trans_id, $source);
 }
 
 sub _sequence1_abc_
 {
     my $self = shift;
     my ($request_type, $name, $pattern, $request_info, $request,
-        $timeout, $priority, $trans_id, $pid) = @_;
+        $timeout, $priority, $trans_id, $source) = @_;
     assert($pattern eq ($self->{api}->prefix() . 'a/b/c/*'));
     assert($request eq 'test2' || $request eq 'test3');
     $self->{api}->return_($request_type, $name, $pattern,
-                          '', $request, $timeout, $trans_id, $pid);
+                          '', $request, $timeout, $trans_id, $source);
 }
 
 sub _sequence1_ab_d
 {
     my $self = shift;
     my ($request_type, $name, $pattern, $request_info, $request,
-        $timeout, $priority, $trans_id, $pid) = @_;
+        $timeout, $priority, $trans_id, $source) = @_;
     assert($pattern eq ($self->{api}->prefix() . 'a/b/*/d'));
     assert($request eq 'test4' || $request eq 'test5');
     $self->{api}->return_($request_type, $name, $pattern,
-                          '', $request, $timeout, $trans_id, $pid);
+                          '', $request, $timeout, $trans_id, $source);
 }
 
 sub _sequence1_a_cd
 {
     my $self = shift;
     my ($request_type, $name, $pattern, $request_info, $request,
-        $timeout, $priority, $trans_id, $pid) = @_;
+        $timeout, $priority, $trans_id, $source) = @_;
     assert($pattern eq ($self->{api}->prefix() . 'a/*/c/d'));
     assert($request eq 'test6' || $request eq 'test7');
     $self->{api}->return_($request_type, $name, $pattern,
-                          '', $request, $timeout, $trans_id, $pid);
+                          '', $request, $timeout, $trans_id, $source);
 }
 
 sub _sequence1__bcd
 {
     my $self = shift;
     my ($request_type, $name, $pattern, $request_info, $request,
-        $timeout, $priority, $trans_id, $pid) = @_;
+        $timeout, $priority, $trans_id, $source) = @_;
     assert($pattern eq ($self->{api}->prefix() . '*/b/c/d'));
     assert($request eq 'test8' || $request eq 'test9');
     $self->{api}->return_($request_type, $name, $pattern,
-                          '', $request, $timeout, $trans_id, $pid);
+                          '', $request, $timeout, $trans_id, $source);
 }
 
 sub _sequence1_ab__
 {
     my $self = shift;
     my ($request_type, $name, $pattern, $request_info, $request,
-        $timeout, $priority, $trans_id, $pid) = @_;
+        $timeout, $priority, $trans_id, $source) = @_;
     assert($pattern eq ($self->{api}->prefix() . 'a/b/*'));
     assert($request eq 'test10');
     $self->{api}->return_($request_type, $name, $pattern,
-                          '', $request, $timeout, $trans_id, $pid);
+                          '', $request, $timeout, $trans_id, $source);
 }
 
 sub _sequence1_a__d
 {
     my $self = shift;
     my ($request_type, $name, $pattern, $request_info, $request,
-        $timeout, $priority, $trans_id, $pid) = @_;
+        $timeout, $priority, $trans_id, $source) = @_;
     assert($pattern eq ($self->{api}->prefix() . 'a/*/d'));
     assert($request eq 'test11');
     $self->{api}->return_($request_type, $name, $pattern,
-                          '', $request, $timeout, $trans_id, $pid);
+                          '', $request, $timeout, $trans_id, $source);
 }
 
 sub _sequence1___cd
 {
     my $self = shift;
     my ($request_type, $name, $pattern, $request_info, $request,
-        $timeout, $priority, $trans_id, $pid) = @_;
+        $timeout, $priority, $trans_id, $source) = @_;
     assert($pattern eq ($self->{api}->prefix() . '*/c/d'));
     assert($request eq 'test12');
     $self->{api}->return_($request_type, $name, $pattern,
-                          '', $request, $timeout, $trans_id, $pid);
+                          '', $request, $timeout, $trans_id, $source);
 }
 
 sub _sequence1_a___
 {
     my $self = shift;
     my ($request_type, $name, $pattern, $request_info, $request,
-        $timeout, $priority, $trans_id, $pid) = @_;
+        $timeout, $priority, $trans_id, $source) = @_;
     assert($pattern eq ($self->{api}->prefix() . 'a/*'));
     assert($request eq 'test13');
     $self->{api}->return_($request_type, $name, $pattern,
-                          '', $request, $timeout, $trans_id, $pid);
+                          '', $request, $timeout, $trans_id, $source);
 }
 
 sub _sequence1____d
 {
     my $self = shift;
     my ($request_type, $name, $pattern, $request_info, $request,
-        $timeout, $priority, $trans_id, $pid) = @_;
+        $timeout, $priority, $trans_id, $source) = @_;
     assert($pattern eq ($self->{api}->prefix() . '*/d'));
     assert($request eq 'test14');
     $self->{api}->return_($request_type, $name, $pattern,
-                          '', $request, $timeout, $trans_id, $pid);
+                          '', $request, $timeout, $trans_id, $source);
 }
 
 sub _sequence1_____
 {
     my $self = shift;
     my ($request_type, $name, $pattern, $request_info, $request,
-        $timeout, $priority, $trans_id, $pid) = @_;
+        $timeout, $priority, $trans_id, $source) = @_;
     assert($pattern eq ($self->{api}->prefix() . '*'));
     assert($request eq 'test15');
     $self->{api}->return_($request_type, $name, $pattern,
-                          '', $request, $timeout, $trans_id, $pid);
+                          '', $request, $timeout, $trans_id, $source);
 }
 
 sub _sequence1
 {
     my $self = shift;
     my ($request_type, $name, $pattern, $request_info, $request,
-        $timeout, $priority, $trans_id, $pid) = @_;
+        $timeout, $priority, $trans_id, $source) = @_;
     my @end = $self->{api}->recv_async(1000);
     while ($end[1] eq 'end')
     {
@@ -361,86 +362,86 @@ sub _sequence1
     # start sequence2
     $self->{api}->send_async($self->{api}->prefix() . 'sequence2', $request);
     $self->{api}->return_($request_type, $name, $pattern,
-                          '', 'end', $timeout, $trans_id, $pid);
+                          '', 'end', $timeout, $trans_id, $source);
 }
 
 sub _sequence2_e1
 {
     my $self = shift;
     my ($request_type, $name, $pattern, $request_info, $request,
-        $timeout, $priority, $trans_id, $pid) = @_;
+        $timeout, $priority, $trans_id, $source) = @_;
     $self->{api}->return_($request_type, $name, $pattern,
-                          '', '1', $timeout, $trans_id, $pid);
+                          '', '1', $timeout, $trans_id, $source);
 }
 
 sub _sequence2_e2
 {
     my $self = shift;
     my ($request_type, $name, $pattern, $request_info, $request,
-        $timeout, $priority, $trans_id, $pid) = @_;
+        $timeout, $priority, $trans_id, $source) = @_;
     $self->{api}->return_($request_type, $name, $pattern,
-                          '', '2', $timeout, $trans_id, $pid);
+                          '', '2', $timeout, $trans_id, $source);
 }
 
 sub _sequence2_e3
 {
     my $self = shift;
     my ($request_type, $name, $pattern, $request_info, $request,
-        $timeout, $priority, $trans_id, $pid) = @_;
+        $timeout, $priority, $trans_id, $source) = @_;
     $self->{api}->return_($request_type, $name, $pattern,
-                          '', '3', $timeout, $trans_id, $pid);
+                          '', '3', $timeout, $trans_id, $source);
 }
 
 sub _sequence2_e4
 {
     my $self = shift;
     my ($request_type, $name, $pattern, $request_info, $request,
-        $timeout, $priority, $trans_id, $pid) = @_;
+        $timeout, $priority, $trans_id, $source) = @_;
     $self->{api}->return_($request_type, $name, $pattern,
-                          '', '4', $timeout, $trans_id, $pid);
+                          '', '4', $timeout, $trans_id, $source);
 }
 
 sub _sequence2_e5
 {
     my $self = shift;
     my ($request_type, $name, $pattern, $request_info, $request,
-        $timeout, $priority, $trans_id, $pid) = @_;
+        $timeout, $priority, $trans_id, $source) = @_;
     $self->{api}->return_($request_type, $name, $pattern,
-                          '', '5', $timeout, $trans_id, $pid);
+                          '', '5', $timeout, $trans_id, $source);
 }
 
 sub _sequence2_e6
 {
     my $self = shift;
     my ($request_type, $name, $pattern, $request_info, $request,
-        $timeout, $priority, $trans_id, $pid) = @_;
+        $timeout, $priority, $trans_id, $source) = @_;
     $self->{api}->return_($request_type, $name, $pattern,
-                          '', '6', $timeout, $trans_id, $pid);
+                          '', '6', $timeout, $trans_id, $source);
 }
 
 sub _sequence2_e7
 {
     my $self = shift;
     my ($request_type, $name, $pattern, $request_info, $request,
-        $timeout, $priority, $trans_id, $pid) = @_;
+        $timeout, $priority, $trans_id, $source) = @_;
     $self->{api}->return_($request_type, $name, $pattern,
-                          '', '7', $timeout, $trans_id, $pid);
+                          '', '7', $timeout, $trans_id, $source);
 }
 
 sub _sequence2_e8
 {
     my $self = shift;
     my ($request_type, $name, $pattern, $request_info, $request,
-        $timeout, $priority, $trans_id, $pid) = @_;
+        $timeout, $priority, $trans_id, $source) = @_;
     $self->{api}->return_($request_type, $name, $pattern,
-                          '', '8', $timeout, $trans_id, $pid);
+                          '', '8', $timeout, $trans_id, $source);
 }
 
 sub _sequence2
 {
     my $self = shift;
     my ($request_type, $name, $pattern, $request_info, $request,
-        $timeout, $priority, $trans_id, $pid) = @_;
+        $timeout, $priority, $trans_id, $source) = @_;
     print "messaging sequence2 start perl ($request)\n";
     while (1)
     {
@@ -486,14 +487,14 @@ sub _sequence2
     # start sequence3
     $self->{api}->send_async($self->{api}->prefix() . 'sequence3', $request);
     $self->{api}->return_($request_type, $name, $pattern,
-                          '', 'end', $timeout, $trans_id, $pid);
+                          '', 'end', $timeout, $trans_id, $source);
 }
 
 sub _sequence3_f1
 {
     my $self = shift;
     my ($request_type, $name, $pattern, $request_info, $request_i,
-        $timeout, $priority, $trans_id, $pid) = @_;
+        $timeout, $priority, $trans_id, $source) = @_;
     if ($request_i == 4)
     {
         return 'done';
@@ -501,34 +502,34 @@ sub _sequence3_f1
     my $request_new = $request_i + 2; # two steps forward
     $self->{api}->forward_($request_type, $self->{api}->prefix() . 'f2',
                            $request_info, $request_new,
-                           $timeout, $priority, $trans_id, $pid);
+                           $timeout, $priority, $trans_id, $source);
 }
 
 sub _sequence3_f2
 {
     my $self = shift;
     my ($request_type, $name, $pattern, $request_info, $request_i,
-        $timeout, $priority, $trans_id, $pid) = @_;
+        $timeout, $priority, $trans_id, $source) = @_;
     my $request_new = $request_i - 1; # one step back
     $self->{api}->forward_($request_type, $self->{api}->prefix() . 'f1',
                            $request_info, $request_new,
-                           $timeout, $priority, $trans_id, $pid);
+                           $timeout, $priority, $trans_id, $source);
 }
 
 sub _sequence3_g1
 {
     my $self = shift;
     my ($request_type, $name, $pattern, $request_info, $request,
-        $timeout, $priority, $trans_id, $pid) = @_;
+        $timeout, $priority, $trans_id, $source) = @_;
     $self->{api}->return_($request_type, $name, $pattern,
-                          '', $request . 'suffix', $timeout, $trans_id, $pid);
+                          '', $request . 'suffix', $timeout, $trans_id, $source);
 }
 
 sub _sequence3
 {
     my $self = shift;
     my ($request_type, $name, $pattern, $request_info, $request,
-        $timeout, $priority, $trans_id, $pid) = @_;
+        $timeout, $priority, $trans_id, $source) = @_;
     print "messaging sequence3 start perl ($request)\n";
     my $test1_id = $self->{api}->send_async(
         $self->{api}->prefix() . 'f1',  '0');
@@ -553,18 +554,18 @@ sub _sequence3
     }
     $self->{api}->send_async($self->{api}->prefix() . 'sequence1', $iteration);
     $self->{api}->return_($request_type, $name, $pattern,
-                          '', 'end', $timeout, $trans_id, $pid);
+                          '', 'end', $timeout, $trans_id, $source);
 }
 
 {
     assert($use_threads);
     my $thread_count = CloudI::API::thread_count();
     my @threads = ();
-    for my $i (0 .. ($thread_count - 1))
+    for my $thread_index (0 .. ($thread_count - 1))
     {
         my $t = threads->create(sub
         {
-            my $task = MessagingTask->new(CloudI::API->new($i), $i);
+            my $task = MessagingTask->new($thread_index);
             return $task->run();
         });
         assert(defined($t));
