@@ -3,7 +3,7 @@
  *
  * MIT License
  *
- * Copyright (c) 2011-2017 Michael Truog <mjtruog at protonmail dot com>
+ * Copyright (c) 2011-2022 Michael Truog <mjtruog at protonmail dot com>
  *
  * Permission is hereby granted, free of charge, to any person obtaining a
  * copy of this software and associated documentation files (the "Software"),
@@ -44,15 +44,15 @@ static void request(int const request_type,
                     uint32_t timeout,
                     int8_t priority,
                     char const * const trans_id,
-                    char const * const pid,
-                    uint32_t const pid_size,
+                    char const * const source,
+                    uint32_t const source_size,
                     void * /*state*/,
                     cloudi_instance_t * api)
 {
     assert(request_size == MSG_SIZE);
     ::memcpy(buffer, request, request_size);
     unsigned int *i = reinterpret_cast<unsigned int *>(buffer);
-    if (*i == 4294967295U)
+    if (*i == 1073741823)
         *i = 0;
     else
         (*i)++;
@@ -61,7 +61,7 @@ static void request(int const request_type,
     cloudi_forward(api, request_type, DESTINATION,
                    request_info, request_info_size,
                    buffer, request_size,
-                   timeout, priority, trans_id, pid, pid_size);
+                   timeout, priority, trans_id, source, source_size);
 }
 
 int main(int, char **)
@@ -89,8 +89,7 @@ int main(int, char **)
     assert(result == cloudi_success);
 
     result = cloudi_poll(&api, -1);
-    if (result != cloudi_success &&
-        result != cloudi_terminate)
+    if (result != cloudi_success)
         std::cerr << "error " << result << std::endl;
     std::cout << "terminate msg_size c++" << std::endl;
 
