@@ -3,7 +3,7 @@
 //
 // MIT License
 //
-// Copyright (c) 2011-2018 Michael Truog <mjtruog at protonmail dot com>
+// Copyright (c) 2011-2022 Michael Truog <mjtruog at protonmail dot com>
 //
 // Permission is hereby granted, free of charge, to any person obtaining a
 // copy of this software and associated documentation files (the "Software"),
@@ -42,7 +42,7 @@ public class Task implements Runnable
                                      String name, String pattern,
                                      byte[] request_info, byte[] request,
                                      Integer timeout, Byte priority,
-                                     byte[] trans_id, OtpErlangPid pid)
+                                     byte[] trans_id, OtpErlangPid source)
                                      throws API.ForwardAsyncException,
                                             API.ForwardSyncException,
                                             API.InvalidInputException
@@ -50,7 +50,7 @@ public class Task implements Runnable
             ByteBuffer buffer = ByteBuffer.wrap(request);
             buffer.order(ByteOrder.nativeOrder());
             int i = buffer.getInt(0);
-            if (i == 4294967295L)
+            if (i == 1073741823)
                 i = 0;
             else
                 i++;
@@ -59,47 +59,33 @@ public class Task implements Runnable
                            i, MsgSize.DESTINATION, timeout);
             api.forward_(request_type, MsgSize.DESTINATION,
                          request_info, request,
-                         timeout, priority, trans_id, pid);
+                         timeout, priority, trans_id, source);
             return null;
         }
     }
     private API api;
-     
+    private final int thread_index;
+
     public Task(final int thread_index)
     {
-        try
-        {
-            this.api = new API(thread_index);
-        }
-        catch (API.InvalidInputException e)
-        {
-            e.printStackTrace(API.err);
-            System.exit(1);
-        }
-        catch (API.MessageDecodingException e)
-        {
-            e.printStackTrace(API.err);
-            System.exit(1);
-        }
-        catch (API.TerminateException e)
-        {
-            API.err.println("terminate msg_size java (before init)");
-            System.exit(1);
-        }
+        this.api = null;
+        this.thread_index = thread_index;
     }
 
     public void run()
     {
         try
         {
+            this.api = new API(thread_index);
+
             // possible with Java >= 8
             //this.api.subscribe("java", MsgSize::request);
 
             // required with Java < 8
             this.api.subscribe("java", MsgSize.class, "request");
 
-            Object result = this.api.poll();
-            assert result == Boolean.FALSE;
+            boolean result = this.api.poll();
+            assert result == false;
         }
         catch (API.TerminateException e)
         {
