@@ -4,7 +4,7 @@
 #
 # MIT License
 #
-# Copyright (c) 2013-2020 Michael Truog <mjtruog at protonmail dot com>
+# Copyright (c) 2013-2022 Michael Truog <mjtruog at protonmail dot com>
 #
 # Permission is hereby granted, free of charge, to any person obtaining a
 # copy of this software and associated documentation files (the "Software"),
@@ -36,12 +36,14 @@ import traceback
 from cloudi_c import API, TerminateException
 
 class _Task(threading.Thread):
-    def __init__(self, api):
+    def __init__(self, thread_index):
         threading.Thread.__init__(self)
-        self.__api = api
+        self.__api = None
+        self.__thread_index = thread_index
 
     def run(self):
         try:
+            self.__api = API(self.__thread_index)
             self.__api.subscribe('bounce/get', self.__request)
             self.__api.subscribe('bounce/delay', self.__delay)
             self.__api.subscribe('bounce/websocket/connect',
@@ -58,7 +60,7 @@ class _Task(threading.Thread):
         print('terminate websockets python_c')
 
     def __connect(self, request_type, name, pattern, request_info, request,
-                  timeout, priority, trans_id, pid):
+                  timeout, priority, trans_id, source):
         # pylint: disable=unused-argument
         # pylint: disable=too-many-arguments
         assert request == b'CONNECT'
@@ -67,7 +69,7 @@ class _Task(threading.Thread):
         return 'got connect! yay!'
 
     def __disconnect(self, request_type, name, pattern, request_info, request,
-                     timeout, priority, trans_id, pid):
+                     timeout, priority, trans_id, source):
         # pylint: disable=unused-argument
         # pylint: disable=too-many-arguments
         assert request == b'DISCONNECT'
@@ -76,7 +78,7 @@ class _Task(threading.Thread):
         return ''
 
     def __request(self, request_type, name, pattern, request_info, request,
-                  timeout, priority, trans_id, pid):
+                  timeout, priority, trans_id, source):
         # pylint: disable=unused-argument
         # pylint: disable=too-many-arguments
 
@@ -86,7 +88,7 @@ class _Task(threading.Thread):
         return request
 
     def __delay(self, request_type, name, pattern, request_info, request,
-                timeout, priority, trans_id, pid):
+                timeout, priority, trans_id, source):
         # pylint: disable=unused-argument
         # pylint: disable=too-many-arguments
         time.sleep(1.0)
@@ -105,7 +107,7 @@ def _main():
     thread_count = API.thread_count()
     assert thread_count >= 1
 
-    threads = [_Task(API(thread_index))
+    threads = [_Task(thread_index)
                for thread_index in range(thread_count)]
     for thread in threads:
         thread.start()
